package sra.isr.geometry.abstracts;

import sra.smalltalk.*;
import sra.isr.geometry.basic.*;

/**
 * IsrGeometry class
 * 
 * 	@author    nisinaka
 * 	@created   1998/09/29 (by nisinaka)
 * 	@updated   2000/02/16 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrGeometry.java,v 4.2 2002/07/11 09:25:03 m-asada Exp $
 */
public abstract class IsrGeometry extends StObject {
	public final static double ACCURACY = 1.0e-12d;
	/**
	 * Answer the accuracy of geometries.
	 *
	 * @return double
	 */
	public final static double Accuracy() {
		return ACCURACY;
	}
	/**
	 * Convert to a glocal geometry.
	 *
	 * @return sra.isr.geometry.abstracts.IsrGeometry
	 */
	public IsrGeometry asGlobalGeometry() {
		return this;
	}
	/**
	 * Answer the distance from specified Point.
	 *
	 * @return double
	 * @param aPoint sra.isr.geometry.basic.Isr2dPoint
	 */
	public double distanceFromPoint_(Isr2dPoint aPoint) {
		throw SmalltalkException.SubclassResponsibility();
	}
	/**
	 * Answer true if the receiver is a 2d geometry element.
	 *
	 * @return boolean
	 */
	public boolean is2d() {
		return false;
	}
	/**
	 * Answer true if the receiver is a 3d geometry element.
	 *
	 * @return boolean
	 */
	public boolean is3d() {
		return false;
	}
	/**
	 * Answer true if the receiver is a curve.
	 *
	 * @return boolean
	 */
	public boolean isCurve() {
		return false;
	}
	/**
	 * Answer true if the receiver is a geometry.
	 *
	 * @return boolean
	 */
	public boolean isGeometry() {
		return true;
	}
	/**
	 * Answer true if the receiver is a global geometry.
	 *
	 * @return boolean
	 */
	public boolean isGlocalGeometry() {
		return true;
	}
	/**
	 * Answer true if the receiver is a surface.
	 *
	 * @return boolean
	 */
	public boolean isSurface() {
		return false;
	}
	/**
	 * Answer true if the receiver is a topology.
	 *
	 * @return boolean
	 */
	public boolean isTopology() {
		return false;
	}
}
