package sra.isr.image.browser;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;
import sra.smalltalk.*;
import sra.isr.image.objects.*;
import sra.isr.image.streams.*;
import sra.isr.interfaces.display.*;
import sra.isr.interfaces.progress.*;
import sra.isr.system.support.*;

/**
 * IsrImageBrowser class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/01/17 (by Mitsuhiro Asada)
 * 	@updated   2000/02/21 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageBrowser.java,v 4.2 2002/07/11 09:25:03 m-asada Exp $
 */
public class IsrImageBrowser extends StApplicationModel {
	protected IsrImageObject[] imageObjects;
	protected Vector selectedIndexes;
	protected boolean indexFlag;
	protected boolean slaveFlag;
	protected IsrDisplayModel displayModelForOriginal;
	protected IsrDisplayModel displayModelForAttributes;
	/**
	 * Add selected index.
	 * 
	 * @param anInteger int
	 */
	public void addSelectedIndex_(int anInteger) {
		Integer number = new Integer(anInteger);
		if (!this.selectedIndexes().contains(number)) {
			this.selectedIndexes.addElement(number);
		}
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * (Smalltalk version only)
	 * 
	 * @return java.lang.Object
	 * @param asserBlock sra.smalltalk.StBlockClosure
	 * @param doBlock sra.smalltalk.StBlockClosure
	 * @param ensureBlock sra.smalltalk.StBlockClosure
	 */
	private Object assert_do_ensure_(StBlockClosure asserBlock, StBlockClosure doBlock, StBlockClosure ensureBlock) {
		/* asserBlock.value();
		Object result = null;
		try {
			result = doBlock.value();
		} finally {
			ensureBlock.value();
		}
		return result; */
		return null;
	}
	/**
	 * Clean up indexes.
	 * 
	 * @return java.uitl.Vector
	 */
	public Vector clearSelectedIndexes() {
		selectedIndexes = new Vector();
		return selectedIndexes;
	}
	/**
	 * Create a default popup menu of the Isr9ColorsModel.
	 *
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuModel defaultMenu() {
		StMenuModel rootMenu = new StMenuModel();
		//
		StMenu editMenu = new StMenu("ImageBrowserMenu");
		editMenu.add(this.createMenuItem("open", "openImageBrowser"));
		editMenu.addSeparator();
		editMenu.add(this.createMenuItem("original", "showOriginalImages"));
		editMenu.add(this.createMenuItem("attributes", "showImageAttributes"));
		editMenu.addSeparator();
		editMenu.add(this.createMenuItem("select all", "selectAll"));
		editMenu.add(this.createMenuItem("select indexes", "selectIndexes"));
		editMenu.add(this.createMenuItem("index on/off", "indexOnOff"));
		editMenu.addSeparator();
		editMenu.add(this.createMenuItem("save selections", "saveSelections"));
		editMenu.add(this.createMenuItem("html selections", "htmlSelections"));
		rootMenu.add(editMenu);
		//
		return rootMenu;
	}
	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new IsrImageBrowserApplicationViewAwt();
		} else {
			return new IsrImageBrowserApplicationViewSwing();
		}
	}
	/**
	 * Construsctor with directory name.
	 * 
	 * @return sra.isr.image.browser.IsrImageBrowser
	 * @param aString java.lang.String
	 */
	public static IsrImageBrowser DirectoryName_(String aString) {
		return ImageFolder_(IsrImageFolder.DirectoryName_(aString));
	}
	/**
	 * Answer the directory of save.
	 * 
	 * @return java.io.File
	 */
	protected File directoryToSave() {
		String string = StDialog.Request_("Enter directory name to save selections, then accept or CR", "");
		if (string == null || string.length() == 0) {
			return null;
		}
		File filename = new File(string);
		if (filename.exists() && filename.isDirectory() == false) {
			StDialog.Warn_(string + " is not directory");
			return null;
		}
		if (filename.exists()) {
			if (StDialog.Confirm_(string + " already exists. Do you want to go head?") == false) {
				return null;
			}
		}
		if (File.separator == ":" && filename.isAbsolute() == false && filename.toString().startsWith(File.separator) == false) {
			string = File.separator + string;
			filename = new File(string);
		}
		if (filename.toString().endsWith(File.separator) == false) {
			string = filename.toString() + File.separator;
			filename = new File(string);
		}
		return filename;
	}
	/**
	 * Answer my display model for attributes.
	 * 
	 * @return sra.isr.goodies.display.IsrDisplayModel
	 */
	public IsrDisplayModel displayModelForAttributes() {
		if (displayModelForAttributes == null) {
			displayModelForAttributes = IsrDisplayModel.Visual_("");
		}
		return displayModelForAttributes;
	}
	/**
	 * Answer my display model for original.
	 * 
	 * @return sra.isr.goodies.display.IsrDisplayModel
	 */
	public IsrDisplayModel displayModelForOriginal() {
		if (displayModelForOriginal == null) {
			displayModelForOriginal = IsrDisplayModel.Visual_("");
		}
		return displayModelForOriginal;
	}
	/**
	 * Create new instance with user parameter.
	 * 
	 * @return sra.isr.image.browser.IsrImageBrowser or null
	 */
	public static IsrImageBrowser FromUser() {
		IsrImageFolder imageFolder = IsrImageFolder.FromUser();
		if (imageFolder == null) {
			return null;
		}
		IsrImageBrowser imageBrowser = ImageFolder_(imageFolder);
		return imageBrowser;
	}
	/**
	 * Write html with selection images.
	 */
	public void htmlSelections() {
		final IsrImageObject[] images = this.selectedImages();
		if (images == null) {
			return;
		}
		final File directory = this.directoryToSave();
		if (directory == null) {
			return;
		}
		if (!directory.exists()) {
			directory.mkdirs();
		}
		//
		final String bigs = "bigs";
		final String smalls = "smalls";
		final String html = "index.htm";
		StBlockClosure block = new StBlockClosure() {
			public Object value_(Object sub) {
				String string = directory.toString();
				if (string.endsWith(File.separator) == false) {
					string = string + File.separator;
				}
				File filename = new File(string, (String) sub);
				if (filename.exists()) {
					if (filename.isDirectory() == false) {
						filename.delete();
					}
				} else {
					filename.mkdir();
				}
				if (filename.toString().endsWith(File.separator) == false) {
					string = filename.toString() + File.separator;
					filename = new File(string);
				}
				return filename;
			}
		};
		final File dirB = (File) block.value_(bigs);
		final File dirS = (File) block.value_(smalls);
		final int size = images.length;
		new IsrProgress().do_(new StBlockClosure() {
			public Object value_(Object obj) {
				IsrProgress progress = (IsrProgress) obj;
				int count = 1;
				progress.message_("html selections ...");
				Vector pairs = new Vector(size);
				for (int i = 0; i < size; i++) {
					IsrImageObject object = images[i];
					float value = ((float) count) / size;
					progress.value_(value);
					File source = object.asFilename();
					String string = dirB.toString();
					if (string.endsWith(File.separator) == false) {
						string = string + File.separator;
					}
					string = string + object.fileName();
					File destination = new File(string);
					InputStream in = null;
					OutputStream out = null;
					try {
						in = new BufferedInputStream(new FileInputStream(source));
						out = new BufferedOutputStream(new FileOutputStream(destination));
						int b;
						while ((b = in.read()) != -1) {
							out.write(b);
						}
					} catch (IOException e) {
						System.out.println(e.getMessage());
						e.printStackTrace();
					} finally {
						try {
							if (in != null) {
								in.close();
								in = null;
							}
							if (out != null) {
								out.flush();
								out.close();
								out = null;
							}
						} catch (IOException e) {
							System.out.println(e.getMessage());
							e.printStackTrace();
						}
					}
					//
					String[] pair = new String[2];
					pair[0] = destination.getName();
					StImage image = object.iconImage();
					string = dirS.toString();
					if (string.endsWith(File.separator) == false) {
						string = string + File.separator;
					}
					string = string + object.fileName();
					File filename = new File(string);
					//
					IsrImageStream imageStream = null;
					try {
						string = filename.toString().toLowerCase();
						if (string.endsWith(".jpg") || string.endsWith(".jpeg")) {
							imageStream = IsrJpegImageStream.On_(new FileOutputStream(filename));
						}
						if (imageStream == null) {
							throw new IOException("unknown image format");
						}
						imageStream.nextPutImage_(image);
					} catch (IOException e) {
						System.out.println(e.getMessage());
						e.printStackTrace();
					} finally {
						try {
							if (imageStream != null) {
								imageStream.flush();
								imageStream.close();
								imageStream = null;
							}
						} catch (IOException e) {
							System.out.println(e.getMessage());
							e.printStackTrace();
						}
					}
					//
					pair[1] = filename.getName();
					pairs.addElement(pair);
					count = count + 1;
				}
				String string = directory.toString();
				if (string.endsWith(File.separator) == false) {
					string = string + File.separator;
				}
				string = string + html;
				File filename = new File(string);
				float value = ((float) count) / size;
				progress.value_(value);
				//
				BufferedWriter bw = null;
				StringWriter sw = null;
				PrintWriter pw = null;
				try {
					sw = new StringWriter();
					pw = new PrintWriter(sw);
					pw.println("<HTML>");
					pw.println("<HEAD>");
					pw.print("<TITLE>Images (");
					SimpleDateFormat formatter = new SimpleDateFormat("yyyy/MM/dd, h:mm:ss");
					pw.print(formatter.format(new Date()));
					pw.println(")</TITLE>");
					pw.println("</HEAD>");
					pw.println("<BODY BGCOLOR=\"#FFFFFF\">");
					pw.println("");
					for (int i = 0; i < size; i++) {
						IsrImageObject object = images[i];
						String[] each = (String[]) pairs.elementAt(i);
						pw.print("<A HREF=\"" + bigs + "/" + each[0] + "\" TARGET=\"_blank\">");
						pw.print("<IMG SRC=\"" + smalls + "/" + each[1] + "\" BORDER=0");
						pw.print(" WIDTH=" + object.iconImage().bounds().width);
						pw.print(" HEIGHT=" + object.iconImage().bounds().height);
						pw.print(" HSPACE=2 VSPACE=2");
						pw.println("></A>");
					}
					pw.println("");
					pw.println("</BODY>");
					pw.println("</HTML>");
					//
					pw.flush();
					bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filename)));
					bw.write(sw.toString());
				} catch (IOException e) {
					System.out.println(e.getMessage());
					e.printStackTrace();
				} finally {
					try {
						if (bw != null) {
							bw.flush();
							bw.close();
							bw = null;
						}
						if (pw != null) {
							pw.close();
							pw = null;
						}
						if (sw != null) {
							sw.close();
							sw = null;
						}
					} catch (IOException e) {
						System.out.println(e.getMessage());
						e.printStackTrace();
					}
				}
				return null;
			}
		});
		return;
	}
	/**
	 * Construsctor with imageFolder
	 * 
	 * @return sra.isr.image.browser.IsrImageBrowser
	 * @param imageFolder sra.isr.image.objects.IsrImageFolder
	 */
	public static IsrImageBrowser ImageFolder_(IsrImageFolder imageFolder) {
		return ImageObjects_(imageFolder.imageObjects());
	}
	/**
	 * Answer my image objects.
	 * 
	 * @return sra.isr.image.objects.IsrImageObject[]
	 */
	public IsrImageObject[] imageObjects() {
		return imageObjects;
	}
	/**
	 * Construsctor with imageObjects
	 * 
	 * @return sra.isr.image.browser.IsrImageBrowser
	 * @param collectionOfImageObjects sra.isr.image.objects.IsrImageObject[]
	 */
	public static IsrImageBrowser ImageObjects_(IsrImageObject[] collectionOfImageObjects) {
		IsrImageBrowser model = new IsrImageBrowser();
		model.imageObjects_(collectionOfImageObjects);
		return model;
	}
	/**
	 * Set my image objects.
	 * 
	 * @param collectionOfImageObjects sra.isr.image.objects.IsrImageObject[]
	 */
	public void imageObjects_(IsrImageObject[] collectionOfImageObjects) {
		if (imageObjects != collectionOfImageObjects) {
			this.setImageObjects_(collectionOfImageObjects);
			this.clearSelectedIndexes();
			this.changed_($("imageObjects"));
		}
	}
	/**
	 * Answer my index flag.
	 * 
	 * @return boolean
	 */
	public boolean indexFlag() {
		return (indexFlag == true);
	}
	/**
	 * Set my index flag.
	 * 
	 * @param aBoolean boolean
	 */
	public void indexFlag_(boolean aBoolean) {
		indexFlag = (aBoolean == true);
	}
	/**
	 * Set image index on/off.
	 */
	public void indexOnOff() {
		this.indexFlag_(!this.indexFlag());
		this.changed();
	}
	/**
	 * Open new image browser.
	 */
	public void openImageBrowser() {
		IsrImageBrowser imageBrowser = this.FromUser();
		if (imageBrowser == null) {
			return;
		}
		imageBrowser.open();
	}
	/**
	 * Remove selected index from selectedIndexes.
	 * 
	 * @param anInteger int
	 */
	public void removeSelectedIndex_(int anInteger) {
		Integer number = new Integer(anInteger);
		if (this.selectedIndexes().contains(number)) {
			this.selectedIndexes.removeElement(number);
		}
	}
	/**
	 * Save selections.
	 *
	 * @return isr.image.object.IsrImageObject[]
	 */
	public IsrImageObject[] saveSelections() {
		final IsrImageObject[] images = this.selectedImages();
		if (images == null) {
			return null;
		}
		final File directory = this.directoryToSave();
		if (directory == null) {
			return null;
		}
		if (!directory.exists()) {
			directory.mkdirs();
		}
		//
		final int size = images.length;
		new IsrProgress().do_(new StBlockClosure() {
			public Object value_(Object obj) {
				IsrProgress progress = (IsrProgress) obj;
				int count = 1;
				progress.message_("save selections ...");
				for (int i = 0; i < size; i++) {
					IsrImageObject object = images[i];
					float value = ((float) count) / size;
					progress.value_(value);
					File source = object.asFilename();
					String string = directory.toString();
					if (string.endsWith(File.separator) == false) {
						string = string + File.separator;
					}
					string = string + object.fileName();
					File destination = new File(string);
					InputStream in = null;
					OutputStream out = null;
					try {
						in = new BufferedInputStream(new FileInputStream(source));
						out = new BufferedOutputStream(new FileOutputStream(destination));
						int b;
						while ((b = in.read()) != -1) {
							out.write(b);
						}
					} catch (IOException e) {
						System.out.println(e.getMessage());
						e.printStackTrace();
					} finally {
						try {
							if (in != null) {
								in.close();
								in = null;
							}
							if (out != null) {
								out.flush();
								out.close();
								out = null;
							}
						} catch (IOException e) {
							System.out.println(e.getMessage());
							e.printStackTrace();
						}
					}
					count = count + 1;
				}
				return null;
			}
		});
		//
		IsrImageFolder folder = IsrImageFolder.DirectoryName_(directory.toString());
		return folder.imageObjects();
	}
	/**
	 * Select all images.
	 */
	public void selectAll() {
		Vector oldIndexes = (Vector) this.selectedIndexes.clone();
		this.selectedIndexes_(new Vector());
		for (int index = 0; index < this.imageObjects().length; index++) {
			this.selectedIndexes().addElement(new Integer(index));
		}
		Vector newIndexes = this.selectedIndexes();
		Vector xor = this.xorIndexes_with_(newIndexes, oldIndexes);
		this.changed_with_($("selectedIndexes"), xor);
	}
	/**
	 * Answer my selected imageObjects.
	 * 
	 * @return sra.isr.image.objects.IsrImageObject[]
	 */
	public IsrImageObject[] selectedImages() {
		Vector selectedImages = new Vector();
		if (this.selectedIndexes().size() == 0) {
			return new IsrImageObject[0];
		}
		StBlockClosure sortBlock = new StBlockClosure() {
			public Object value_value_(Object obj1, Object obj2) {
				return new Boolean(((Integer) obj1).intValue() <= ((Integer) obj2).intValue());
			}
		};
		StSortedCollection sortedCollection = new StSortedCollection(sortBlock, this.selectedIndexes());
		Object[] collection = sortedCollection._asArray();
		for (int i = 0; i < collection.length; i++) {
			selectedImages.addElement(this.imageObjects()[ ((Integer) collection[i]).intValue()]);
		}
		IsrImageObject[] results = new IsrImageObject[selectedImages.size()];
		selectedImages.copyInto((Object[]) results);
		return results;
	}
	/**
	 * Answer seleted indexes.
	 * 
	 * @return java.util.Vector
	 */
	public Vector selectedIndexes() {
		if (selectedIndexes == null) {
			this.clearSelectedIndexes();
		}
		return selectedIndexes;
	}
	/**
	 * Set seleted indexes.
	 * 
	 * @param indexes java.util.Vector
	 */
	public void selectedIndexes_(Vector indexes) {
		selectedIndexes = indexes;
	}
	/**
	 * Select all images.
	 */
	public void selectIndexes() {
		String string = StDialog.Request_("Enter indexes, then accept. Wild cards (*?[-]) are allowrd", "");
		if (string == null || string.length() == 0) {
			return;
		}
		Vector tokens = IsrImageFolder.Separate_dividers_(string, IsrImageFolder.Dividers());
		Vector oldIndexes = (Vector) this.selectedIndexes.clone();
		Vector indexes = new Vector();
		for (int index = 0; index < this.imageObjects().length; index++) {
			Enumeration enum = tokens.elements();
			while (enum.hasMoreElements()) {
				String token = (String) enum.nextElement();
				if (IsrImageFolder.StringMatch_and_(String.valueOf(index), token)) {
					indexes.addElement(new Integer(index));
				}
			}
		}
		this.selectedIndexes_(indexes);
		Vector newIndexes = this.selectedIndexes();
		Vector xor = this.xorIndexes_with_(newIndexes, oldIndexes);
		this.changed_with_($("selectedIndexes"), xor);
	}
	/**
	 * Set my image objects.
	 * 
	 * @param collectionOfImageObjects sra.isr.image.objects.IsrImageObject[]
	 */
	protected void setImageObjects_(IsrImageObject[] collectionOfImageObjects) {
		imageObjects = collectionOfImageObjects;
	}
	/**
	 * Open selected image objects's attributes image.
	 */
	public void showImageAttributes() {
		Frame window = null;
		IsrImageObject[] images = this.selectedImages();
		if (images.length == 0) {
			return;
		}
		IsrImageObject first = images[0];
		IsrDisplayModel model = this.displayModelForAttributes();
		String label = first.fileName().toString();
		Frame[] frames = model.builder().frames();
		for (int i = 0; i < frames.length; i++) {
			model.visual_(first.imageAttributes().image());
			window = frames[i];
			window.setTitle(label);
			if (window.isShowing()) {
				window.toFront();
			} else {
				window.show();
			}
		}
		if (frames.length == 0) {
			model.visual_(first.imageAttributes().image());
			window = model.open();
			window.setTitle(label);
		}
		if (images.length > 1) {
			for (int i = 1; i < images.length; i++) {
				IsrImageObject each = images[i];
				label = each.fileName().toString();
				model.visual_(each.imageAttributes().image());
				window.setTitle(label);
			}
		}
	}
	/**
	 * Open selected image objects's attributes image.
	 */
	public void showNewImageAttributes() {
		IsrImageObject[] images = this.selectedImages();
		if (images.length == 0) {
			return;
		}
		for (int i = 0; i < images.length; i++) {
			IsrImageObject each = images[i];
			String label = each.fileName().toString();
			IsrDisplayModel.OpenVisual_label_(each.imageAttributes().image(), label);
		}
	}
	/**
	 * Open selected image objects's original image.
	 */
	public void showNewOriginalImages() {
		IsrImageObject[] images = this.selectedImages();
		if (images.length == 0) {
			return;
		}
		for (int i = 0; i < images.length; i++) {
			IsrImageObject each = images[i];
			String label = each.fileName().toString();
			IsrDisplayModel.OpenVisual_label_(each.originalImage().image(), label);
		}
	}
	/**
	 * Open selected image objects's original image.
	 */
	public void showOriginalImages() {
		Frame window = null;
		IsrImageObject[] images = this.selectedImages();
		if (images.length == 0) {
			return;
		}
		IsrImageObject first = images[0];
		IsrDisplayModel model = this.displayModelForOriginal();
		String label = first.fileName().toString();
		Frame[] frames = model.builder().frames();
		for (int i = 0; i < frames.length; i++) {
			model.visual_(first.originalImage().image());
			window = frames[i];
			window.setTitle(label);
			if (window.isShowing()) {
				window.toFront();
			} else {
				window.show();
			}
		}
		if (frames.length == 0) {
			model.visual_(first.originalImage().image());
			window = model.open();
			window.setTitle(label);
		}
		if (images.length > 1) {
			for (int i = 1; i < images.length; i++) {
				IsrImageObject each = images[i];
				label = each.fileName().toString();
				model.visual_(each.originalImage().image());
				window.setTitle(label);
			}
		}
	}
	/**
	 * Answer my slave flag.
	 * 
	 * @return boolean
	 */
	public boolean slaveFlag() {
		return (slaveFlag == true);
	}
	/**
	 * Set my slave flag.
	 * 
	 * @param aBoolean boolean
	 */
	public void slaveFlag_(boolean aBoolean) {
		slaveFlag = (aBoolean == true);
	}
	/**
	 * Invoked when a window is in the process of being closed.
	 * The close operation can be overridden at this point.
	 *
	 * @param e java.awt.event.WindowEvent
	 */
	public void windowClosing(WindowEvent e) {
		if (this.slaveFlag()) {
			StApplicationModel[] models = new StApplicationModel[2];
			models[0] = this.displayModelForOriginal();
			models[1] = this.displayModelForAttributes();
			for (int i = 0; i < models.length; i++) {
				StApplicationModel model = models[i];
				Frame[] frames = model.builder().frames();
				for (int j = 0; j < frames.length; j++) {
					frames[j].setVisible(false);
				}
			}
			super.windowClosing(e);
		} else {
			Window aWindow = e.getWindow();
			Point location = aWindow.getLocationOnScreen();
			Dimension size = aWindow.getSize();
			location = new Point(location.x + size.width, location.y);
			if (StDialog.Confirm_("Really close?", false, location) == true) {
				StApplicationModel[] models = new StApplicationModel[2];
				models[0] = this.displayModelForOriginal();
				models[1] = this.displayModelForAttributes();
				for (int i = 0; i < models.length; i++) {
					StApplicationModel model = models[i];
					Frame[] frames = model.builder().frames();
					for (int j = 0; j < frames.length; j++) {
						frames[j].setVisible(false);
					}
				}
				super.windowClosing(e);
			}
		}
	}
	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Image Browser";
	}
	/**
	 * Set seleted indexes.
	 * 
	 * @return java.util.Vector
	 * @param newCollection java.util.Vector
	 * @param oldCollection java.util.Vector
	 */
	public Vector xorIndexes_with_(Vector newCollection, Vector oldCollection) {
		Integer each;
		Vector intersect = new Vector();
		Enumeration enum = oldCollection.elements();
		while (enum.hasMoreElements()) {
			each = (Integer) enum.nextElement();
			if (newCollection.contains(each)) {
				intersect.addElement(each);
			}
		}
		enum = newCollection.elements();
		while (enum.hasMoreElements()) {
			each = (Integer) enum.nextElement();
			oldCollection.addElement(each);
		}
		Vector xor = new Vector();
		enum = oldCollection.elements();
		while (enum.hasMoreElements()) {
			each = (Integer) enum.nextElement();
			if (!intersect.contains(each)) {
				xor.addElement(each);
			}
		}
		return xor;
	}
}
