package sra.isr.image.browser;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import sra.smalltalk.*;
import sra.isr.system.support.*;

/**
 * IsrImageBrowserController class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/01/17 (by Mitsuhiro Asada)
 * 	@updated   2000/02/21 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageBrowserController.java,v 4.2 2002/07/11 09:25:03 m-asada Exp $
 */
public class IsrImageBrowserController extends StController implements MouseListener {
	/**
	 * Add listeners.
	 * 
	 * @param newView sra.smalltalk.StView
	 */
	protected void buildListener(StView newView) {
		Component aView = (Component) newView;
		aView.addMouseListener(this);
	}
	/**
	 * Answer a model.
	 *
	 * @return sra.isr.image.browser.IsrImageBrowser
	 */
	protected final IsrImageBrowser getModel() {
		return (IsrImageBrowser) this.model();
	}
	/**
	 * Answer this view.
	 *
	 * @return sra.isr.image.browser.IsrImageBrowserView interface
	 */
	protected IsrImageBrowserView getView() {
		return (IsrImageBrowserView) this.view();
	}
	/**
	 * Invoked when the mouse has been clicked on a component.
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseClicked(MouseEvent event) {
		if (this.isMenuButtonPressed_(event)) {
			return;
		}
		if (event.getClickCount() == 1) {
			this.selectActivity_(event);
		} else
			if (event.getClickCount() == 2) {
				this.mouseDoubleClicked(event);
			}
	}
	/**
	 * Invoked when a mouse button has been pressed. 
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseDoubleClicked(MouseEvent event) {
		if (event.isControlDown()) {
			if (event.isShiftDown()) {
				this.getModel().showNewImageAttributes();
			} else {
				this.getModel().showImageAttributes();
			}
		} else {
			if (event.isShiftDown()) {
				this.getModel().showNewOriginalImages();
			} else {
				this.getModel().showOriginalImages();
			}
		}
	}
	/**
	 * Invoked when a mouse button has been pressed. 
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent event) {
		// popup menu
		if (event.isMetaDown()) {
			IsrImageBrowserView view = this.getView();
			view.showPopupMenu(event.getPoint());
		}
	}
	/**
	 * Select image object. 
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void selectActivity_(MouseEvent event) {
		if (this.isMenuButtonPressed_(event)) {
			return;
		}
		IsrImageBrowserView view = this.getView();
		Point point = event.getPoint();
		int index = view.whichIndex_(point);
		if (index == -1) {
			return;
		}
		IsrImageBrowser browser = this.getModel();
		Vector oldSelection = (Vector) browser.selectedIndexes().clone();
		boolean bool = event.isShiftDown();
		if (bool) {
			if (browser.selectedIndexes().contains(new Integer(index))) {
				browser.removeSelectedIndex_(index);
			} else {
				browser.addSelectedIndex_(index);
			}
		} else {
			if (!browser.selectedIndexes().contains(new Integer(index))) {
				browser.clearSelectedIndexes();
				browser.addSelectedIndex_(index);
			}
		}
		Vector newSelection = browser.selectedIndexes();
		Vector xor = browser.xorIndexes_with_(newSelection, oldSelection);
		browser.changed_with_($("selectedIndexes"), xor);
	}
}
