package sra.isr.image.database;

import java.awt.*;
import java.io.*;
import java.sql.*;
import java.util.*;
import oracle.sql.*;
import oracle.jdbc.driver.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.image.objects.*;

/**
 * IsrImageDatabaseForOracle class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/12/28 (by Mitsuhiro Asada)
 * 	@updated   2001/07/10 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageDatabaseForOracle.java,v 4.2 2002/07/11 09:25:03 m-asada Exp $
 */
public class IsrImageDatabaseForOracle extends IsrImageDatabase {
	/**
	 * Get the original image information with connection and primary key on database.
	 *
	 * @return sra.isr.image.objects.IsrImageFileInfo
	 * @param connection java.sql.Connection
	 * @param pkey int
	 * @exception java.sql.SQLException
	 */
	protected IsrImageFileInfo getOriginalImageInfo(Connection connection, int pkey) throws SQLException {
		IsrImageFileInfo imageInfo = new IsrImageFileInfo();
		String sql = null;
		PreparedStatement pst = null;
		ResultSet rs = null;

		try {
			sql = "SELECT original_image, original_image_type FROM image_searcher WHERE pkey = ?";
			pst = connection.prepareStatement(sql);
			pst.clearParameters();
			pst.setInt(1, pkey);
			rs = pst.executeQuery();
			if (rs != null) {
				if (rs.next()) {
					imageInfo.setBytes(this.readOnBlob(((OracleResultSet) rs).getBLOB(1)));
					imageInfo.setContentType(rs.getString(2));
				}
				rs.close();
				rs = null;
			}
			pst.close();
			pst = null;
		} catch (IOException e) {
			throw new SQLException(e.getMessage());
		} finally {
			if (rs != null) {
				rs.close();
				rs = null;
			}
			if (pst != null) {
				pst.close();
				pst = null;
			}
		}

		return imageInfo;
	}
	/**
	 * Get the thumbnail image information with connection and primary key on database.
	 *
	 * @return sra.isr.image.objects.IsrImageFileInfo
	 * @param connection java.sql.Connection
	 * @param pkey int
	 * @exception java.sql.SQLException
	 */
	protected IsrImageFileInfo getThumbnailImageInfo(Connection connection, int pkey) throws SQLException {
		IsrImageFileInfo imageInfo = new IsrImageFileInfo();
		String sql = null;
		PreparedStatement pst = null;
		ResultSet rs = null;

		try {
			sql = "SELECT icon_image, icon_image_type FROM image_searcher WHERE pkey = ?";
			pst = connection.prepareStatement(sql);
			pst.clearParameters();
			pst.setInt(1, pkey);
			rs = pst.executeQuery();
			if (rs != null) {
				if (rs.next()) {
					imageInfo.setBytes(this.readOnBlob(((OracleResultSet) rs).getBLOB(1)));
					imageInfo.setContentType(rs.getString(2));
				}
				rs.close();
				rs = null;
			}
			pst.close();
			pst = null;
		} catch (IOException e) {
			throw new SQLException(e.getMessage());
		} finally {
			if (rs != null) {
				rs.close();
				rs = null;
			}
			if (pst != null) {
				pst.close();
				pst = null;
			}
		}

		return imageInfo;
	}
	/**
	 * Get the next value of image searcher primary key.
	 *
	 * @return int
	 * @param connection java.sql.Connection
	 * @exception java.sql.SQLException
	 */
	protected int nextImageSearcherPkey(Connection connection) throws SQLException {
		int pkey = -1;
		String sql = null;
		PreparedStatement pst = null;
		ResultSet rs = null;

		try {
			sql = "SELECT image_searcher_pkey_seq.nextval FROM dual";
			pst = connection.prepareStatement(sql);
			pst.clearParameters();
			rs = pst.executeQuery();
			if (rs != null) {
				if (rs.next()) {
					pkey = rs.getInt(1);
				} else {
					throw new SQLException("Don't get value of primary key on image searcher");
				}
				rs.close();
				rs = null;
			} else {
				throw new SQLException("Don't get value of primary key on image searcher");
			}
			pst.close();
			pst = null;
		} finally {
			if (rs != null) {
				rs.close();
				rs = null;
			}
			if (pst != null) {
				pst.close();
				pst = null;
			}
		}
		return pkey;
	}
	/**
	 * Read the byte array to the BLOB on database.
	 *
	 * @return bytes byte[]
	 * @param blob oracle.sql.BLOB
	 * @exception java.io.IOException
	 * @exception java.sql.SQLException
	 */
	private byte[] readOnBlob(BLOB blob) throws IOException, SQLException {
		InputStream inStream = null;
		ByteArrayOutputStream outStream = null;
		byte[] result = null;
		//
		try {
			inStream = blob.getBinaryStream();
			outStream = new ByteArrayOutputStream();
			//
			byte[] buffer = new byte[1024];
			int length;
			while ((length = inStream.read(buffer)) != -1) {
				outStream.write(buffer, 0, length);
			}
		} finally {
			if (inStream != null) {
				inStream.close();
				inStream = null;
			}
			if (outStream != null) {
				outStream.flush();
				result = outStream.toByteArray();
				outStream.close();
				outStream = null;
			}
		}
		return result;
	}
	/**
	 * Search images with sql conditions.
	 *
	 * @return java.util.Hashtable[]
	 * @param connection java.sql.Connection
	 * @param sqlConditions java.lang.String
	 * @param double cutOff
	 * @exception java.sql.SQLException
	 */
	protected Hashtable[] searchWithConditions(Connection connection, String sqlConditions, double cutOff) throws SQLException {
		String sql = null;
		PreparedStatement pst = null;
		ResultSet rs = null;
		Vector results = new Vector();

		try {
			// create tamporary table for search results.
			sql = "INSERT INTO search SELECT pkey, " + sqlConditions + " FROM image_searcher";
			pst = connection.prepareStatement(sql);
			pst.clearParameters();
			pst.executeUpdate();
			pst.close();
			pst = null;
			
			// sort results.
			sql = "SELECT image_searcher.pkey, search.value, image_searcher.icon_area_x, image_searcher.icon_area_y ";
			sql = sql + "FROM image_searcher, search WHERE image_searcher.pkey = search.pkey AND ";
			sql = sql + "search.value <= ? ORDER BY search.value";
			pst = connection.prepareStatement(sql);
			pst.clearParameters();
			pst.setFloat(1, new Double(1.0d - cutOff).floatValue());
			rs = pst.executeQuery();
			if (rs != null) {
				while (rs.next()) {
					Hashtable answer = new Hashtable();
					answer.put("pkey", new Integer(rs.getInt(1)));
					answer.put("value", new Float(rs.getFloat(2)));
					answer.put("iconArea", new Dimension(rs.getInt(3), rs.getInt(4)));
					results.addElement(answer);
				}
				rs.close();
				rs = null;
			}
			pst.close();
			pst = null;
		} finally {
			if (rs != null) {
				rs.close();
				rs = null;
			}
			if (pst != null) {
				pst.close();
				pst = null;
			}
		}

		Hashtable[] ids = new Hashtable[results.size()];
		results.copyInto(ids);
		return ids;
	}
	/**
	 * Store an image object on database with pkey.
	 *
	 * @param connection java.sql.Connection
	 * @param pkey int
	 * @param imageObject sra.isr.image.objects.IsrImageObject
	 * @param originalImage sra.isr.image.objects.IsrImageFileInfo
	 * @param thumbnailImage sra.isr.image.objects.IsrImageFileInfo
	 * @exception java.sql.SQLException
	 */
	protected void storeImage(Connection connection, int pkey, IsrImageObject imageObject, IsrImageFileInfo originalImage, IsrImageFileInfo thumbnailImage) throws SQLException {
		String sql = null;
		PreparedStatement pst = null;
		ResultSet rs = null;

		try {
			// store image object.
			sql = "INSERT INTO image_searcher (pkey, original_image, original_image_type, icon_image, icon_image_type, ";
			sql = sql + "icon_area_x, icon_area_y, width_ratio, luminance, center_x, center_y, tc_r, tc_g, tc_b, ";
			sql = sql + "nc1_r, nc1_g, nc1_b, nc2_r, nc2_g, nc2_b, nc3_r, nc3_g, nc3_b, ";
			sql = sql + "nc4_r, nc4_g, nc4_b, nc5_r, nc5_g, nc5_b, nc6_r, nc6_g, nc6_b, ";
			sql = sql + "nc7_r, nc7_g, nc7_b, nc8_r, nc8_g, nc8_b, nc9_r, nc9_g, nc9_b) ";
			sql = sql + "values (?, empty_blob(), ?, empty_blob(), ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ";
			sql = sql + "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
			pst = connection.prepareStatement(sql);
			pst.clearParameters();
			pst.setInt(1, pkey);
			pst.setString(2, originalImage.getContentType());
			pst.setString(3, thumbnailImage.getContentType());
			pst.setInt(4, imageObject.iconArea().width);
			pst.setInt(5, imageObject.iconArea().height);
			pst.setInt(6, imageObject.iconArea().width * 255 / (imageObject.iconArea().width + imageObject.iconArea().height));
			pst.setInt(7, imageObject.averageColor().getRed());
			pst.setInt(8, (int) (imageObject.centerSpectra().x() * 255));
			pst.setInt(9, (int) (imageObject.centerSpectra().y() * 255));
			pst.setInt(10, imageObject.typicalColor().getRed());
			pst.setInt(11, imageObject.typicalColor().getGreen());
			pst.setInt(12, imageObject.typicalColor().getBlue());
			Color[] colors = imageObject.nineColors();
			for (int i = 0; i < 9; i++) {
				Color nineColor = colors[i];
				pst.setInt(13 + i * 3 + 0, nineColor.getRed());
				pst.setInt(13 + i * 3 + 1, nineColor.getGreen());
				pst.setInt(13 + i * 3 + 2, nineColor.getBlue());
			}
			pst.executeUpdate();
			pst.close();
			pst = null;

			// store original image and thumbnail image.
			sql = "SELECT original_image, icon_image FROM image_searcher WHERE pkey = ? FOR UPDATE";
			pst = connection.prepareStatement(sql);
			pst.clearParameters();
			pst.setInt(1, pkey);
			rs = pst.executeQuery();
			if (rs != null) {
				if (rs.next()) {
					this.writeOnBlob(((OracleResultSet) rs).getBLOB(1), originalImage.getBytes());
					this.writeOnBlob(((OracleResultSet) rs).getBLOB(2), thumbnailImage.getBytes());
				}
				rs.close();
				rs = null;
			}
			pst.close();
			pst = null;
		} catch (IOException e) {
			throw new SQLException(e.getMessage());
		} finally {
			if (rs != null) {
				rs.close();
				rs = null;
			}
			if (pst != null) {
				pst.close();
				pst = null;
			}
		}
	}
	/**
	 * Write the byte array to the BLOB on database.
	 *
	 * @param blob oracle.sql.BLOB
	 * @param bytes byte[]
	 * @exception java.io.IOException
	 * @exception java.sql.SQLException
	 */
	private void writeOnBlob(BLOB blob, byte[] bytes) throws IOException, SQLException {
		OutputStream outStream = null;
		try {
			outStream = new BufferedOutputStream(blob.getBinaryOutputStream());
			outStream.write(bytes);
		} finally {
			if (outStream != null) {
				outStream.flush();
				outStream.close();
				outStream = null;
			}
		}
	}
}
