package sra.isr.image.database;

import java.awt.*;
import java.io.*;
import java.sql.*;
import java.util.*;
import sra.smalltalk.*;
import sra.isr.image.streams.*;
import sra.isr.image.objects.*;
import sra.isr.interfaces.progress.*;
import sra.isr.system.support.*;

/**
 * IsrImageDatabaseTestExamples class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2001/06/15 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageDatabaseTestExamples.java,v 4.2 2002/07/11 09:25:03 m-asada Exp $
 */
public class IsrImageDatabaseTestExamples extends IsrTestExamples {
	/**
	 * Create the original image.
	 *
	 * @return byte[]
	 * @param imageObject sra.isr.image.objects.IsrImageObject
	 */
	private static byte[] CreateOriginalImage(IsrImageObject imageObject) {
		byte[] originalImage = null;
		//
		InputStream in = null;
		ByteArrayOutputStream out = null;
		try {
			in = new FileInputStream(imageObject.asFilename());
			out = new ByteArrayOutputStream();
			//
			byte[] buffer = new byte[1024];
			int length;
			while ((length = in.read(buffer)) != -1) {
				out.write(buffer, 0, length);
			}
			originalImage = out.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			try {
				if (in != null) {
					in.close();
				}
				if (out != null) {
					out.flush();
					out.close();
				}
			} catch (IOException e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
		}
		return originalImage;
	}
	/**
	 * Create the thumbnail image.
	 *
	 * @return byte[]
	 * @param imageObject sra.isr.image.objects.IsrImageObject
	 */
	private static byte[] CreateThumbnailImage(IsrImageObject imageObject) {
		byte[] thumbnailImage = null;
		IsrImageStream imageStream = null;
		try {
			StImage iconImage = imageObject.iconImage();
			Rectangle iconArea = imageObject.iconArea();
			StImage newIconImage = new StImage(iconArea.width, iconArea.height);
			newIconImage = newIconImage.copy_from_in_rule_(newIconImage.bounds(), iconArea.getLocation(), iconImage, StImage.Over);
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
			imageStream = IsrJpegImageStream.On_(byteStream);
			imageStream.nextPutImage_(newIconImage);
			imageStream.flush();
			imageStream.close();
			thumbnailImage = byteStream.toByteArray();
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		} finally {
			if (imageStream != null) {
				imageStream = null;
			}
		}
		return thumbnailImage;
	}
	/**
	 * Example.
	 *
	 * @return boolean.
	 */
	public static boolean Example1() {
		IsrImageFolder imageFolder = IsrImageFolder.FromUser();
		if (imageFolder == null) {
			return false;
		}
		String filename = StDialog.Request_("Enter file name of isr.peoperties, then Ok");
		if (filename == null) {
			return false;
		}
		Properties properties = new Properties();
		try {
			File file = new File(filename);
			if (file.exists() == false || file.isDirectory() == true) {
				return false;
			}
			properties.load(new FileInputStream(file));
		} catch (IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
		}

		final IsrImageObject[] imageObjects = imageFolder.imageObjects();
		final int size = imageObjects.length;
		final IsrImageDatabase database = IsrImageDatabase.GetInstance(properties);

		(new IsrProgress()).do_(new StBlockClosure() {
			public Object value_(Object obj) {
				IsrProgress progress = (IsrProgress) obj;
				int count = 1;
				progress.message_("Store image ...");
				for (int i = 0; i < size; i++) {
					IsrImageObject imageObject = imageObjects[i];
					progress.message_("Store image ... " + imageObject.fileName());
					IsrImageFileInfo originalImage = new IsrImageFileInfo();
					IsrImageFileInfo thumbnailImage = new IsrImageFileInfo();
					originalImage.setContentType("image/jpeg");
					originalImage.setBytes(CreateOriginalImage(imageObject));
					thumbnailImage.setContentType("image/jpeg");
					thumbnailImage.setBytes(CreateThumbnailImage(imageObject));
					try {
						database.storeImage(imageObject, originalImage, thumbnailImage);
					} catch (SQLException e) {
						System.out.println(e.getMessage());
						e.printStackTrace();
					}
					progress.value_(((float) count) / size);
					count = count + 1;
				}
				return null;
			}
		});
		return true;
	}
	/**
	 * Execute all examples.
	 *
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new IsrImageDatabaseTestExamples();
	}
}
