package sra.isr.image.objects;

import java.awt.*;
import java.util.*;
import java.io.*;
import sra.smalltalk.*;
import sra.isr.system.support.*;

/**
 * IsrImageObjectTestExamples class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/27 (by Mitsuhiro Asada)
 * 	@updated   2000/03/01 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageObjectTestExamples.java,v 4.2 2002/07/11 09:25:03 m-asada Exp $
 */
public class IsrImageObjectTestExamples extends IsrTestExamples {
	/**
	 * 
	 */
	public static boolean Example1() {
		IsrImageObject imageObject = IsrImageObject.FileName_imageFolder_("Img0001.jpg", IsrImageFolder.DirectoryName_("temp"));
		imageObject.originalImage()._display();
		System.out.println(imageObject.centerSpectra());
		System.out.println(imageObject.averageColor().toString());
		System.out.println(imageObject.averageGray().toString());
		System.out.println(imageObject.typicalColor().toString());
		System.out.println(imageObject.typicalGray().toString());
		System.out.println(imageObject.iconArea());
		Color[] nineColors = imageObject.nineColors();
		for (int i = 0; i < nineColors.length; i++) {
			System.out.println(i + " " + nineColors[i].toString());
		}
		imageObject.iconImage()._display();
		imageObject.grayImage()._display();
		imageObject.imageCenterSpectra()._display();
		imageObject.xSpectrum()._display();
		imageObject.ySpectrum()._display();
		imageObject.imageAverageColor()._display();
		imageObject.imageAverageGray()._display();
		imageObject.imageTypicalColor()._display();
		imageObject.imageTypicalGray()._display();
		imageObject.imageNineColors()._display();
		imageObject.imageTracePolylines()._display();
		imageObject.thinImage()._display();
		imageObject.imageAttributes()._display();
		return true;
	}
	/**
	 * 
	 */
	public static boolean Example2() {
		IsrImageObject imageObject = IsrImageObject.FileName_imageFolder_("Img0001.jpg", IsrImageFolder.DirectoryName_("temp"));
		imageObject.makeCaches();
		imageObject.setImageFolder_(null);
		//
		ObjectOutputStream out = null;
		try {
			out = new ObjectOutputStream(new FileOutputStream("temp\\foo.isr"));
			out.writeObject(imageObject);
		} catch (Exception e) {
			SmalltalkException.Error("Write error");
		} finally {
			try {
				if (out != null) {
					out.flush();
					out.close();
				}
			} catch (IOException e) {
			}
		}
		return true;
	}
	/**
	 * 
	 */
	public static boolean Example3() {
		IsrImageObject imageObject = null;
		//
		ObjectInputStream in = null;
		try {
			in = new ObjectInputStream(new FileInputStream("temp\\foo.isr"));
			imageObject = (IsrImageObject) in.readObject();
		} catch (Exception e) {
			SmalltalkException.Error("Read error");
		} finally {
			try {
				if (in != null) {
					in.close();
				}
			} catch (IOException e) {
			}
		}
		imageObject.imageAttributes()._display();
		return true;
	}
	/**
	 * Execute all examples.
	 * @param args java.lang.String[]
	 */
	public static void main(java.lang.String[] args) {
		new IsrImageObjectTestExamples();
	}
}
