package sra.isr.image.searcher;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * IsrImageSearcherApplet class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2001/02/15 (by Mitsuhiro Asada)
 * 	@updated   2001/06/15 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageSearcherApplet.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrImageSearcherApplet extends Applet implements ActionListener {
	IsrImageSearcherPanel imageSearcherPane;
	/**
	 * IsrImageSearcherApplet constructor.
	 */
	public IsrImageSearcherApplet() {
		super();
	}
	/**
	 * ActionPerformed method.
	 *
	 * @param event java.awt.ActionEvent
	 */
	public void actionPerformed(ActionEvent event) {
		String name = ((Button) event.getSource()).getName();
		if (name == "do") {
			this.doSearch();
		}
	}
	/**
	 * Answer the condition parameters string added to URL.
	 *
	 * @return java.lang.String
	 */
	public String conditionStrings() {
		Vector conditions = new Vector();
		IsrImageSearcherPanel imageSearcher = this.getImageSearcherPane();

		// typical color
		if (imageSearcher.getTypicalColorSwitch()) {
			conditions.addElement("typical=" + imageSearcher.getTypicalColor().getRGB());
		}
		// average luminance
		if (imageSearcher.getAverageGraySwitch()) {
			conditions.addElement("luminance=" + imageSearcher.getAverageGray());
		}
		// nine colors
		if (imageSearcher.getNineColorsSwitch()) {
			Color[] nineColors = imageSearcher.getNineColors();
			for (int i = 0; i < 9; i++) {
				Color color = nineColors[i];
				if (color != null) {
					conditions.addElement("color" + (i + 1) + "=" + color.getRGB());
				}
			}
		}
		// icon area
		if (imageSearcher.getIconAreaSwitch()) {
			conditions.addElement("extent=" + imageSearcher.getIconArea().x() + "@" + imageSearcher.getIconArea().y());
		}
		// center spectra
		if (imageSearcher.getCenterSpectraSwitch()) {
			conditions.addElement("center=" + imageSearcher.getCenterSpectra().x() + "@" + imageSearcher.getCenterSpectra().y());
		}
		// cut off
		conditions.addElement("cutoff=" + imageSearcher.getCutOff());
		//
		return this.createUrlParameter(conditions);
	}
	/**
	 * Answer the string of url parameters.
	 *
	 * @return java.lang.String
	 * @param conditions java.lang.String[]
	 */
	protected String createUrlParameter(String[] conditions) {
		String result = null;
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		try {
			for (int i = 0; i < conditions.length - 1; i++) {
				pw.write(conditions[i]);
				pw.write("&");
			}
			pw.write(conditions[conditions.length - 1]);
			result = sw.toString();
		} finally {
			pw.flush();
			pw.close();
			pw = null;
		}
		return result;
	}
	/**
	 * Answer the string of url parameters.
	 *
	 * @return java.lang.String
	 * @param conditions java.util.Vector
	 */
	protected String createUrlParameter(Vector conditions) {
		String[] strings = new String[conditions.size()];
		conditions.copyInto(strings);
		return this.createUrlParameter(strings);
	}
	/**
	 * Handle the main Search Button fuction
	 */
	public void doSearch() {
		try {
			URL url = new URL(this.urlString());
			this.getAppletContext().showDocument(url, "_self");
		} catch (MalformedURLException e) {
		}
	}
	/**
	 * Answer do button container.
	 *
	 * rerurn java.awt.Container
	 */
	public Container getCommandPane() {
		Container container = new Panel();
		container.setLayout(new FlowLayout());
		Button doButton = new Button("   search   ");
		doButton.setName("do");
		doButton.addActionListener(this);
		container.add(doButton);
		return container;
	}
	/**
	 * Answer image searcher container.
	 *
	 * rerurn sra.isr.web.applet.IsrImageSearcherPanel
	 */
	public IsrImageSearcherPanel getImageSearcherPane() {
		if (imageSearcherPane == null) {
			imageSearcherPane = new IsrImageSearcherPanel();
		}
		return imageSearcherPane;
	}
	/**
	 * Handle the Applet init method.
	 */
	public void init() {
		this.setLayout(new BorderLayout());
		this.add("Center", this.getImageSearcherPane());
		this.add("South", this.getCommandPane());
		this.getImageSearcherPane().setCutOff(0.75d);
		this.setBackgraoundColor(this, new Color(0x0066CDAA));
	}
	/**
	 * Utility of set background color.
	 *
	 * @param component java.awt.Component
	 * @param backColor java.awt.Color
	 */
	public void setBackgraoundColor(Component component, Color backColor) {
		if (component instanceof Container) {
			Component[] components = ((Container) component).getComponents();
			for (int i = 0; i < components.length; i++) {
				this.setBackgraoundColor(components[i], backColor);
			}
		}
		if (!(component instanceof Button)) {
			component.setBackground(backColor);
		}
	}
	/**
	 * Handle the Applet start method.
	 */
	public void start() {
	}
	/**
	 * Handle the Applet stop method.
	 */
	public void stop() {
	}
	/**
	 * Answer the url of image searcher.
	 *
	 * @return java.lang.String
	 */
	public String urlString() {
		String url = this.getDocumentBase().toString();
		url = url.substring(0, url.lastIndexOf("/"));
		url = url + "/ImageSearcher";
		String conditions = this.conditionStrings();
		if (conditions != null && conditions.length() != 0) {
			url = url + "?" + conditions;
		}
		return url;
	}
}
