package sra.isr.image.searcher;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.interfaces.area.*;
import sra.isr.interfaces.colors.*;
import sra.isr.interfaces.freehand.*;
import sra.isr.interfaces.gauge.*;
import sra.isr.interfaces.nineColors.*;
import sra.isr.interfaces.spot.*;

/**
 * IsrImageSearcherJPanel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/06/23 (by Mitsuhiro Asada)
 * 	@updated   2001/06/15 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageSearcherJPanel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrImageSearcherJPanel extends JPanel {
	private IsrImageSearcher imageSearcher = null;
	private IsrColorChoiceJPanel typicalColorPanel = null;
	private IsrGaugeJPanel averageGrayPanel = null;
	private Isr9ColorsJPanel nineColorsPanel = null;
	private IsrAreaJPanel iconAreaPanel = null;
	private IsrSpotJPanel centerSpectraPanel = null;
	private IsrFreehandImageJPanel freehandImagePanel = null;
	private IsrGaugeJPanel cutOffPanel = null;
	/**
	 * IsrImageSearcherJpanel constructor.
	 */
	public IsrImageSearcherJPanel() {
		super();
		initialize();
	}
	/**
	 * IsrImageSearcherJpanel constructor.
	 *
	 * @param layout java.awt.LayoutManager
	 */
	public IsrImageSearcherJPanel(LayoutManager layout) {
		super(layout);
	}
	/**
	 * IsrImageSearcherJpanel constructor.
	 * 
	 * @param layout java.awt.LayoutManager
	 * @param isDoubleBuffered boolean
	 */
	public IsrImageSearcherJPanel(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
	}
	/**
	 * IsrImageSearcherJpanel constructor.
	 * 
	 * @param isDoubleBuffered boolean
	 */
	public IsrImageSearcherJPanel(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
	}
	/**
	 * Answer average gray parameter.
	 *
	 * @return double 
	 */
	public double getAverageGray() {
		return this.getAverageGrayPanel().gaugeValue();
	}
	/**
	 * Return the averageGrayPanel property value.
	 * 
	 * @return sra.isr.interfaces.applet.IsrGaugeJPanel
	 */
	protected IsrGaugeJPanel getAverageGrayPanel() {
		if (averageGrayPanel == null) {
			try {
				averageGrayPanel = new IsrGaugeJPanel();
				averageGrayPanel.setName("AverageGrayPanel");
				averageGrayPanel.setLocation(2, 164);
				averageGrayPanel.setGaugeModel(this.getImageSearcher().averageGrayModel());
				averageGrayPanel.getGaugeSwitch().setText("average luminance");
			} catch (Throwable e) {
				this.handleException(e);
			}
		}
		return averageGrayPanel;
	}
	/**
	 * Answer average gray switch.
	 *
	 * @return boolean 
	 */
	public boolean getAverageGraySwitch() {
		return this.getAverageGrayPanel().gaugeSwitch();
	}
	/**
	 * Answer average gray parameter.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint 
	 */
	public Isr2dPoint getCenterSpectra() {
		return this.getCenterSpectraPanel().xy();
	}
	/**
	 * Return the centerSpectraPanel property value.
	 * 
	 * @return sra.isr.interfaces.applet.IsrSpotJPanel
	 */
	protected IsrSpotJPanel getCenterSpectraPanel() {
		if (centerSpectraPanel == null) {
			try {
				centerSpectraPanel = new IsrSpotJPanel();
				centerSpectraPanel.setName("CenterSpectraPanel");
				centerSpectraPanel.setLocation(492, 2);
				centerSpectraPanel.setSpotModel(this.getImageSearcher().centerSpectraModel());
				centerSpectraPanel.getSpotSwitch().setText("center spectra");
			} catch (Throwable e) {
				this.handleException(e);
			}
		}
		return centerSpectraPanel;
	}
	/**
	 * Answer center spectra switch.
	 *
	 * @return boolean 
	 */
	public boolean getCenterSpectraSwitch() {
		return this.getCenterSpectraPanel().spotSwitch();
	}
	/**
	 * Answer cut off parameter.
	 *
	 * @return double 
	 */
	public double getCutOff() {
		return this.getCutOffPanel().gaugeValue();
	}
	/**
	 * Return the cutOffPanel property value.
	 *
	 * @return sra.isr.interfaces.applet.IsrGaugeJPanel
	 */
	protected IsrGaugeJPanel getCutOffPanel() {
		if (cutOffPanel == null) {
			try {
				cutOffPanel = new IsrGaugeJPanel();
				cutOffPanel.setName("CutOffPanel");
				cutOffPanel.setLocation(348, 164);
				cutOffPanel.setGaugeModel(this.getImageSearcher().cutOffModel());
				cutOffPanel.getGaugeSwitch().setText("cut off");
				//
				cutOffPanel.getGaugeSwitch().setVisible(false);
				JLabel label = new JLabel("cut off");
				label.setName("CutOffLabel");
				label.setBounds(15, 2, 130, 25);
				cutOffPanel.add(label, label.getName());
			} catch (Throwable e) {
				this.handleException(e);
			}
		}
		return cutOffPanel;
	}
	/**
	 * Answer cut off switch.
	 *
	 * @return boolean 
	 */
	public boolean getCutOffSwitch() {
		return this.getCutOffPanel().gaugeSwitch();
	}
	/**
	 * Answer freehand image parameter.
	 *
	 * @return java.awt.Image 
	 */
	public Image getFreehandImage() {
		return this.getFreehanImagePanel().freehandImage();
	}
	/**
	 * Return the freehandImagePanel property value.
	 * 
	 * @return sra.isr.interfaces.applet.IsrFreehandImageJPanel
	 */
	protected IsrFreehandImageJPanel getFreehanImagePanel() {
		if (freehandImagePanel == null) {
			try {
				freehandImagePanel = new IsrFreehandImageJPanel();
				freehandImagePanel.setName("FreehanImagePanel");
				freehandImagePanel.setLocation(138, 164);
				freehandImagePanel.setFreehandModel(this.getImageSearcher().freehandImageModel());
				freehandImagePanel.getFreehandSwitch().setText("freehand image");
			} catch (Throwable e) {
				handleException(e);
			}
		}
		return freehandImagePanel;
	}
	/**
	 * Answer freehand image switch.
	 *
	 * @return boolean 
	 */
	public boolean getFreehanImageSwitch() {
		return this.getFreehanImagePanel().freehandSwitch();
	}
	/**
	 * Answer icon area parameter.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint 
	 */
	public Isr2dPoint getIconArea() {
		return this.getIconAreaPanel().xy();
	}
	/**
	 * Return the iconAreaPanel property value.
	 * 
	 * @return sra.isr.interfaces.applet.IsrAreaJPanel
	 */
	protected IsrAreaJPanel getIconAreaPanel() {
		if (iconAreaPanel == null) {
			try {
				iconAreaPanel = new IsrAreaJPanel();
				iconAreaPanel.setName("IconAreaPanel");
				iconAreaPanel.setLocation(365, 2);
				iconAreaPanel.setAreaModel(this.getImageSearcher().iconAreaModel());
				iconAreaPanel.getAreaSwitch().setText("image extent");
			} catch (Throwable e) {
				this.handleException(e);
			}
		}
		return iconAreaPanel;
	}
	/**
	 * Answer icon area switch.
	 *
	 * @return boolean 
	 */
	public boolean getIconAreaSwitch() {
		return this.getIconAreaPanel().areaSwitch();
	}
	/**
	 * Return the ImageSearcher property value.
	 * 
	 * @return sra.isr.image.searcher.IsrImageSearcher
	 */
	protected IsrImageSearcher getImageSearcher() {
		if (imageSearcher == null) {
			try {
				imageSearcher = new IsrImageSearcher();
			} catch (Throwable e) {
				this.handleException(e);
			}
		}
		return imageSearcher;
	}
	/**
	 * Answer nine colors parameter.
	 *
	 * @return java.awt.Color[9] 
	 */
	public Color[] getNineColors() {
		return this.getNineColorsPanel().colors();
	}
	/**
	 * Return the nineColorsPanel property value.
	 * 
	 * @return sra.isr.interfaces.applet.Isr9ColorsJPanel
	 */
	protected Isr9ColorsJPanel getNineColorsPanel() {
		if (nineColorsPanel == null) {
			try {
				nineColorsPanel = new Isr9ColorsJPanel();
				nineColorsPanel.setName("NineColorsPanel");
				nineColorsPanel.setLocation(238, 2);
				nineColorsPanel.setNineColorsModel(this.getImageSearcher().nineColorsModel());
				nineColorsPanel.getNineColorsSwitch().setText("nine colors");
			} catch (Throwable e) {
				this.handleException(e);
			}
		}
		return nineColorsPanel;
	}
	/**
	 * Answer nine colors switch.
	 *
	 * @return boolean 
	 */
	public boolean getNineColorsSwitch() {
		return this.getNineColorsPanel().nineColorsSwitch();
	}
	/**
	 * Answer typical color parameter.
	 *
	 * @return java.awt.Color 
	 */
	public Color getTypicalColor() {
		return this.getTypicalColorPanel().color();
	}
	/**
	 * Return the typicalColorPanel property value.
	 * 
	 * @return sra.isr.interfaces.applet.IsrColorChoiceJPanel
	 */
	protected IsrColorChoiceJPanel getTypicalColorPanel() {
		if (typicalColorPanel == null) {
			try {
				typicalColorPanel = new IsrColorChoiceJPanel();
				typicalColorPanel.setName("typicalColorPanel");
				typicalColorPanel.setLocation(2, 2);
				typicalColorPanel.setColorChoiceModel(this.getImageSearcher().typicalColorModel());
				typicalColorPanel.getColorChoiceSwitch().setText("typical color");
			} catch (Throwable e) {
				this.handleException(e);
			}
		}
		return typicalColorPanel;
	}
	/**
	 * Answer typical color switch.
	 *
	 * @return boolean 
	 */
	public boolean getTypicalColorSwitch() {
		return this.getTypicalColorPanel().colorChoiceSwitch();
	}
	/**
	 * Called whenever the part throws an exception.
	 * 
	 * @param exception java.lang.Throwable
	 */
	private void handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}
	/**
	 * Initialize the class.
	 */
	private void initialize() {
		try {
			this.setName("IsrImageSearcherJpanel");
			this.setLayout(null);
			this.setSize(623, 411);
			this.add(this.getTypicalColorPanel(), this.getTypicalColorPanel().getName());
			this.add(this.getNineColorsPanel(), this.getNineColorsPanel().getName());
			this.add(this.getIconAreaPanel(), this.getIconAreaPanel().getName());
			this.add(this.getCenterSpectraPanel(), this.getCenterSpectraPanel().getName());
			this.add(this.getFreehanImagePanel(), this.getFreehanImagePanel().getName());
			this.add(this.getAverageGrayPanel(), this.getAverageGrayPanel().getName());
			this.add(this.getCutOffPanel(), this.getCutOffPanel().getName());
		} catch (Throwable e) {
			this.handleException(e);
		}
	}
	/**
	 * Set average gray parameter.
	 *
	 * @param value double
	 */
	public void setAverageGray(double value) {
		this.getAverageGrayPanel().gaugeValue_(value);
	}
	/**
	 * Set average gray parameter.
	 *
	 * @param color java.awt.Color 
	 */
	public void setAverageGray(Color color) {
		double value = (double) StColorValue._GetBrightness(color);
		value = Math.max(0, Math.min(value * 100, 100));
		this.setAverageGray(value);
	}
	/**
	 * Set average gray switch.
	 *
	 * @param aBoolean boolean 
	 */
	public void setAverageGraySwitch(boolean aBoolean) {
		this.getAverageGrayPanel().gaugeSwitch_(aBoolean);
	}
	/**
	 * Set average gray parameter.
	 *
	 * @param x double
	 * @param y double
	 */
	public void setCenterSpectra(double x, double y) {
		this.setCenterSpectra(new Isr2dPoint(x,y));
	}
	/**
	 * Set average gray parameter.
	 *
	 * @param point sra.isr.geometry.basic.Isr2dPoint 
	 */
	public void setCenterSpectra(Isr2dPoint point) {
		this.getCenterSpectraPanel().xy_(point);
	}
	/**
	 * Set center spectra switch.
	 *
	 * @param aBoolean boolean 
	 */
	public void setCenterSpectraSwitch(boolean aBoolean) {
		this.getCenterSpectraPanel().spotSwitch_(aBoolean);
	}
	/**
	 * Set cut off parameter.
	 *
	 * @param value double 
	 */
	public void setCutOff(double value) {
		this.getCutOffPanel().gaugeValue_(value);
	}
	/**
	 * Set freehand image parameter.
	 *
	 * @param image java.awt.Image 
	 */
	public void setFreehandImage(Image image) {
		this.getFreehanImagePanel().freehandImage_(image);
	}
	/**
	 * Set freehand image switch.
	 *
	 * @param aBoolean boolean 
	 */
	public void setFreehanImageSwitch(boolean aBoolean) {
		this.getFreehanImagePanel().freehnadSwitch_(aBoolean);
	}
	/**
	 * Set icon area parameter.
	 *
	 * @param x double
	 * @param y double
	 */
	public void setIconArea(double x, double y) {
		this.setIconArea(new Isr2dPoint(x, y));
	}
	/**
	 * Set icon area parameter.
	 *
	 * @param point sra.isr.geometry.basic.Isr2dPoint 
	 */
	public void setIconArea(Isr2dPoint point) {
		this.getIconAreaPanel().xy_(point);
	}
	/**
	 * Set icon area switch.
	 *
	 * @param aBoolean boolean 
	 */
	public void setIconAreaSwitch(boolean aBoolean) {
		this.getIconAreaPanel().areaSwitch_(aBoolean);
	}
	/**
	 * Set nine colors parameter.
	 *
	 * @param colors java.awt.Color[9] 
	 */
	public void setNineColors(Color[] colors) {
		this.getNineColorsPanel().colors_(colors);
	}
	/**
	 * Set nine colors switch.
	 *
	 * @param aBoolean boolean 
	 */
	public void setNineColorsSwitch(boolean aBoolean) {
		this.getNineColorsPanel().nineColorsSwitch_(aBoolean);
	}
	/**
	 * Set typical color parameter.
	 *
	 * @param color java.awt.Color 
	 */
	public void setTypicalColor(Color color) {
		this.getTypicalColorPanel().color_(color);
	}
	/**
	 * Set typical color switch.
	 *
	 * @param aBoolean boolean 
	 */
	public void setTypicalColorSwitch(boolean aBoolean) {
		this.getTypicalColorPanel().colorChoiceSwitch_(aBoolean);
	}
}
