package sra.isr.image.searcher;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import sra.smalltalk.*;
import sra.isr.interfaces.button.*;
import sra.isr.system.support.*;

/**
 * IsrImageSearcherViewSwing class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/03/13 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageSearcherViewSwing.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrImageSearcherViewSwing extends StViewJPanel implements IsrImageSearcherView {
	protected IsrButtonViewSwing menuButtonView;
	protected IsrButtonViewSwing iconAreaButtonView;
	protected IsrButtonViewSwing iconAreaSwitchView;
	protected IsrButtonViewSwing nineColorsButtonView;
	protected IsrButtonViewSwing nineColorsSwitchView;
	protected IsrButtonViewSwing typicalColorButtonView;
	protected IsrButtonViewSwing typicalColorSwitchView;
	protected IsrButtonViewSwing averageGrayButtonView;
	protected IsrButtonViewSwing averageGraySwitchView;
	protected IsrButtonViewSwing centerSpectraButtonView;
	protected IsrButtonViewSwing centerSpectraSwitchView;
	protected IsrButtonViewSwing freehandImageButtonView;
	protected IsrButtonViewSwing freehandImageSwitchView;
	protected IsrButtonViewSwing cutOffButtonView;
	protected IsrButtonViewSwing imageSearchButtonView;
	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		JPanel topPanel, leftPanel, centerPanel, downPanel;
		this.setLayout(new BorderLayout());
		this.setPreferredSize(new Dimension(170, 160));

		// top panel
		topPanel = new JPanel(new BorderLayout());
		//
		menuButtonView = new IsrButtonViewSwing() {
			protected StController defaultController() {
				return new IsrButtonController() {
					public void mousePressed(MouseEvent e) {
						IsrButtonViewSwing view = (IsrButtonViewSwing) this.getView();
						IsrButtonModel model = this.getModel();
						if (!view.getBounds().contains(e.getPoint())) {
							return;
						}
						StBlockClosure action = model.action();
						if (action != null) {
							switch (action.numArgs()) {
								case 0 :
									action.value();
									break;
								case 1 :
									action.value_(model);
									break;
								case 2 :
									action.value_value_(model, e);
									break;
							}
						}
						e.consume();
					}
					public void mouseReleased(MouseEvent e) {
					}
					public void mouseEntered(MouseEvent e) {
					}
					public void mouseExited(MouseEvent e) {
					}
				};
			}
		};
		menuButtonView.setPreferredSize(new Dimension(180, 20));
		topPanel.add(menuButtonView);

		// left panel
		leftPanel = new JPanel(new GridLayout(6, 1));
		//
		typicalColorSwitchView = new IsrButtonViewSwing();
		typicalColorSwitchView.setPreferredSize(new Dimension(20, 20));
		leftPanel.add(typicalColorSwitchView);
		//
		averageGraySwitchView = new IsrButtonViewSwing();
		averageGraySwitchView.setPreferredSize(new Dimension(20, 20));
		leftPanel.add(averageGraySwitchView);
		//
		nineColorsSwitchView = new IsrButtonViewSwing();
		nineColorsSwitchView.setPreferredSize(new Dimension(20, 20));
		leftPanel.add(nineColorsSwitchView);
		//
		iconAreaSwitchView = new IsrButtonViewSwing();
		iconAreaSwitchView.setPreferredSize(new Dimension(20, 20));
		leftPanel.add(iconAreaSwitchView);
		//
		centerSpectraSwitchView = new IsrButtonViewSwing();
		centerSpectraSwitchView.setPreferredSize(new Dimension(20, 20));
		leftPanel.add(centerSpectraSwitchView);
		//
		freehandImageSwitchView = new IsrButtonViewSwing();
		freehandImageSwitchView.setPreferredSize(new Dimension(20, 20));
		leftPanel.add(freehandImageSwitchView);

		// center panel
		centerPanel = new JPanel(new GridLayout(6, 1));
		//
		typicalColorButtonView = new IsrButtonViewSwing();
		typicalColorButtonView.setPreferredSize(new Dimension(160, 20));
		centerPanel.add(typicalColorButtonView);
		//
		averageGrayButtonView = new IsrButtonViewSwing();
		averageGrayButtonView.setPreferredSize(new Dimension(160, 20));
		centerPanel.add(averageGrayButtonView);
		//
		nineColorsButtonView = new IsrButtonViewSwing();
		nineColorsButtonView.setPreferredSize(new Dimension(160, 20));
		centerPanel.add(nineColorsButtonView);
		//
		iconAreaButtonView = new IsrButtonViewSwing();
		iconAreaButtonView.setPreferredSize(new Dimension(160, 20));
		centerPanel.add(iconAreaButtonView);
		//
		centerSpectraButtonView = new IsrButtonViewSwing();
		centerSpectraButtonView.setPreferredSize(new Dimension(160, 20));
		centerPanel.add(centerSpectraButtonView);
		//
		freehandImageButtonView = new IsrButtonViewSwing();
		freehandImageButtonView.setPreferredSize(new Dimension(160, 20));
		centerPanel.add(freehandImageButtonView);

		// down panel
		downPanel = new JPanel(new GridLayout(1, 2));
		//
		cutOffButtonView = new IsrButtonViewSwing();
		cutOffButtonView.setPreferredSize(new Dimension(90, 20));
		downPanel.add(cutOffButtonView);
		//
		imageSearchButtonView = new IsrButtonViewSwing();
		imageSearchButtonView.setPreferredSize(new Dimension(90, 20));
		downPanel.add(imageSearchButtonView);
		this.add(topPanel, "North");
		this.add(leftPanel, "West");
		this.add(centerPanel, "Center");
		this.add(downPanel, "South");

		// controller
		controller = defaultController();
		controller.view_(this);
	}
	/**
	 * Answer my default controller.
	 *
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new IsrImageSearcherController();
	}
	/**
	 * Answer a default model.
	 *
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new IsrImageSearcher();
	}
	/**
	 * Answer a Model.
	 *
	 * @return sra.isr.image.searcher.IsrImageSearcher
	 */
	public IsrImageSearcher getModel() {
		return (IsrImageSearcher) this.model();
	}
	/**
	 * Rebuild this component.
	 *
	 * @param oldModel sra.smalltalk.StModel
	 * @param newModel sra.smalltalk.StModel
	 */
	protected void rebuildComponent(StModel oldModel, StModel newModel) {
		IsrImageSearcher model = (IsrImageSearcher) this.model();

		// top panel
		menuButtonView.model_(model.menuButton());

		// left panel
		typicalColorSwitchView.model_(model.typicalColorSwitch());
		averageGraySwitchView.model_(model.averageGraySwitch());
		nineColorsSwitchView.model_(model.nineColorsSwitch());
		iconAreaSwitchView.model_(model.iconAreaSwitch());
		centerSpectraSwitchView.model_(model.centerSpectraSwitch());
		freehandImageSwitchView.model_(model.freehandImageSwitch());

		//center panel
		typicalColorButtonView.model_(model.typicalColorButton());
		averageGrayButtonView.model_(model.averageGrayButton());
		nineColorsButtonView.model_(model.nineColorsButton());
		iconAreaButtonView.model_(model.iconAreaButton());
		centerSpectraButtonView.model_(model.centerSpectraButton());
		freehandImageButtonView.model_(model.freehandImageButton());

		// down panel
		cutOffButtonView.model_(model.cutOffButton());
		imageSearchButtonView.model_(model.imageSearchButton());
	}
	/**
	 * Model property accessing.
	 *
	 * @param aModel sra.isr.image.searcher.IsrImageSearcher
	 */
	public void setModel(IsrImageSearcher aModel) {
		this.model_(aModel);
	}
}
