package sra.isr.image.streams;

import java.awt.*;
import java.io.*;
import sra.smalltalk.*;

/**
 * IsrDefaultImageStream class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2001/04/18 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrDefaultImageStream.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrDefaultImageStream extends IsrImageStream {
	/**
	 * Read the image on input stream.
	 * (used Java core API - toolkil)
	 * 
	 * @return sra.smalltalk.StImage
	 * @exception java.io.IOException
	 */
	public StImage nextImage() throws IOException {
		if (inStream != null) {
			Frame frame = new Frame();
			Toolkit toolkit = frame.getToolkit();
			Image image = toolkit.createImage(this.readInputStream());
			MediaTracker tracker = new MediaTracker(frame);
			tracker.addImage(image, 0);
			try {
				tracker.waitForAll();
			} catch (InterruptedException exception) {
				throw new IOException("image load interrupted.");
			}
			int status = tracker.statusAll(false);
			if ((status & MediaTracker.ABORTED) != 0 || (status & MediaTracker.ERRORED) != 0) {
				throw new IOException("image load interrupted.");
			}
			image.flush();
			imageObject = new StImage(image);
		}
		return imageObject;
	}
	/**
	 * Write the image on output stream.
	 * (used Java core API - toolkil)
	 *
	 * @param newImage sra.smalltalk.StImage
	 * @exception java.io.IOException
	 */
	public void nextPutImage_(StImage newImage) throws IOException {
		throw new IOException("could not save image");
	}
	/**
	 * Constractor with input stream.
	 *
	 * @return sra.isr.image.streams.IsrImageStream 
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 */
	public static IsrImageStream On_(InputStream stream) throws IOException {
		return On_(new IsrDefaultImageStream(), stream);
	}
	/**
	 * Constractor with output stream.
	 *
	 * @return sra.isr.image.streams.IsrImageStream 
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 */
	public static IsrImageStream On_(OutputStream stream) throws IOException {
		throw new IOException("could not save image");
	}
	/**
	 * Set the output stream.
	 * 
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 */
	protected void on_(OutputStream stream) throws IOException {
		throw new IOException("could not save image");
	}
	/**
	 * Read the input stream.
	 * 
	 * @return sra.smalltalk.StImage
	 * @exception java.io.IOException
	 */
	private byte[] readInputStream() throws IOException {
		byte[] bytes = null;
		if (inStream != null) {
			ByteArrayOutputStream byteStream = null;
			BufferedOutputStream bufferedStream = null;
			try {
				byteStream = new ByteArrayOutputStream();
				bufferedStream = new BufferedOutputStream(byteStream);
				byte[] buffer = new byte[1024];
				int size;
				while ((size = inStream.read(buffer, 0, buffer.length)) != -1) {
					bufferedStream.write(buffer, 0, size);
				}
				bufferedStream.flush();
				bytes = byteStream.toByteArray();
			} finally {
				if (bufferedStream != null) {
					bufferedStream.close();
					bufferedStream = null;
				}
				if (byteStream != null) {
					byteStream.close();
					byteStream = null;
				}
			}
		}
		return bytes;
	}
}
