package sra.isr.image.streams;

import java.awt.image.*;
import java.io.*;
import com.sun.image.codec.jpeg.*;
import sra.smalltalk.*;

/**
 * IsrJava2JpegImageStream class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/11/22 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrJava2JpegImageStream.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrJava2JpegImageStream extends IsrJpegImageStream {
	/**
	 * Read the image on input stream.
	 * (used Java2 core API)
	 * 
	 * @return sra.smalltalk.StImage
	 * @exception java.io.IOException
	 */
	public StImage nextImage() throws IOException {
		if (inStream != null) {
			JPEGImageDecoder imageDecoder = JPEGCodec.createJPEGDecoder(inStream);
			BufferedImage anImage = imageDecoder.decodeAsBufferedImage();
			imageObject = new StImage(anImage);
		}
		return imageObject;
	}
	/**
	 * Write the image on output stream.
	 * (used Java2 core API)
	 *
	 * @param newImage sra.smalltalk.StImage
	 * @exception java.io.IOException
	 */
	public void nextPutImage_(StImage newImage) throws IOException {
		if (outStream != null) {
			imageObject = newImage;
			//
			int width = imageObject.width();
			int height = imageObject.height();
			BufferedImage anImage = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR);
			anImage.setRGB(0, 0, width, height, imageObject.getPixels(), 0, width);
			JPEGImageEncoder imageEncoder = JPEGCodec.createJPEGEncoder(outStream);
			imageEncoder.encode(anImage);
		}
	}
	/**
	 * Constractor with input stream.
	 *
	 * @return sra.isr.image.streams.IsrImageStream 
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 */
	public static IsrImageStream On_(InputStream stream) throws IOException {
		return On_(new IsrJava2JpegImageStream(), stream);
	}
	/**
	 * Constractor with output stream.
	 *
	 * @return sra.isr.image.streams.IsrImageStream 
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 */
	public static IsrImageStream On_(OutputStream stream) throws IOException {
		return On_(new IsrJava2JpegImageStream(), stream);
	}
}
