package sra.isr.image.streams;

import java.io.*;

/**
 * IsrPngImageStream class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/11/16 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrPngImageStream.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public abstract class IsrPngImageStream extends IsrImageStream {
	/**
	 * Answer an instance of png image stream.
	 * 
	 * @return sra.isr.image.streams.IsrImageStream
	 */
	protected static IsrImageStream _CreateImageStream() {
		IsrImageStream imageStream = null;
		try {
			Class.forName("com.sun.media.jai.codec.ImageCodec");
			imageStream = (IsrImageStream) Class.forName("sra.isr.image.streams.IsrJaiPngImageStream").newInstance();
		} catch (Throwable e) {
			try {
				Class.forName("com.sun.jimi.core.Jimi");
				imageStream = (IsrImageStream) Class.forName("sra.isr.image.streams.IsrJimiPngImageStream").newInstance();
			} catch (Exception ex) {
			}
		}
		return imageStream;
	}
	/**
	 * Constractor with input stream.
	 *
	 * @return sra.isr.image.streams.IsrImageStream 
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 */
	public static IsrImageStream On_(InputStream stream) throws IOException {
		return On_(_CreateImageStream(), stream);
	}
	/**
	 * Constractor with output stream.
	 *
	 * @return sra.isr.image.streams.IsrImageStream 
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 */
	public static IsrImageStream On_(OutputStream stream) throws IOException {
		return On_(_CreateImageStream(), stream);
	}
}
