package sra.isr.image.streams;

import java.awt.*;
import java.io.*;
import sra.smalltalk.*;

/**
 * IsrSraBmpImageStream class
 * 
 * 	@author    Hirotsugu Kondo
 * 	@created   1999/01/20 (by Hirotsugu Kondo)
 * 	@updated   2000/23/03 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrSraBmpImageStream.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrSraBmpImageStream extends IsrBmpImageStream {
	protected int position = 0;
	private int bfType;
	private long bfSize;
	private int bfReserved1;
	private int bfReserved2;
	private long bfOffBits;
	private long biSize;
	private long biWidth;
	private long biHeight;
	private int biPlanes;
	private int biBitCount;
	private long biCompression;
	private long biSizeImage;
	private long biXPelsPerMeter;
	private long biYPelsPerMeter;
	private long biClrUsed;
	private long biClrImportant;
	private int[] imagePalette;
	/**
	 * Answer next value.
	 * 
	 * @return int
	 */
	private int next() {
		int nextValue = -1;
		try {
			nextValue = this.inStream.read();
			this.position = this.position + 1;
		} catch (Exception e) {
		}
		return nextValue;
	}
	/**
	 * Read the image on input stream.
	 *
	 * @return sra.smalltalk.StImage
	 * @exception java.io.IOException
	 */
	public StImage nextImage() throws IOException {
		if (inStream != null) {
			boolean check = this.readBitmapFileHeader();
			if (check == true) {
				this.readBitmapInfoHeader();
				this.readImagePalette();
				boolean check2 = this.readImageData();
				if (check2 == false) {
					imageObject = null;
				} 
			}
		}
		return imageObject;
	}
	/**
	 * Answer next long.
	 * 
	 * @return long
	 */
	private long nextLong() {
		int byte1 = this.next();
		int byte2 = this.next();
		int byte3 = this.next();
		int byte4 = this.next();
		return (byte1 << 24) + (byte2 << 16) + (byte3 << 8) + byte4;
	}
	/**
	 * Answer next LSB long. 
	 *
	 * @return long
	 */
	private long nextLSBLong() {
		int byte1 = this.next();
		int byte2 = this.next();
		int byte3 = this.next();
		int byte4 = this.next();
		return (byte4 << 24) + (byte3 << 16) + (byte2 << 8) + byte1;
	}
	/**
	 * Answer next LSB word.
	 * 
	 * @return int
	 */
	private int nextLSBWord() {
		int byte1 = this.next();
		int byte2 = this.next();
		return (byte2 << 8) + byte1;
	}
	/**
	 * Write the image on output stream.
	 * 
	 * @param newImage sra.smalltalk.StImage
	 * @exception java.io.IOException
	 */
	public void nextPutImage_(StImage newImage) throws IOException {
		throw new IOException("could not save image");
	}
	/**
	 * Answer next word.
	 * 
	 * @return int
	 */
	private int nextWord() {
		int byte1 = this.next();
		int byte2 = this.next();
		return (byte1 << 8) + byte2;
	}
	/**
	 * Constractor with input stream.
	 *
	 * @return sra.isr.image.streams.IsrImageStream 
	 * @param stream java.io.InputStream
	 * @exception java.io.IOException
	 */
	public static IsrImageStream On_(InputStream stream) throws IOException {
		return On_(new IsrSraBmpImageStream(), stream);
	}
	/**
	 * Constractor with output stream.
	 *
	 * @return sra.isr.image.streams.IsrImageStream 
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 */
	public static IsrImageStream On_(OutputStream stream) throws IOException {
		throw new IOException("could not save image");
	}
	/**
	 * Set the output stream.
	 * 
	 * @param stream java.io.OutputStream
	 * @exception java.io.IOException
	 */
	protected void on_(OutputStream stream) throws IOException {
		throw new IOException("could not save image");
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @return boolean
	 */
	private boolean readBitmapFileHeader() {
		int checker = ((int) 'B' << 8) + (int) 'M';
		this.bfType = this.nextWord();
		if (checker != this.bfType) {
			return false;
		}
		this.bfSize = this.nextLSBLong();
		this.bfReserved1 = this.nextLSBWord();
		this.bfReserved2 = this.nextLSBWord();
		this.bfOffBits = this.nextLSBLong();
		return true;
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	private void readBitmapInfoHeader() {
		this.biSize = this.nextLSBLong();
		this.biWidth = this.nextLSBLong();
		this.biHeight = this.nextLSBLong();
		this.biPlanes = this.nextLSBWord();
		this.biBitCount = this.nextLSBWord();
		this.biCompression = this.nextLSBLong();
		this.biSizeImage = this.nextLSBLong();
		this.biXPelsPerMeter = this.nextLSBLong();
		this.biYPelsPerMeter = this.nextLSBLong();
		this.biClrUsed = this.nextLSBLong();
		this.biClrImportant = this.nextLSBLong();
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @return sra.smalltalk.StImage
	 */
	private StImage readDepth1Data() {
		System.out.print("1bit BMP detect");
		return null;
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @return sra.smalltalk.StImage
	 */
	private StImage readDepth24Data() {
		while (this.position < this.bfOffBits) {
			this.next();
		}
		int width = (int) this.biWidth;
		int height = (int) this.biHeight;
		StImage image = new StImage(width, height);
		int depth = this.biBitCount;
		int total = width * height;
		for (int y = height - 1; y > -1; y--) {
			int count = 0;
			for (int x = 0; x < width; x++) {
				int rgb = (this.next() << 0) + (this.next() << 8) + (this.next() << 16);
				image.setPixel(x, y, new Color(rgb).getRGB());
				count = count + 3;
			}
			while (count % 4 != 0) {
				this.next();
				count++;
			}
		}
		return image;
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @return sra.smalltalk.StImage
	 */
	private StImage readDepth8Data() {
		int[] palette = this.imagePalette;
		int width = (int) this.biWidth;
		int height = (int) this.biHeight;
		StImage image = new StImage(width, height);
		int depth = this.biBitCount;
		int total = width * height;
		for (int y = height - 1; y > -1; y--) {
			int count = 0;
			for (int x = 0; x < width; x++) {
				int index = this.next();
				image.setPixel(x, y, palette[index]);
				count++;
			}
			while (count % 4 != 0) {
				this.next();
				count++;
			}
		}
		return image;
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	private void readDepth8Palette() {
		long size = 1 << this.biBitCount;
		if (this.biClrUsed != 0) {
			size = this.biClrUsed;
		}
		int[] colors = new int[ (int) size];
		for (int index = 0; index < size; index++) {
			int rgb = this.next() + (this.next() << 8) + (this.next() << 16);
			this.next();
			colors[index] = 0xff000000 | rgb;
		}
		this.imagePalette = colors;
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @return boolean
	 */
	private boolean readImageData() {
		if (this.biCompression != 0) {
			return false;
		}
		int bit = this.biBitCount;
		this.imageObject = null;
		if (bit == 24) {
			this.imageObject = this.readDepth24Data();
		}
		if (bit == 8) {
			this.imageObject = this.readDepth8Data();
		}
		if (bit == 1) {
			this.imageObject = this.readDepth1Data();
		}
		return this.imageObject != null;
	}
	/**
	 * Palette ignored but depth 8 BMP.
	 */
	private void readImagePalette() {
		this.imagePalette = null;
		if (this.biBitCount == 8) {
			this.readDepth8Palette();
		}
	}
}
