package sra.isr.image.support;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.interfaces.colors.*;
import sra.isr.interfaces.cursors.*;
import sra.isr.system.support.*;

/**
 * IsrImageProcessorTestExamples class
 * 
 * 	@author    Hirotsugu Kondo
 * 	@created   1998/11/16 (by Hirotsugu Kondo)
 * 	@updated   2000/02/16 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageProcessorTestExamples.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrImageProcessorTestExamples extends IsrTestExamples {
	/**
	 * Image generated from Smalltalk 8-bits image.
	 * 
	 * @return java.awt.Image
	 */
	protected static Image _ImageSmalltalkBalloon() {
		return SystemResourceSupport.createImage("/sra/isr/image/support/imageSmalltalkBalloon.jpg");
	}
	/**
	 * Answer test image.
	 * 
	 * @return sra.smalltalk.StImage
	 */
	protected static StImage _TestImage() {
		int width = 50;
		int height = 50;
		int index = 0;
		StImage anImage = new StImage(width * 3, height * 3);
		Color[] colors = new Color[9];
		colors[0] = Color.black;
		colors[1] = Color.red;
		colors[2] = Color.green;
		colors[3] = Color.blue;
		colors[4] = Color.white;
		colors[5] = Color.yellow;
		colors[6] = Color.cyan;
		colors[7] = Color.magenta;
		colors[8] = Color.gray;
		for (int y = 0; y < 3; y++) {
			for (int x = 0; x < 3; x++) {
				StImage pattern = new StImage(16, 16);
				int value = colors[index].getRGB();
				for (int x2 = 0; x2 < pattern.width(); x2++) {
					for (int y2 = 0; y2 < pattern.height(); y2++) {
						pattern.setPixel(x2, y2, value);
					}
				}
				anImage.tile_from_in_rule_(new Rectangle(width * x + 1, height * y + 1, width - 2, height - 2), new Point(0, 0), pattern, StImage.Over);
				index = index + 1;
			}
		}
		return anImage;
	}
	/**
	 * Example: Copy image onto another image
	 *
	 * @return boolean.
	 */
	public static boolean Example01() {
		StImage source = new IsrColorChoiceSBH().plate();
		StImage destination = ((StImage) new IsrColorChoiceHBS().plate().copy());
		Rectangle area = new Rectangle(25,25,55,55);
		destination.copy_from_in_rule_(area,new Point(0,0),source,StImage.Over);
		destination._display();
		return true;
	}
	/**
	 * Example: Tile image onto another image
	 *
	 * @return boolean.
	 */
	public static boolean Example02() {
		StImage destination = new StImage(IsrImageProcessorTestExamples._ImageSmalltalkBalloon());
		StImage source = new StImage(IsrCursors.StopCursorImage());
		Rectangle area = new Rectangle(10, 10, 30, 30);
		destination.tile_from_in_rule_(area, new Point(0, 0), source, StImage.Over);
		destination._display();
		return true;
	}
	/**
	 * Example: Scaled image
	 *
	 * @return boolean.
	 */
	public static boolean Example03() {
		StImage source = new IsrColorChoiceSBH().plate();
		StImage result = IsrImageProcessor.Scale_factor_(source,new Isr2dPoint(2.5,2));
		result._display();
		return true;
	}
	/**
	 * Example: Scaled b/w image
	 *
	 * @return boolean.
	 */
	public static boolean Example04() {
		StImage source = new StImage(IsrSystem.LogoImage());
		StImage result = IsrImageProcessor.Scale_factor_(source,new Isr2dPoint(2.5,2));
		result._display();
		return true;
	}
	/**
	 * Example: get stippled image
	 *
	 * @return boolean.
	 */
	public static boolean Example05() {
		StImage destination = new StImage(512, 512);
		for (int index = 0; index < 66; index++) {
			Rectangle area = new Rectangle(index % 8 * 24, index / 8 * 24, 16, 16);
			destination.copy_from_in_rule_(area, new Point(0, 0), new StImage(IsrImageProcessor.MaskAt_(index)), StImage.Over);
		}
		destination._display();
		return true;
	}
	/**
	 * Example: get stippled value
	 *
	 * @return boolean.
	 */
	public static boolean Example06() {
		System.out.print(IsrImageProcessor.Stipple_(0.5));
		return true;
	}
	/**
	 * Example: Fill image onto color.pink
	 *
	 * @return boolean.
	 */
	public static boolean Example07() {
		StImage anImage;
		anImage = IsrImageProcessorTestExamples._TestImage();
		IsrImageProcessor.Fill_rectangle_color_(anImage, new Rectangle(20, 20, 60, 60), Color.pink);
		anImage._display();
		anImage = IsrImageProcessorTestExamples._TestImage();
		IsrImageProcessor.Fill_seed_color_(anImage, new Point(75, 75), Color.pink);
		anImage._display();
		return true;
	}
	/**
	 * Example: Color histogram image
	 *
	 * @return boolean.
	 */
	public static boolean Example08() {
		StImage anImage = new StImage(IsrImageProcessorTestExamples._ImageSmalltalkBalloon());
		Object[] histogram = IsrImageProcessor.ColorHistogram_(anImage);
		StImage image = new StImage(401, histogram.length * 4);
		for (int i = 0; i < histogram.length; i++) {
			int width = (int) (((Double) ((StAssociation) histogram[i]).value()).doubleValue() * 400);
			if (width == 0) {
				width = 1;
			}
			Rectangle box = new Rectangle(1, i * 4, width, 3);
			Color color = (Color) (((StAssociation) histogram[i]).key());
			image = IsrImageProcessor.Fill_rectangle_color_(image, box, color);
		}
		image._display();
		return true;
	}
	/**
	 * Example: icon image
	 *
	 * @return boolean.
	 */
	public static boolean Example09() {
		StImage anImage = new StImage(IsrImageProcessorTestExamples._ImageSmalltalkBalloon());
		IsrImageProcessor.Icon_size_(anImage, 64)._display();
		((StImage) IsrImageProcessor.Icon2_size_(anImage, 64).firstElement())._display();
		((StImage) IsrImageProcessor.Icon3_size_(anImage, 64).firstElement())._display();
		return true;
	}
	/**
	 * Example: x spectrum image
	 *
	 * @return boolean.
	 */
	public static boolean Example10() {
		StImage anImage = new StImage(IsrImageProcessorTestExamples._ImageSmalltalkBalloon());
		Object[] xspectrum = IsrImageProcessor.XSpectrum_(anImage);
		StImage image = new StImage(xspectrum.length, 100);
		for (int i = 0; i < xspectrum.length; i++) {
			int x = ((Integer) ((StAssociation) xspectrum[i]).key()).intValue();
			int value = (int) (((Double) ((StAssociation) xspectrum[i]).value()).doubleValue() * 100);
			if (value == 0) {
				value = 1;
			}
			Rectangle box = new Rectangle(x, 0, 1, value);
			image = IsrImageProcessor.Fill_rectangle_color_(image, box, Color.black);
		}
		image._display();
		return true;
	}
	/**
	 * Example: y spectrum image
	 *
	 * @return boolean.
	 */
	public static boolean Example11() {
		StImage anImage = new StImage(IsrImageProcessorTestExamples._ImageSmalltalkBalloon());
		Object[] yspectrum = IsrImageProcessor.YSpectrum_(anImage);
		StImage image = new StImage(100, yspectrum.length);
		for (int i = 0; i < yspectrum.length; i++) {
			int x = ((Integer) ((StAssociation) yspectrum[i]).key()).intValue();
			int value = (int) (((Double) ((StAssociation) yspectrum[i]).value()).doubleValue() * 100);
			if (value == 0) {
				value = 1;
			}
			Rectangle box = new Rectangle(0, x, value, 1);
			image = IsrImageProcessor.Fill_rectangle_color_(image, box, Color.black);
		}
		image._display();
		return true;
	}
	/**
	 * Example: 256 color palette.
	 *
	 * @return boolean.
	 */
	public static boolean Example12() {
		IsrImageProcessor.ColorPalette256Image_(new Point(8, 8))._display();
		IsrImageProcessor.GrayPalette256Image_(new Point(8, 8))._display();
		return true;
	}
	/**
	 * Example: Shape image onto color.green
	 *
	 * @return boolean.
	 */
	public static boolean Example13() {
		StImage anImage = IsrImageProcessorTestExamples._TestImage();
		anImage = IsrImageProcessor.Shape_color_(anImage, Color.green);
		anImage._display();
		return true;
	}
	/**
	 * Example: image border following
	 *
	 * @return boolean.
	 */
	public static boolean Example14() {
		Graphics gc = null;
		StImage anImage = new StImage(IsrSystem.LogoImage());
		StAssociation[] collection = IsrImageProcessor.BorderFollowing_(anImage);
		StImage stImage = new StImage(anImage.width() + 2, anImage.height() + 2);
		Image newImage = stImage.image();
		try {
			gc = newImage.getGraphics();
			if (gc instanceof Graphics2D) {
				((Graphics2D)gc).setBackground(SystemColor.window);
			}
			gc.setColor(Color.black);
			for (int i = 0; i < collection.length; i++) {
				StAssociation assoc = collection[i];
				Point[] points = (Point[]) assoc.value();
				int[] xPoints = new int[points.length];
				int[] yPoints = new int[points.length];
				for (int j = 0; j < points.length; j++) {
					xPoints[j] = points[j].x;
					yPoints[j] = points[j].y;
				}
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}
			(new StImage(newImage))._display();
		} finally {
			gc.dispose();
			newImage.flush();
		}
		return true;
	}
	/**
	 * Example: image trace border
	 *
	 * @return boolean.
	 */
	public static boolean Example15() {
		Graphics gc = null;
		StImage anImage = new StImage(IsrSystem.LogoImage());
		Object[] collection = IsrImageProcessor.TraceBorder_(anImage);
		StImage stImage = new StImage(anImage.width() + 2, anImage.height() + 2);
		Image newImage = stImage.image();
		try {
			gc = newImage.getGraphics();
			if (gc instanceof Graphics2D) {
				((Graphics2D)gc).setBackground(SystemColor.window);
			}
			gc.setColor(Color.black);
			for (int i = 0; i < collection.length; i++) {
				Point[] points = (Point[]) collection[i];
				int[] xPoints = new int[points.length];
				int[] yPoints = new int[points.length];
				for (int j = 0; j < points.length; j++) {
					xPoints[j] = points[j].x;
					yPoints[j] = points[j].y;
				}
				gc.drawPolyline(xPoints, yPoints, xPoints.length);
			}
			stImage._display();
		} finally {
			gc.dispose();
			newImage.flush();
		}
		return true;
	}
	/**
	 * Example: thinning image
	 *
	 * @return boolean.
	 */
	public static boolean Example16() {
		StImage anImage = new StImage(IsrSystem.LogoImage());
		anImage = IsrImageProcessor.MakeThin_(anImage);
		anImage._display();
		return true;
	}
	/**
	 * Example: image convert to palette
	 *
	 * @return boolean.
	 */
	public static boolean Example17() {
		StImage anImage = new StImage(IsrImageProcessorTestExamples._ImageSmalltalkBalloon());
		anImage._display();
		StImage newImage = anImage._convertToPalette_RenderedByNearistPaint(IsrImageProcessor.GrayPalette256());
		newImage._display();
		newImage = anImage._convertToPalette_RenderedByErrorDiffusion(IsrImageProcessor.GrayPalette256());
		newImage._display();
		return true;
	}
	/**
	 * Execute all examples.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		new IsrImageProcessorTestExamples();
	}
}
