package sra.isr.interfaces.area;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.interfaces.cursors.IsrCursors;

/**
 * IsrAreaController class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2000/02/16 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrAreaController.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrAreaController extends StController implements MouseListener, MouseMotionListener {
	/**
	 * Add listeners.
	 * 
	 * @param newView sra.mvctest.StView
	 */
	protected void buildListener(StView newView) {
		Component aView = (Component) newView;
		aView.addMouseListener(this);
		aView.addMouseMotionListener(this);
	}
	/**
	 * Answer this model.
	 *
	 * @return sra.isr.interfaces.area.IsrAreaModel
	 */
	protected IsrAreaModel getModel() {
		return (IsrAreaModel) this.model();
	}
	/**
	 * Answer this view.
	 * 
	 * @return sra.isr.interfaces.area.IsrSpotView interface
	 */
	protected IsrAreaView getView() {
		return (IsrAreaView) this.view();
	}
	/**
	 * The General dragging action.
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseDragged(MouseEvent event) {
		if (this.isMenuButtonPressed_(event))
			return;
		IsrAreaView view = this.getView();
		if (view.plateArea().contains(event.getPoint())) {
			this.mousePressed(event);
		}
	}
	/**
	 * Invoked when the mouse button has been moved on a component
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseMoved(MouseEvent event) {
		IsrAreaView view = this.getView();
		if (view.plateArea().contains(event.getPoint())) {
			((Component) view).setCursor(IsrCursors.CrossCursor());
		} else {
			((Component) view).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}
	/**
	 * Invoked when a mouse button has been pressed. 
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent event) {
		if (this.isMenuButtonPressed_(event))
			return;
		Point point = event.getPoint();
		IsrAreaView view = this.getView();
		if (view.plateArea().contains(point)) {
			double x = ((double) (point.x - view.plateArea().getLocation().x)) / (view.plateArea().width - 1.0);
			double y = ((double) (point.y - view.plateArea().getLocation().y)) / (view.plateArea().height - 1.0);
			Isr2dPoint pointXy = new Isr2dPoint(x, y);
			((IsrAreaModel) this.model()).xy_(pointXy);
		}
	}
}
