package sra.isr.interfaces.area;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import sra.isr.geometry.basic.*;

/**
 * IsrAreaJPanel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/06/23 (by Mitsuhiro Asada)
 * 	@updated   2001/06/15 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrAreaJPanel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrAreaJPanel extends JPanel {
	private IsrAreaModel areaModel = null;
	private IsrAreaView areaView = null;
	private JCheckBox areaSwitch = null;
	/**
	 * IsrAreaViewSwing constructor.
	 */
	public IsrAreaJPanel() {
		super();
		initialize();
	}
	/**
	 * IsrAreaViewSwing constructor.
	 * 
	 * @param layout java.awt.LayoutManager
	 */
	public IsrAreaJPanel(LayoutManager layout) {
		super(layout);
		initialize();
	}
	/**
	 * IsrAreaViewSwing constructor.
	 * 
	 * @param layout java.awt.LayoutManager
	 * @param isDoubleBuffered boolean
	 */
	public IsrAreaJPanel(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
		initialize();
	}
	/**
	 * IsrAreaViewSwing constructor.
	 * 
	 * @param isDoubleBuffered boolean
	 */
	public IsrAreaJPanel(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
		initialize();
	}
	/**
	 * connAreaViewSetTarget:  (AreaModel.this <--> AreaView.model)
	 */
	private void _connAreaViewSetTarget() {
		try {
			this.getAreaView().setModel(this.getAreaModel());
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Called whenever the part throws an exception.
	 * 
	 * @param exception java.lang.Throwable
	 */
	private void _handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}
	/**
	 * Initializes connections
	 * 
	 * @exception java.lang.Exception
	 */
	private void _initConnections() throws Exception {
		this._connAreaViewSetTarget();
	}
	/**
	 * Return the area value.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint
	 */
	public Isr2dPoint area() {
		return this.xy();
	}
	/**
	 * Set the area value.
	 *
	 * @param point sra.isr.geometry.basic.Isr2dPoint
	 */
	public void area_(Isr2dPoint point) {
		this.xy_(point);
	}
	/**
	 * Return the area switch.
	 *
	 * @return boolean
	 */
	public boolean areaSwitch() {
		return this.getAreaSwitch().isSelected();
	}
	/**
	 * Set the area switch.
	 *
	 * @param aBoolean boolean
	 */
	public void areaSwitch_(boolean aBoolean) {
		this.getAreaSwitch().setSelected(aBoolean);
	}
	/**
	 * Return the AreaModel property value.
	 *
	 * @return sra.isr.interfaces.area.IsrAreaModel
	 */
	public IsrAreaModel getAreaModel() {
		if (areaModel == null) {
			try {
				areaModel = new IsrAreaModel();
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return areaModel;
	}
	/**
	 * Return the AreaSwitch property value.
	 * 
	 * @return javax.swing.JCheckBox
	 */
	public JCheckBox getAreaSwitch() {
		if (areaSwitch == null) {
			try {
				areaSwitch = new JCheckBox();
				areaSwitch.setName("AreaSwitch");
				areaSwitch.setText("area");
				areaSwitch.setBounds(2, 2, 123, 25);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return areaSwitch;
	}
	/**
	 * Return the AreaView property value.
	 * 
	 * @return sra.isr.interfaces.area.IsrAreaView
	 */
	protected IsrAreaView getAreaView() {
		if (areaView == null) {
			try {
				areaView = new IsrAreaViewSwing();
				((Component) areaView).setName("AreaViewSwing");
				((Component) areaView).setBounds(2, 26, 123, 123);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return areaView;
	}
	/**
	 * Initialize the class.
	 */
	private void initialize() {
		try {
			this.setName("IsrAreaJPanel");
			this.setLayout(null);
			this.setSize(127, 151);
			this.add(this.getAreaSwitch(), this.getAreaSwitch().getName());
			this.add((Component) this.getAreaView(), ((Component) this.getAreaView()).getName());
			this._initConnections();
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Set the AreaModel property value.
	 *
	 * @param model sra.isr.interfaces.area.IsrAreaModel
	 */
	public void setAreaModel(IsrAreaModel model) {
		areaModel = model;
		this.getAreaView().setModel(model);
	}
	/**
	 * Return the area value.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint
	 */
	public Isr2dPoint xy() {
		return this.getAreaModel().xy();
	}
	/**
	 * Set the area value.
	 *
	 * @param point sra.isr.geometry.basic.Isr2dPoint
	 */
	public void xy_(Isr2dPoint point) {
		this.getAreaModel().xy_(point);
	}
}
