package sra.isr.interfaces.button;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;

/**
 * IsrButtonController class
 * 
 * 	@author    He Weijie
 * 	@created   1998/08/27 (by He Weijie)
 * 	@updated   1999/11/12 (by MATSUDA Ryouichi)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrButtonController.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrButtonController extends StController implements MouseListener, Runnable {
	transient protected boolean mousePress;
	transient protected boolean mouseEnter;
	transient protected Thread repeatThread;
	/**
	 * Add listeners.
	 * 
	 * @param newView sra.smalltalk.StView
	 */
	protected void buildListener(StView newView) {
		final Component aView = (Component) newView;
		aView.addMouseListener(this);
	}
	/**
	 * Answer this model.
	 * 
	 * @return IsrButtonModel
	 */
	protected IsrButtonModel getModel() {
		return (IsrButtonModel) this.model();
	}
	/**
	 * Answer this view.
	 * 
	 * @return IsrButtonView interface
	 */
	protected IsrButtonView getView() {
		return (IsrButtonView) this.view();
	}
	/**
	 * Invoked when the mouse enters.
	 *
	 * @param e java.awt.event.MouseEvent 
	 */
	public void mouseEntered(MouseEvent e) {
		final IsrButtonView view = this.getView();
		mouseEnter = true;
		if (mousePress) {
			Graphics gc = ((Component) view).getGraphics();
			view.displayTrueOn_(gc);
			gc.setColor(Color.black);
			Dimension size = ((Component) view).getSize();
			gc.drawRect(0, 0, size.width - 1, size.height - 1);
			gc.dispose();
		}
	}
	/**
	 * Invoked when the mouse exits.
	 *
	 * @param e java.awt.event.MouseEvent 
	 */
	public void mouseExited(MouseEvent e) {
		final IsrButtonView view = this.getView();
		mouseEnter = false;
		if (mousePress) {
			Graphics gc = ((Component) view).getGraphics();
			if (this.getModel().value()) {
				view.displayTrueOn_(gc);
			} else {
				view.displayFalseOn_(gc);
			}
			gc.dispose();
		}
	}
	/**
	 * Invoked when a mouse button has been pressed on a component.
	 *
	 * @param e java.awt.event.MouseEvent 
	 */
	public void mousePressed(MouseEvent e) {
		final IsrButtonView view = this.getView();
		mousePress = true;
		mouseEnter = true;
		Graphics gc = ((Component) view).getGraphics();
		view.displayTrueOn_(gc);
		gc.setColor(Color.black);
		Dimension size = ((Component) view).getSize();
		gc.drawRect(0, 0, size.width - 1, size.height - 1);
		gc.dispose();
		if (this.getModel().repeatAction()) {
			repeatThread = new Thread(this);
			repeatThread.start();
		}
	}
	/**
	 * Invoked when a mouse button has been released on a component.
	 *
	 * @param e java.awt.event.MouseEvent 
	 */
	public void mouseReleased(MouseEvent e) {
		final IsrButtonView view = this.getView();
		final IsrButtonModel model = this.getModel();
		if (mousePress) {
			Graphics gc = ((Component) view).getGraphics();
			if (model.value()) {
				view.displayTrueOn_(gc);
			} else {
				view.displayFalseOn_(gc);
			}
			gc.dispose();
			mousePress = false;
		}
		if (repeatThread != null) {
			repeatThread = null;
		}
		if (mouseEnter) {
			StBlockClosure action = model.action();
			if (action != null) {
				switch (action.numArgs()) {
					case 0:
						action.value();
						break;
					case 1:
						action.value_(model);
						break;
					case 2:
						action.value_value_(model, e);
						break;
				}
			}
			mouseEnter = false;
		}
	}
	/**
	 * Start the repeat thread.
	 */
	public void run() {
		final IsrButtonModel model = this.getModel();
		try {
			Thread.sleep(500);
		} catch (InterruptedException e) {
		};
		StBlockClosure action = model.action();
		while (mousePress) {
				switch (action.numArgs()) {
					case 0:
						action.value();
						break;
					case 1:
						action.value_(model);
						break;
					case 2:
						action.value_value_(model, null);
						break;
				}
			Thread.yield();
		}
	}
}
