package sra.isr.interfaces.button;

import java.awt.*;
import sra.smalltalk.*;

/**
 * IsrButtonModel class
 * 
 * 	@author    He Weijie
 * 	@created   1998/08/13 (by He Weijie)
 * 	@updated   1999/11/12 (by MATSUDA Ryouichi)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrButtonModel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrButtonModel extends StApplicationModel {
	/**
	 * A boolean value as a button model.
	 */
	protected boolean buttonValue;

	/**
	 * An object which will be displayed on an associated view.
	 */
	protected Object displayObject;

	/**
	 * An action which will be evaluated when a button is pressed.
	 */
	protected StBlockClosure buttonAction;

	/**
	 * A flag to allow a repeat action or not.
	 */
	protected boolean repeatAction;

	/**
	 * A boolean value as a check button.
	 */
	protected boolean checkButton;
	/**
	 * Return the current button action.
	 * If not assigned, create a fake action and return it.
	 *
	 * @return sra.smalltalk.StBlockClosure
	 */
	public StBlockClosure action() {
		return buttonAction;
	}
	/**
	 * Set the action.
	 * 
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void action_(StBlockClosure aBlock) {
		buttonAction = aBlock;
	}
	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		IsrButtonView aView;
		if (GetDefaultViewMode() == VIEW_AWT) {
			aView = new IsrButtonViewAwt();
		} else {
			aView = new IsrButtonViewSwing();
		}
		aView.model_(this);
		aView.adjustSize();
		return aView;
	}
	/**
	 * Just return the repeatAction.
	 * 
	 * @return boolean
	 */
	public boolean repeatAction() {
		return repeatAction;
	}
	/**
	 * Set the repeatAction.
	 * 
	 * @param aBoolean boolean
	 */
	public void repeatAction_(boolean aBoolean) {
		repeatAction = aBoolean;
	}
	/**
	 * Just return the current button value.
	 * 
	 * @return boolean
	 */
	public boolean value() {
		return buttonValue;
	}
	/**
	 * Set the new button value.
	 * 
	 * If the value is changed, announce it.
	 *
	 * @param aBoolean boolean
	 */
	public void value_(boolean aBoolean) {
		if (aBoolean != buttonValue) {
			buttonValue = aBoolean;
			this.changed_($("value"));
		}
	}
	/**
	 * Create a new instance of IsrButtonModel.
	 *
	 * @return sra.jun.goodies.button.JunButtonModel
	 * @param aClass java.lang.Class
	 * @param aBoolean boolean
	 * @param anImage java.awt.Image
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public static IsrButtonModel Value_visual_action_(Class aClass, boolean aBoolean, Image anImage, StBlockClosure aBlock) {
		IsrButtonModel buttonModel = (IsrButtonModel) _New(aClass);
		buttonModel.value_(aBoolean);
		buttonModel.visual_(anImage);
		buttonModel.action_(aBlock);
		return buttonModel;
	}
	/**
	 * Create a new instance of IsrButtonModel.
	 *
	 * @return sra.jun.goodies.button.JunButtonModel
	 * @param aClass java.lang.Class
	 * @param aBoolean boolean
	 * @param anObject java.lang.Object
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public static IsrButtonModel Value_visual_action_(Class aClass, boolean aBoolean, Object anObject, StBlockClosure aBlock) {
		IsrButtonModel buttonModel = (IsrButtonModel) _New(aClass);
		buttonModel.value_(aBoolean);
		buttonModel.visual_(anObject);
		buttonModel.action_(aBlock);
		return buttonModel;
	}
	/**
	 * Create a new instance of IsrButtonModel.
	 *
	 * @return sra.jun.goodies.button.JunButtonModel
	 * @param aBoolean boolean
	 * @param anImage java.awt.Image
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public final static IsrButtonModel Value_visual_action_(boolean aBoolean, Image anImage, StBlockClosure aBlock) {
		return Value_visual_action_(IsrButtonModel.class, aBoolean, anImage, aBlock);
	}
	/**
	 * Create a new instance of IsrButtonModel.
	 *
	 * @return sra.jun.goodies.button.JunButtonModel
	 * @param aBoolean boolean
	 * @param anObject java.lang.Object
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public final static IsrButtonModel Value_visual_action_(boolean aBoolean, Object anObject, StBlockClosure aBlock) {
		return Value_visual_action_(IsrButtonModel.class, aBoolean, anObject, aBlock);
	}
	/**
	 * Return the current display object. If not assigned yet, set a
	 * null string as a fake display object and return it.
	 *
	 * @return java.lang.Object
	 */
	public Object visual() {
		if (displayObject == null) {
			this.visual_(new String());
		}
		return displayObject;
	}
	/**
	 * Set a string coverted from an object as a visual object.
	 * If the visual object is changed, announce it.
	 *
	 * @param anObject java.lang.Object
	 */
	public void visual_(Object anObject) {
		if (anObject != displayObject) {
			displayObject = anObject;
			this.changed_($("visual"));
		}
	}
}
