package sra.isr.interfaces.button;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;

/**
 * IsrButtonViewSwing class
 * 
 * 	@author    He Weijie
 * 	@created   1998/08/27 (by He Weijie)
 * 	@updated   1999/11/12 (by MATSUDA Ryouichi)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrButtonViewSwing.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrButtonViewSwing extends StViewJPanel implements IsrButtonView {
	/**
	 * Display a string on a graphics at a center.
	 *
	 * @param aString java.lang.String
	 * @param aGraphics java.awt.Graphics
	 */
	protected void _displayObject_on_(Object anObject, Graphics aGraphics) {
		if (anObject instanceof Image) {
			Image anImage = (Image) anObject;
			int width = anImage.getWidth(null);
			int height = anImage.getHeight(null);
			int x = (this.getSize().width - width) / 2;
			int y = (this.getSize().height - height) / 2;
			aGraphics.drawImage(anImage, x, y, null);
		} else {
			String aString;
			if (anObject instanceof String) {
				aString = (String) anObject;
			} else {
				aString = anObject.toString();
			}
			FontMetrics fontMetrics = aGraphics.getFontMetrics();
			int width = fontMetrics.stringWidth(aString);
			int height = fontMetrics.getAscent();
			int x = (this.getSize().width - width) / 2;
			int y = (this.getSize().height + height) / 2;
			aGraphics.setColor(Color.black);
			aGraphics.drawString(aString, x, y);
		}
	}
	/**
	 * Answer the preferred size of the object.
	 *
	 * @return java.awt.Dimension
	 * @param anObject java.lang.Object
	 */
	protected Dimension _getPreferredSizeOf(Object anObject) {
		if (anObject instanceof Image) {
			Image anImage = (Image) anObject;
			return new Dimension(anImage.getWidth(null), anImage.getHeight(null));
		} else {
			String aString;
			if (anObject instanceof String) {
				aString = (String) anObject;
			} else {
				aString = anObject.toString();
			}
			Point extent = SystemResourceSupport._getStringExtentFor_(aString, null);
			return new Dimension(extent.x, extent.y);
		}
	}
	/**
	 * Adjust size.
	 */
	public void adjustSize() {
		Dimension size = this._getPreferredSizeOf(getModel().visual());
		this.setPreferredSize(size);
		this.setSize(size);
	}
	/**
	 * Answer my default controller.
	 *
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new IsrButtonController();
	}
	/**
	 * Answer a default model.
	 *
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		IsrButtonModel aModel = new IsrButtonModel();
		aModel.visual_("IsrButtonModel");
		return aModel;
	}
	/**
	 * Update the view when the value of the model is false.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayFalseOn_(Graphics aGraphics) {
		Rectangle box = new Rectangle(0, 0, this.getSize().width, this.getSize().height);
		aGraphics.clipRect(box.x, box.y, box.width, box.height);
		aGraphics.setColor(new Color((float) 0.7, (float) 0.7, (float) 0.7));
		aGraphics.fillRect(box.x, box.y, box.width, box.height);
		Object displayObject = this.getModel().visual();
		this._displayObject_on_(displayObject, aGraphics);
		aGraphics.setColor(new Color((float) 0.8, (float) 0.8, (float) 0.8));
		aGraphics.drawLine(box.x, box.y, box.x + box.width - 1, box.y);
		aGraphics.drawLine(box.x, box.y, box.x, box.y + box.height - 1);
		aGraphics.setColor(new Color((float) 0.2, (float) 0.2, (float) 0.2));
		aGraphics.drawLine(box.x, box.y + box.height - 1, box.x + box.width - 1, box.y + box.height - 1);
		aGraphics.drawLine(box.x + box.width - 1, box.y, box.x + box.width - 1, box.y + box.height - 1);
	}
	/**
	 * Paint. 
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayOn_(Graphics aGraphics) {
		if (this.getModel().value() == true) {
			this.displayTrueOn_(aGraphics);
		} else {
			this.displayFalseOn_(aGraphics);
		}
	}
	/**
	 * Update the view when the value of the model is true.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayTrueOn_(Graphics aGraphics) {
		Rectangle box = new Rectangle(0, 0, this.getSize().width, this.getSize().height);
		aGraphics.clipRect(box.x, box.y, box.width, box.height);
		aGraphics.setColor(new Color((float) 0.3, (float) 0.3, (float) 0.3));
		aGraphics.fillRect(box.x, box.y, box.width, box.height);
		Object displayObject = this.getModel().visual();
		this._displayObject_on_(displayObject, aGraphics);
		aGraphics.setColor(new Color((float) 0.2, (float) 0.2, (float) 0.2));
		aGraphics.drawLine(box.x, box.y, box.x + box.width - 1, box.y);
		aGraphics.drawLine(box.x, box.y, box.x, box.y + box.height - 1);
		aGraphics.setColor(new Color((float) 0.8, (float) 0.8, (float) 0.8));
		aGraphics.drawLine(box.x, box.y + box.height - 1, box.x + box.width - 1, box.y + box.height - 1);
		aGraphics.drawLine(box.x + box.width - 1, box.y, box.x + box.width - 1, box.y + box.height - 1);
	}
	/**
	 * Model property accessing.
	 *
	 * @return sra.jun.goodies.JunButtonModel
	 */
	public IsrButtonModel getModel() {
		return (IsrButtonModel) this.model();
	}
	/**
	 * Value property accessing.
	 *
	 * @return boolean
	 */
	public boolean getValue() {
		return getModel().value();
	}
	/**
	 *VVisual property accessing.
	 *
	 * @return java.lang.Object
	 */
	public Object getVisual() {
		return this.getModel().visual();
	}
	/**
	 * Model property accessing.
	 *
	 * @param aModel sra.isr.interfaces.button.IsrButtonModel
	 */
	public void setModel(IsrButtonModel aModel) {
		this.model_(aModel);
	}
	/**
	 * Value property accessing.
	 *
	 * @param aValue boolean
	 */
	public void setValue(boolean aValue) {
		this.getModel().value_(aValue);
	}
	/**
	 * Visual property accessing.
	 *
	 * @param anObject java.lang.Object
	 */
	public void setVisual(Object anObject) {
		this.getModel().visual_(anObject);
	}
	/**
	 * Update
	 * 
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("value")) {
			this.repaint();
			this.firePropertyChange("value", null, new Boolean(this.getValue()));
		} else if (evt.getAspect() == $("visual")) {
			this.repaint();
			this.firePropertyChange("visual", null, this.getVisual());
		} else {
			super.update_(evt);
		}
	}
}
