package sra.isr.interfaces.colors;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.interfaces.cursors.IsrCursors;

/**
 * IsrColorChoiceController class
 * 
 * 	@author    Hirotsugu Kondo
 * 	@created   1998/11/05 (by Hirotsugu Kondo)
 * 	@updated   1999/08/31 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrColorChoiceController.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrColorChoiceController extends StController implements MouseListener, MouseMotionListener {
	/**
	 * Add listeners.
	 * 
	 * @param aView sra.smalltalk.StView
	 */
	protected void buildListener(StView newView) {
		Component aView = (Component) newView;
		aView.addMouseListener(this);
		aView.addMouseMotionListener(this);
	}
	/**
	 * Invoked when the mouse button has been moved.
	 * 
	 * @param e java.awt.event.MouseEvent 
	 */
	public void mouseDragged(MouseEvent event) {
		Point point = event.getPoint();
		IsrColorChoiceView view = ((IsrColorChoiceView) this.view());
		if ((view.plateArea().contains(point)) || (view.barArea().contains(point))) {
			this.mousePressed(event);
		}
	}
	/**
	 * Tempolary cursor set method. Not called now.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseMoved(MouseEvent event) {
		Point point = event.getPoint();
		IsrColorChoiceView view = ((IsrColorChoiceView) this.view());
		if ((view.plateArea().contains(point)) || (view.barArea().contains(point)) || (view.hueArea().contains(point)) || (view.saturationArea().contains(point)) || (view.brightnessArea().contains(point))) {
			((Component) view).setCursor(IsrCursors.CrossCursor());
		} else {
			((Component) view).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}
	/**
	 * Invoked when a mouse button has been pressed. 
	 * press button to set color.
	 * 
	 * @param event java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent event) {
		if (this.isMenuButtonPressed_(event)) {
			return;
		}
		Point point = event.getPoint();
		IsrColorChoiceView view = ((IsrColorChoiceView) this.view());
		Point globalOrigin = event.getComponent().getParent().getLocation();
		if (view.plateArea().contains(point)) {
			double x = ((double) (point.x - view.plateArea().getLocation().x)) / (view.plateArea().width - 1.0);
			double y = ((double) (point.y - view.plateArea().getLocation().y)) / (view.plateArea().height - 1.0);
			Isr2dPoint jPoint = new Isr2dPoint(x, y);
			((IsrColorChoiceModel) this.model()).xy_(jPoint);
		}
		if (view.barArea().contains(point)) {
			double z = ((double) (point.y - view.barArea().getLocation().y)) / (view.barArea().height - 1.0);
			((IsrColorChoiceModel) this.model()).z_(z);
		}
		if (view.hueArea().contains(point)) {
			String result = StDialog.Request_("hue?", view.hueString(), new Point(globalOrigin.x + point.x, globalOrigin.y + point.y));
			if (result != null) {
				try {
					double resultValue = new Double(result).doubleValue();
					if ((resultValue >= 0.0) && (resultValue <= 1.0)) {
						((IsrColorChoiceModel) this.model()).hue_(resultValue);
					}
				} catch (NumberFormatException e) {
				}
			}
		}
		if (view.saturationArea().contains(point)) {
			String result = StDialog.Request_("saturation?", view.saturationString(), new Point(globalOrigin.x + point.x, globalOrigin.y + point.y));
			if (result != null) {
				try {
					double resultValue = new Double(result).doubleValue();
					if ((resultValue >= 0.0) && (resultValue <= 1.0)) {
						((IsrColorChoiceModel) this.model()).saturation_(resultValue);
					}
				} catch (NumberFormatException e) {
				}
			}
		}
		if (view.brightnessArea().contains(point)) {
			String result = StDialog.Request_("brightness?", view.brightnessString(), new Point(globalOrigin.x + point.x, globalOrigin.y + point.y));
			if (result != null) {
				try {
					double resultValue = new Double(result).doubleValue();
					if ((resultValue >= 0.0) && (resultValue <= 1.0)) {
						((IsrColorChoiceModel) this.model()).brightness_(resultValue);
					}
				} catch (NumberFormatException e) {
				}
			}
		}
		return;
	}
}
