package sra.isr.interfaces.colors;

import java.awt.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;

/**
 * IsrColorChoiceHBS class
 * 
 * 	@author    Hirotsugu Kondo
 * 	@created   1998/11/05 (by Hirotsugu Kondo)
 * 	@updated   1999/08/31 (by m-asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrColorChoiceHBS.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrColorChoiceHBS extends IsrColorChoiceModel {
	private static StImage[] plates;
	private static StImage[] bars;
	
	static {
		plates = _InitializePlates();
		bars = _InitializeBars();
	}
	/**
	 * Answer the image of color bars.
	 *
	 * @return sra.smalltalk.StImage[]
	 */
	static StImage[] _InitializeBars() {
		int gap = _Gap();
		int interval = _Intervals();
		int ySize = interval / gap;
		int zSize = interval / gap;
		int width = _barArea().width;
		StImage[] images = new StImage[zSize];
		for (int z = 0; z < zSize; z++) {
			int[] pixels = new int[interval * width];
			for (int y = 0; y < ySize; y++) {
				double hue = z * 1.0 / zSize;
				double saturation = 1 - (y * 1.0 / ySize);
				double brightness = 1.0;
				Color color = Color.getHSBColor((float) hue, (float) saturation, (float) brightness);
				for (int dy = 0; dy < gap; dy++) {
					for (int dx = 0; dx < width; dx++) {
						pixels[ (y * gap + dy) * width + dx] = color.getRGB();
					}
				}
			}
			images[z] = new StImage(width, interval, pixels);
		}
		return images;
	}
	/**
	 * Answer the image of color plate.
	 *
	 * @return sra.smalltalk.StImage[]
	 */
	static StImage[] _InitializePlates() {
		int gap = _Gap();
		int interval = _Intervals();
		int xSize = interval / gap;
		int ySize = interval / gap;
		int zSize = interval / gap;
		StImage[] images = new StImage[zSize];
		for (int z = 0; z < zSize; z++) {
			int[] pixels = new int[interval * interval];
			for (int y = 0; y < ySize; y++) {
				for (int x = 0; x < xSize; x++) {
					double hue = x * 1.0 / xSize;
					double saturation = 1 - (z * 1.0 / zSize);
					double brightness = 1 - (y * 1.0 / ySize);
					Color color = Color.getHSBColor((float) hue, (float) saturation, (float) brightness);
					for (int dy = 0; dy < gap; dy++) {
						for (int dx = 0; dx < gap; dx++) {
							pixels[ (y * gap + dy) * interval + x * gap + dx] = color.getRGB();
						}
					}
				}
			}
			images[z] = new StImage(interval, interval, pixels);
		}
		return images;
	}
	/**
	 * Answer the image of color bars.
	 *
	 * @return sra.smalltalk.StImage[]
	 */
	public StImage[] bars() {
		StImage[] images = this.bars;
		return images;
	}
	/**
	 * Answer the brightness of color.
	 *
	 * @return double
	 */
	public double brightness() {
		double value = 1 -  this.xy().y();
		return value;
	}
	/**
	 * Set the brightness of color.
	 *
	 * @param value double
	 */
	public void brightness_(double value) {
		this.xy_(new Isr2dPoint(this.xy().x(), (1 - value)));
	}
	/**
	 * Receives Color and mapping this.
	 *
	 * @param color java.awt.Color
	 */
	public void color_(Color color) {
		this.z_(1 - (StColorValue._GetSaturation(color)));
		this.xy_(new Isr2dPoint(StColorValue._GetHue(color), (1 - (StColorValue._GetBrightness(color)))));
		this.changed_($("color"));
	}
	/**
	 * Answer the hue of color.
	 *
	 * @return double
	 */
	public double hue() {
		double value = this.xy().x();
		return value;
	}
	/**
	 * Set the hue of color.
	 *
	 * @param value double
	 */
	public void hue_(double value) {
		this.xy_(new Isr2dPoint(value,this.xy().y()));
	}
	/**
	 * Return the plate.
	 *
	 * @return sra.smalltalk.StImage[]
	 */
	public StImage[] plates() {
		StImage[] images = this.plates;
		return images;
	}
	/**
	 * Create the Dialog for IsrColorChoiceModel.
	 *
	 * @return java.awt.Color or null
	 */
	public static Color Request() {
		return Request_(Color.white);
	}
	/**
	 * Create the Dialog for IsrColorChoiceModel with initial value.
	 *
	 * @return java.awt.Color or null
	 * @param colorValue java.awt.Color
	 */
	public static Color Request_(Color colorValue) {
		return _Request_(colorValue, new IsrColorChoiceHBS());
	}
	/**
	 * Answer the saturation of color.
	 *
	 * @return double
	 */
	public double saturation() {
		double value = 1 - this.z();
		return value;
	}
	/**
	 * Set the saturation of color.
	 *
	 * @param value double
	 */
	public void saturation_(double value) {
		this.z_(1 - value);
	}
}
