package sra.isr.interfaces.colors;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * IsrColorChoiceJPanel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/06/23 (by Mitsuhiro Asada)
 * 	@updated   2001/06/15 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrColorChoiceJPanel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrColorChoiceJPanel extends JPanel {
	private IsrColorChoiceModel colorChoiceModel = null;
	private IsrColorChoiceView colorChoiceView = null;
	private JCheckBox colorChoiceSwitch = null;
	/**
	 * IsrColorChoiceJPanel constructor.
	 */
	public IsrColorChoiceJPanel() {
		super();
		initialize();
	}
	/**
	 * IsrColorChoiceJPanel constructor.
	 * 
	 * @param layout java.awt.LayoutManager
	 */
	public IsrColorChoiceJPanel(LayoutManager layout) {
		super(layout);
		initialize();
	}
	/**
	 * IsrColorChoiceJPanel constructor.
	 *
	 * @param layout java.awt.LayoutManager
	 * @param isDoubleBuffered boolean
	 */
	public IsrColorChoiceJPanel(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
		initialize();
	}
	/**
	 * IsrColorChoiceJPanel constructor.
	 * 
	 * @param isDoubleBuffered boolean
	 */
	public IsrColorChoiceJPanel(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
		initialize();
	}
	/**
	 * connColorChoiceViewSetTarget:  (ColorChoiceModel.this <--> ColorChoiceView.model)
	 */
	private void _connColorChoiceViewSetTarget() {
		try {
			this.getColorChoiceView().setModel(this.getColorChoiceModel());
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Called whenever the part throws an exception.
	 * 
	 * @param exception java.lang.Throwable
	 */
	private void _handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}
	/**
	 * Initializes connections
	 * 
	 * @exception java.lang.Exception
	 */
	private void _initConnections() throws java.lang.Exception {
		this._connColorChoiceViewSetTarget();
	}
	/**
	 * Return the choice color.
	 *
	 * @return java.awt.Color
	 */
	public Color color() {
		return this.getColorChoiceModel().color();
	}
	/**
	 * Set the choice color.
	 *
	 * @param color java.awt.Color
	 */
	public void color_(Color color) {
		this.getColorChoiceModel().color_(color);
	}
	/**
	 * Return the color choice switch.
	 *
	 * @return boolean
	 */
	public boolean colorChoiceSwitch() {
		return this.getColorChoiceSwitch().isSelected();
	}
	/**
	 * Set the color choice switch.
	 *
	 * @param aBoolean boolean
	 */
	public void colorChoiceSwitch_(boolean aBoolean) {
		this.getColorChoiceSwitch().setSelected(aBoolean);
	}
	/**
	 * Return the ColorChoiceModel property value.
	 * 
	 * @return sra.isr.interfaces.colors.IsrColorChoiceModel
	 */
	public IsrColorChoiceModel getColorChoiceModel() {
		if (colorChoiceModel == null) {
			try {
				colorChoiceModel = new IsrColorChoiceHSB();
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return colorChoiceModel;
	}
	/**
	 * Return the ColorChoiceSwitch property value.
	 * 
	 * @return javax.swing.JCheckBox
	 */
	public JCheckBox getColorChoiceSwitch() {
		if (colorChoiceSwitch == null) {
			try {
				colorChoiceSwitch = new JCheckBox();
				colorChoiceSwitch.setName("ColorSwitch");
				colorChoiceSwitch.setText("color choice");
				colorChoiceSwitch.setBounds(2, 2, 123, 25);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return colorChoiceSwitch;
	}
	/**
	 * Return the ColorChoiceView property value.
	 * 
	 * @return sra.isr.interfaces.colors.IsrColorChoiceView
	 */
	protected IsrColorChoiceView getColorChoiceView() {
		if (colorChoiceView == null) {
			try {
				colorChoiceView = new IsrColorChoiceViewSwing();
				((Component) colorChoiceView).setName("ColorChoiceViewSwing");
				((Component) colorChoiceView).setBounds(2, 26, 230, 125);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return colorChoiceView;
	}
	/**
	 * Initialize the class.
	 */
	private void initialize() {
		try {
			this.setName("IsrColorChoiceJPanel");
			this.setLayout(null);
			this.setSize(234, 153);
			this.add(this.getColorChoiceSwitch(), this.getColorChoiceSwitch().getName());
			this.add((Component) this.getColorChoiceView(), ((Component) this.getColorChoiceView()).getName());
			this._initConnections();
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Set the ColorChoiceModel property value.
	 * 
	 * @param model sra.isr.interfaces.colors.IsrColorChoiceModel
	 */
	public void setColorChoiceModel(IsrColorChoiceModel model) {
		colorChoiceModel = model;
		this.getColorChoiceView().setModel(model);
	}
}
