package sra.isr.interfaces.colors;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;

/**
 * IsrColorChoiceModel class
 * 
 * 	@author    Hirotsugu Kondo
 * 	@created   1998/11/05 (by Hirotsugu Kondo)
 * 	@updated   2000/08/21 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrColorChoiceModel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public abstract class IsrColorChoiceModel extends StApplicationModel {
	private Isr2dPoint xy = new Isr2dPoint(0,0);
	private double z = 0;
	private boolean accepted = false;
	/**
	 * Constructor.
	 */
	public IsrColorChoiceModel() {
		super();
		this.color_(Color.white);
	}
	/**
	 * Answer the area of color bar.
	 *
	 * @return Rectangle
	 */
	public static Rectangle _barArea() {
		int width = 16;
		int height = _Intervals();
		return new Rectangle(0, 0, width, height);
	}
	/**
	 * Gap is a color grid size.
	 *
	 * @return int
	 */
	protected static int _Gap() {
		return 2;
	}
	/**
	 * intervals is a color plate width and height.
	 *
	 * @return int
	 */
	protected static int _Intervals() {
		return 100;
	}
	/**
	 * Defines OK/Cancel dialog buttons.
	 *
	 * @return java.awt.Panel
	 * @param target java.awt.Dialog
	 */
	public Panel _okCancelButtons(final Dialog target) {
		Panel panel = new Panel();
		final IsrColorChoiceModel this_ = this;
		panel.add("West", new Button("OK") {
			public boolean action(Event ev, Object what) {
				this_.accept();
				target.dispose();
				return true;
			}
		});
		panel.add("East", new Button("Cancel") {
			public boolean action(Event ev, Object what) {
				this_.cancel();
				target.dispose();
				return true;
			}
		});
		return panel;
	}
	/**
	 * Answer the area of color plate.
	 *
	 * @return Rectangle
	 */
	public static Rectangle _plateArea() {
		int width = _Intervals();
		int height = _Intervals();
		return new Rectangle(0, 0, width, height);
	}
	/**
	 * Create the Dialog for IsrColorChoiceModel with initial value.
	 *
	 * @return java.awt.Color or null
	 * @param colorValue java.awt.Color
	 * @parama model sra.isr.interfaces.colors.IsrColorChoiceModel
	 */
	public static Color _Request_(Color colorValue, IsrColorChoiceModel model) {
		Dialog dialog;
		Frame parent = new Frame();
		IsrColorChoiceView view = (IsrColorChoiceView) model.defaultView();
		view.setModel(model);
		model.color_(colorValue);
		final IsrColorChoiceModel model_ = model;
		Color result = null;
		dialog = new Dialog(parent, "Color Choice", true);
		dialog.setResizable(false);
		((Component) view).setBounds(0, 0, 230, 125);
		dialog.add("Center", (Component) view);
		dialog.add("South", model._okCancelButtons(dialog));
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				model_.cancel();
				e.getWindow().dispose();
			};
		});
		dialog.pack();
		try {
			Thread.sleep(0);
		} catch (Exception e) {
		}
		dialog.pack();
		model.changed();
		_ShowAtMousePoint(dialog);
		if (model.accepted) {
			result = model.color();
		}
		return result;
	}
	/**
	 * Accept button pressed action(in dialog mode)
	 */
	public void accept() {
		accepted = true;
	}
	/**
	 * Return current selected bar.
	 *
	 * @return sra.smalltalk.StImage
	 */
	public StImage bar() {
		StImage[] thisBars = this.bars();
		int index = ((int) (this.xy.x() * (thisBars.length - 1)));
		StImage value = thisBars[index];
		return value; 
	}
	/**
	 * Answer the image of color bars.
	 *
	 * @return sra.smalltalk.StImage[]
	 */
	protected abstract StImage[] bars();
	/**
	 * Answer the brightness of color.
	 *
	 * @return double
	 */
	public abstract double brightness();
	/**
	 * Set the brightness of color.
	 *
	 * @param value double
	 */
	public abstract void brightness_(double value);
	/**
	 * Accept button pressed action(in dialog mode). do nothing.
	 */
	public void cancel() {
	}
	/**
	 * Answer the color value.
	 *
	 * @return java.awt.Color
	 */
	public Color color() {
		Color color = Color.getHSBColor((float) this.hue(), (float) this.saturation(), (float) this.brightness());
		return color;
	}
	/**
	 * Receives Color and mapping this.
	 *
	 * @param color java.awt.Color
	 */
	public abstract void color_(Color color);
	/**
	 * Answer a BlockValue that computes aBlock with the receiver's value 
	 * as the argument. aBlock will become a dependent of the receiver,
	 * and will be sent the message value: when the receiver is sent the
	 * message value:.
	 * 
	 * @return sra.smalltalk.StBlockValue
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public StBlockValue compute_(StBlockClosure aBlock) {
		StObject[] aArray = {this};
		return StBlockValue.Block_arguments_(aBlock, aArray);
	}
	/**
	 * Return the names of all the constant color
	 *
	 * @return java.util.Vector
	 */
	public static Vector ConstantNames() {
		Vector colorNames = new Vector();
		colorNames.addElement($("black"));
		colorNames.addElement($("blue"));
		colorNames.addElement($("brown"));
		colorNames.addElement($("chartreuse"));
		colorNames.addElement($("cyan"));
		colorNames.addElement($("darkCyan"));
		colorNames.addElement($("darkGray"));
		colorNames.addElement($("darkGreen"));
		colorNames.addElement($("darkMagenta"));
		colorNames.addElement($("darkRed"));
		colorNames.addElement($("olive"));
		colorNames.addElement($("gray"));
		colorNames.addElement($("green"));
		colorNames.addElement($("lightYellow"));
		colorNames.addElement($("lightGray"));
		colorNames.addElement($("magenta"));
		colorNames.addElement($("navy"));
		colorNames.addElement($("orange"));
		colorNames.addElement($("orchid"));
		colorNames.addElement($("paleGreen"));
		colorNames.addElement($("pink"));
		colorNames.addElement($("purple"));
		colorNames.addElement($("red"));
		colorNames.addElement($("royalBlue"));
		colorNames.addElement($("salmon"));
		colorNames.addElement($("lightCyan"));
		colorNames.addElement($("springGreen"));
		colorNames.addElement($("veryDarkGray"));
		colorNames.addElement($("veryLightGray"));
		colorNames.addElement($("white"));
		colorNames.addElement($("yellow"));
		return colorNames;
	}
	/**
	 * Answer a default view.
	 *
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new IsrColorChoiceViewAwt();
		} else {
			return new IsrColorChoiceViewSwing();
		}
	}
	/**
	 * Answer the hue of color.
	 *
	 * @return double
	 */
	public abstract double hue();
	/**
	 * Set the hue of color.
	 *
	 * @param value double
	 */
	public abstract void hue_(double value);
	/**
	 * Return current selected plate.
	 *
	 * @return sra.smalltalk.StImage
	 */
	public StImage plate() {
		StImage[] thisPlates = this.plates();
		int index = ((int) ((this.z * (thisPlates.length - 1))));
		StImage value = thisPlates[index];
		return value;
	}
	/**
	 * Return the plate.
	 *
	 * @return sra.smalltalk.StImage[]
	 */
	public abstract StImage[] plates();
	/**
	 * Answer the saturation of color.
	 *
	 * @return double
	 */
	public abstract double saturation();
	/**
	 * Set the saturation of color.
	 *
	 * @param value double
	 */
	public abstract void saturation_(double value);
	/**
	 * Answer the selected point.
	 *
	 * @return Isr2dPoint
	 */
	public Isr2dPoint xy() {
		return xy;
	}
	/**
	 * Set the selected point.
	 *
	 * @param aPoint sra.isr.geometry.basic.Isr2dPoint
	 */
	public void xy_(Isr2dPoint aPoint) {
		xy = aPoint;
		this.changed_($("xy"));
	}
	/**
	 * Answer the value of color bars.
	 *
	 * @return double
	 */
	public double z() {
		return z;
	}
	/**
	 * Set the value of color bars.
	 *
	 * @param aDouble double
	 */
	public void z_(double aDouble) {
		z = aDouble;
		this.changed_($("z"));
	}
}
