package sra.isr.interfaces.cursors;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Hashtable;
import sra.smalltalk.StObject;

/**
 * IsrCursors class
 * 
 * 	@author    MATSUDA Ryouichi
 * 	@created   1998/11/24 (by MATSUDA Ryouichi)
 * 	@updated   1999/08/04 (by nisinaka)
 * 	@updated   2002/04/04 (by nisinaka)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrCursors.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrCursors extends StObject {
	public final static int WIDTH = 16;
	public final static int HEIGHT = 16;

	public final static Hashtable images = new Hashtable();
	public final static Hashtable cursors = new Hashtable();
	/**
	 * Create an image from the array of pixels.
	 *
	 * @return java.awt.image.BufferedImage
	 * @param pixels int[]
	 */
	private static BufferedImage _CreateImage(int[] pixels) {
		BufferedImage image = new BufferedImage(WIDTH, HEIGHT, BufferedImage.TYPE_INT_ARGB);
		image.setRGB(0, 0, WIDTH, HEIGHT, pixels, 0, WIDTH);
		return image;
	}
	/**
	 * Create an array of pixels from an image string and a mask string.
	 * 
	 * @return int[]
	 * @param imageString java.lang.String
	 * @param maskString java.lang.String
	 */
	private static int[] _Pixels(String imageString, String maskString) {
		int[] pixels = new int[WIDTH * HEIGHT];
		for (int i = 0; i < WIDTH * HEIGHT; i++) {
			int pixel = 0;
			if (imageString.charAt(i) != '0') {
				pixel = Color.black.getRGB();
			} else if (maskString.charAt(i) != '0') {
				pixel = Color.white.getRGB();
			}
			pixels[i] = pixel;
		}
		return pixels;
	}
	/**
	 * Resize the image to fit as a cursor.
	 *
	 * @return java.awt.image.BufferedImage
	 * @param anImage java.awt.image.BufferedImage
	 */
	private static BufferedImage _ResizeToFit(BufferedImage anImage) {
		Dimension size = Toolkit.getDefaultToolkit().getBestCursorSize(WIDTH, HEIGHT);
		BufferedImage cursorImage = new BufferedImage(size.width, size.height, BufferedImage.TYPE_INT_ARGB);
		cursorImage.setData(anImage.getRaster());
		return cursorImage;
	}
	/**
	 * Answer an arrow cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor ArrowCursor() {
		if (cursors.contains($("arrow")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ArrowCursorImage()), new Point(7, 7), "arrow");
			cursors.put($("arrow"), cursor);
		}
		return (Cursor) cursors.get($("arrow"));
	}
	/**
	 * Answer an image of arrow cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage ArrowCursorImage() {
		if (images.contains($("arrow")) == false) {
 			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000010000"+
				"0000000000011000"+
				"0011111111111100"+
				"0000000000011000"+
				"0000000000010000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000010000"+
				"0000000000111000"+
				"0011111111111100"+
				"0111111111111110"+
				"0011111111111100"+
				"0000000000111000"+
				"0000000000010000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("arrow"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("arrow"));
	}
	/**
	 * Answer a between cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor BetweenCursor() {
		if (cursors.contains($("between")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ArrowCursorImage()), new Point(7, 7), "between");
			cursors.put($("between"), cursor);
		}
		return (Cursor) cursors.get($("between"));
	}
	/**
	 * Answer an image of between cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage BetweenCursorImage() {
		if (images.contains($("between")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000100000010000"+
				"0001100000011000"+
				"0011111111111100"+
				"0001100000011000"+
				"0000100000010000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000100000010000"+
				"0001110000111000"+
				"0011111111111100"+
				"0111111111111110"+
				"0011111111111100"+
				"0001110000111000"+
				"0000100000010000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("between"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("between"));;
	}
	/**
	 * Answer a brush cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor BrushCursor() {
		if (cursors.contains($("brush")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(BrushCursorImage()), new Point(2, 14), "brush");
			cursors.put($("brush"), cursor);
		}
		return (Cursor) cursors.get($("brush"));
	}
	/**
	 * Answer an image of bull cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage BrushCursorImage() {
		if (images.contains($("brush")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000001001000"+
				"0000000001001000"+
				"0000000010010000"+
				"0000000010010000"+
				"0000000100100000"+
				"0000000100100000"+
				"0000001111000000"+
				"0000010011000000"+
				"0000100011000000"+
				"0000100111000000"+
				"0001001110000000"+
				"0001011100000000"+
				"0010111000000000"+
				"0111100000000000"+
				"0000000000000000";
			String maskString =
				"0000000001001000"+
				"0000000011111100"+
				"0000000011111100"+
				"0000000111111000"+
				"0000000111111000"+
				"0000001111110000"+
				"0000001111110000"+
				"0000011111100000"+
				"0000111111100000"+
				"0001111111100000"+
				"0001111111100000"+
				"0011111111000000"+
				"0011111110000000"+
				"0111111100000000"+
				"1111111000000000"+
				"0111100000000000";
			images.put($("brush"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("brush"));
	}
	/**
	 * Answer a bull cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor BullCursor() {
		if (cursors.contains($("bull")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(BullCursorImage()), new Point(7, 7), "bull");
			cursors.put($("bull"), cursor);
		}
		return (Cursor) cursors.get($("bull"));
	}
	/**
	 * Answer an image of bull cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage BullCursorImage() {
		if (images.contains($("bull")) == false) {
			String imageString =
				"0000000000000000"+
				"0000011111000000"+
				"0001100100110000"+
				"0010000100001000"+
				"0100000100000100"+
				"0100000100000100"+
				"1000000100000010"+
				"1000000100000010"+
				"1111111111111110"+
				"1000000100000010"+
				"1000000100000010"+
				"0100000100000100"+
				"0100000100000100"+
				"0010000100001000"+
				"0001100100110000"+
				"0000011111000000";
			String maskString =
				"0000000000000000"+
				"0000111111100000"+
				"0011111111111000"+
				"0111001110011100"+
				"1110001110001110"+
				"1110001110001110"+
				"1100001110000110"+
				"1111111111111110"+
				"1111111111111110"+
				"1111111111111110"+
				"1100001110000110"+
				"1110001110001110"+
				"1110001110001110"+
				"0111001110011100"+
				"0011111111111000"+
				"0000111111100000";
			images.put($("bull"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("bull"));
	}
	/**
	 * Answer a camera cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor CameraCursor() {
		if (cursors.contains($("camera")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(CameraCursorImage()), new Point(7, 7), "camera");
			cursors.put($("camera"), cursor);
		}
		return (Cursor) cursors.get($("camera"));
	}
	/**
	 * Answer an image of camera cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage CameraCursorImage() {
		if (images.contains($("camera")) == false) {
			String imageString =
				"0000000000000000"+
				"0011100000000000"+
				"0010100000111100"+
				"0111111111111110"+
				"0100000000000010"+
				"0100001111000010"+
				"0100011001100010"+
				"0100110000110010"+
				"0100100000010010"+
				"0100100001010010"+
				"0100110010110010"+
				"0100011001100010"+
				"0100001111000010"+
				"0100000000000010"+
				"0111111111111110"+
				"0000000000000000";
			String maskString =
				"0011100000000000"+
				"0111110000111100"+
				"0111111111111110"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"0111111111111110";
			images.put($("camera"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("camera"));;
	}
	/**
	 * Answer a clock cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor ClockCursor() {
		if (cursors.contains($("clock")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ClockCursorImage()), new Point(5, 8), "clock");
			cursors.put($("clock"), cursor);
		}
		return (Cursor) cursors.get($("clock"));
	}
	/**
	 * Answer an image of clock cursor.
	 * 
	 * @return java.awt.Image
	 */
	public final static BufferedImage ClockCursorImage() {
		if (images.contains($("clock")) == false) {
			String imageString =
				"0011111100000000"+
				"0011111100000000"+
				"0011111100000000"+
				"0011111100000000"+
				"0100000010000000"+
				"1000010001000000"+
				"1000010001000000"+
				"1000010001100000"+
				"1001110001100000"+
				"1000000001000000"+
				"1000000001000000"+
				"0100000010000000"+
				"0011111100000000"+
				"0011111100000000"+
				"0011111100000000"+
				"0011111100000000";
			String maskString =
				"0011111100000000"+
				"0011111100000000"+
				"0011111100000000"+
				"0011111100000000"+
				"0111111110000000"+
				"1111111111000000"+
				"1111111111000000"+
				"1111111111100000"+
				"1111111111100000"+
				"1111111111000000"+
				"1111111111000000"+
				"0111111110000000"+
				"0011111100000000"+
				"0011111100000000"+
				"0011111100000000"+
				"0011111100000000";
			images.put($("clock"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("clock"));
	}
	/**
	 * Answer a cross cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor CrossCursor() {
		if (cursors.contains($("cross")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(CrossCursorImage()), new Point(7, 7), "cross");
			cursors.put($("cross"), cursor);
		}
		return (Cursor) cursors.get($("cross"));
	}
	/**
	 * Answer an image of cross cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage CrossCursorImage() {
		if (images.contains($("cross")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000000000000"+
				"0111110001111110"+
				"0000000000000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000100000000"+
				"0000001110000000"+
				"0000001110000000"+
				"0000001110000000"+
				"0000001110000000"+
				"0000001110000000"+
				"0111111011111110"+
				"1111110001111111"+
				"0111111011111110"+
				"0000001110000000"+
				"0000001110000000"+
				"0000001110000000"+
				"0000001110000000"+
				"0000001110000000"+
				"0000000100000000"+
				"0000000000000000";
			images.put($("cross"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("cross"));
	}
	/**
	 * Answer a dolly2 cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor Dolly2Cursor() {
		if (cursors.contains($("dolly2")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(Dolly2CursorImage()), new Point(7, 7), "dolly2");
			cursors.put($("dolly2"), cursor);
		}
		return (Cursor) cursors.get($("dolly2"));
	}
	/**
	 * Answer an image of dolly cursor 2.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage Dolly2CursorImage() {
		if (images.contains($("dolly2")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000011110000"+
				"0000000000110000"+
				"0000000000110000"+
				"0000111100110000"+
				"0011000011110000"+
				"0011000000110000"+
				"0011000000110000"+
				"0011000000110000"+
				"0000111111111100"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000011110000"+
				"0000001111111100"+
				"0000000011111100"+
				"0000111111111100"+
				"0011111111111100"+
				"1111111111111100"+
				"1111110011111100"+
				"1111110011111100"+
				"1111111111111100"+
				"0011111111111111"+
				"0000111111111100"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("dolly2"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("dolly2"));
	}
	/**
	 * Answer a dolly cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor DollyCursor() {
		if (cursors.contains($("dolly")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(DollyCursorImage()), new Point(7, 7), "dolly");
			cursors.put($("dolly"), cursor);
		}
		return (Cursor) cursors.get($("dolly"));
	}
	/**
	 * Answer an image of dolly cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage DollyCursorImage() {
		if (images.contains($("dolly")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000011000000"+
				"0000000001000000"+
				"0000000001000000"+
				"0000001101000000"+
				"0000010011000000"+
				"0000010001000000"+
				"0000010001000000"+
				"0000010001000000"+
				"0000001111100000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000011000000"+
				"0000000111100000"+
				"0000000011100000"+
				"0000001111100000"+
				"0000011111100000"+
				"0000111111100000"+
				"0000111011100000"+
				"0000111011100000"+
				"0000111111100000"+
				"0000011111110000"+
				"0000001111100000"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("dolly"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("dolly"));
	}
	/**
	 * Answer an epilogue cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor EpilogueCursor() {
		if (cursors.contains($("epilogue")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(EpilogueCursorImage()), new Point(7, 7), "epilogue");
			cursors.put($("epilogue"), cursor);
		}
		return (Cursor) cursors.get($("epilogue"));
	}
	/**
	 * Answer an image of epilogue cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage EpilogueCursorImage() {
		if (images.contains($("epilogue")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0001100000011000"+
				"0001110000011000"+
				"0001111000011000"+
				"0001101100011000"+
				"0001100110011000"+
				"0001100011011000"+
				"0001100011011000"+
				"0001100110011000"+
				"0001101100011000"+
				"0001111000011000"+
				"0001110000011000"+
				"0001100000011000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0001100000011000"+
				"0011110000111100"+
				"0011111000111100"+
				"0011111100111100"+
				"0011111110111100"+
				"0011111111111100"+
				"0011110111111100"+
				"0011110111111100"+
				"0011111111111100"+
				"0011111110111100"+
				"0011111100111100"+
				"0011111000111100"+
				"0011110000111100"+
				"0001100000011000"+
				"0000000000000000";
			images.put($("epilogue"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("epilogue"));
	}
	/**
	 * Answer an eraser cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor EraserCursor() {
		if (cursors.contains($("eraser")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(EraserCursorImage()), new Point(1, 12), "eraser");
			cursors.put($("eraser"), cursor);
		}
		return (Cursor) cursors.get($("eraser"));
	}
	/**
	 * Answer an image of eraser cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage EraserCursorImage() {
		if (images.contains($("eraser")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000111111110"+
				"0000001000000110"+
				"0000010000001010"+
				"0000100000010010"+
				"0001000000100100"+
				"0010000001001000"+
				"0111111110010000"+
				"0100000010100000"+
				"0100000011000000"+
				"0111111110000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000111111110"+
				"0000001111111111"+
				"0000011111111111"+
				"0000111111111111"+
				"0001111111111111"+
				"0011111111111110"+
				"0111111111111100"+
				"1111111111111000"+
				"1111111111110000"+
				"1111111111100000"+
				"1111111111000000"+
				"0111111110000000"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("eraser"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("eraser"));;
	}
	/**
	 * Answer a film cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor FilmCursor() {
		if (cursors.contains($("film")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(FilmCursorImage()), new Point(7, 7), "film");
			cursors.put($("film"), cursor);
		}
		return (Cursor) cursors.get($("film"));
	}
	/**
	 * Answer an image of film cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage FilmCursorImage() {
		if (images.contains($("film")) == false) {
			String imageString =
				"0000000000000000"+
				"0111000000001110"+
				"0111000000001110"+
				"0101111111111010"+
				"0111000000001110"+
				"0111000000001110"+
				"0101000000001010"+
				"0111000000001110"+
				"0111000000001110"+
				"0101000000001010"+
				"0111000000001110"+
				"0111000000001110"+
				"0101111111111010"+
				"0111000000001110"+
				"0111000000001110"+
				"0000000000000000";
			String maskString =
				"0111000000001110"+
				"1111100000011111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"1111100000011111"+
				"0111000000001110";
			images.put($("film"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("film"));
	}
	/**
	 * Answer a frame cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor FrameCursor() {
		if (cursors.contains($("frame")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(FrameCursorImage()), new Point(7, 7), "frame");
			cursors.put($("frame"), cursor);
		}
		return (Cursor) cursors.get($("frame"));
	}
	/**
	 * Answer an image of frame cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage FrameCursorImage() {
		if (images.contains($("frame")) == false) {
			String imageString =
				"0000000000000000"+
				"0111100111001110"+
				"0100000000000010"+
				"0100000000000010"+
				"0000000000000010"+
				"0000000000000000"+
				"0100000000000000"+
				"0100000000000010"+
				"0100000000000010"+
				"0000000000000010"+
				"0000000000000000"+
				"0100000000000000"+
				"0100000000000010"+
				"0100000000000010"+
				"0111001110011110"+
				"0000000000000000";
			String maskString =
				"0111100111001110"+
				"1111111111111111"+
				"1111100111001111"+
				"1110000000000111"+
				"0100000000000111"+
				"0100000000000010"+
				"1110000000000010"+
				"1110000000000111"+
				"1110000000000111"+
				"0100000000000111"+
				"0100000000000010"+
				"1110000000000010"+
				"1110000000000111"+
				"1111001110011111"+
				"1111111111111111"+
				"0111001110011110";
			images.put($("frame"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("frame"));
	}
	/**
	 * Answer a gauge cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor GaugeCursor() {
		if (cursors.contains($("gauge")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(GaugeCursorImage()), new Point(7, 7), "gauge");
			cursors.put($("gauge"), cursor);
		}
		return (Cursor) cursors.get($("gauge"));
	}
	/**
	 * Answer an image of gauge cursor.
	 * 
	 * @return java.awt.image.BufferedImage
     */
	public final static BufferedImage GaugeCursorImage() {
		if (images.contains($("gauge")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0001111011111000"+
				"0000000010001000"+
				"0000011010001000"+
				"0000000010001000"+
				"0000011010001000"+
				"0000000010001000"+
				"0000111010001000"+
				"0000000010001000"+
				"0000011011111000"+
				"0000000011111000"+
				"0000011011111000"+
				"0000000011111000"+
				"0001111011111000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0001111011111000"+
				"0011111111111100"+
				"0001111111111100"+
				"0000111111111100"+
				"0000011111111100"+
				"0000111111111100"+
				"0000111111111100"+
				"0001111111111100"+
				"0000111111111100"+
				"0000111111111100"+
				"0000011111111100"+
				"0000111111111100"+
				"0001111111111100"+
				"0011111111111100"+
				"0001111011111000";
			images.put($("gauge"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("gauge"));
	}
	/**
	 * Answer a glass cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor GlassCursor() {
		if (cursors.contains($("glass")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(GlassCursorImage()), new Point(5, 5), "glass");
			cursors.put($("glass"), cursor);
		}
		return (Cursor) cursors.get($("glass"));
	}
	/**
	 * Answer an image of grass cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage GlassCursorImage() {
		if (images.contains($("glass")) == false) {
			String imageString =
				"0000000000000000"+
				"0000111100000000"+
				"0011000011000000"+
				"0010000001000000"+
				"0100000000100000"+
				"0100000000100000"+
				"0100000000100000"+
				"0100000000100000"+
				"0010000001000000"+
				"0011000011100000"+
				"0000111101110000"+
				"0000000000111000"+
				"0000000000011100"+
				"0000000000001110"+
				"0000000000000110"+
				"0000000000000000";
			String maskString =
				"0000111100000000"+
				"0011111111000000"+
				"0111111111100000"+
				"0111000011100000"+
				"1110000001110000"+
				"1110000001110000"+
				"1110000001110000"+
				"1110000001110000"+
				"0111000011100000"+
				"0111111111110000"+
				"0011111111111000"+
				"0000111101111100"+
				"0000000000111110"+
				"0000000000011111"+
				"0000000000001111"+
				"0000000000000110";
			images.put($("glass"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("glass"));
	}
	/**
	 * Answer a go cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor GoCursor() {
		if (cursors.contains($("go")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(GoCursorImage()), new Point(7, 7), "go");
			cursors.put($("go"), cursor);
		}
		return (Cursor) cursors.get($("go"));
	}
	/**
	 * Answer an image of go cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage GoCursorImage() {
		if (images.contains($("go")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0001110000111000"+
				"0010001001000100"+
				"0010000001000100"+
				"0010000001000100"+
				"0010011101000100"+
				"0010001001000100"+
				"0010001001000100"+
				"0001110000111000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0001110000111000"+
				"0011111001111100"+
				"0111111111111110"+
				"0111001011101110"+
				"0111011111101110"+
				"0111111111101110"+
				"0111011111101110"+
				"0111111111111110"+
				"0011111001111100"+
				"0001110000111000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("go"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("go"));
	}
	/**
	 * Answer a hand cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor HandCursor() {
		if (cursors.contains($("hand")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(HandCursorImage()), new Point(7, 7), "hand");
			cursors.put($("hand"), cursor);
		}
		return (Cursor) cursors.get($("hand"));
	}
	/**
	 * Answer an image of hand cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage HandCursorImage() {
		if (images.contains($("hand")) == false) {
			String imageString =
				"0000000110000000"+
				"0001101001110000"+
				"0010011001001000"+
				"0010011001001010"+
				"0001001001001101"+
				"0001001001001001"+
				"0110100000001001"+
				"1001100000000001"+
				"1000100000000010"+
				"0100000000000010"+
				"0010000000000010"+
				"0010000000000100"+
				"0001000000000100"+
				"0000100000001000"+
				"0000010000001000"+
				"0000010000001000";
			String maskString =
				"0000000110000000"+
				"0001101111110000"+
				"0011111111111000"+
				"0011111111111010"+
				"0001111111111111"+
				"0001111111111111"+
				"0110111111111111"+
				"1111111111111111"+
				"1111111111111110"+
				"0111111111111110"+
				"0011111111111110"+
				"0011111111111100"+
				"0001111111111100"+
				"0000111111111000"+
				"0000011111111000"+
				"0000011111111000";
			images.put($("hand"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("hand"));
	}
	/**
	 * Answer a juice cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor JuiceCursor() {
		if (cursors.contains($("juice")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(JuiceCursorImage()), new Point(7, 8), "juice");
			cursors.put($("juice"), cursor);
		}
		return (Cursor) cursors.get($("juice"));
	}
	/**
	 * Answer an image of juice cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage JuiceCursorImage() {
		if (images.contains($("juice")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000001010000000"+
				"0000001010000000"+
				"0000010001000000"+
				"0000100000100000"+
				"0000100000100000"+
				"0001000000010000"+
				"0001000000010000"+
				"0001000000010000"+
				"0001000000010000"+
				"0000100000100000"+
				"0000010001000000"+
				"0000001110000000"+
				"0000000000000000";
			String maskString =
				"0000000100000000"+
				"0000001110000000"+
				"0000001110000000"+
				"0000011111000000"+
				"0000011111000000"+
				"0000111011100000"+
				"0001110001110000"+
				"0001110001110000"+
				"0011100000111000"+
				"0011100000111000"+
				"0011100000111000"+
				"0011100000111000"+
				"0001110001110000"+
				"0000111111100000"+
				"0000011111000000"+
				"0000001110000000";
			images.put($("juice"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("juice"));
	}
	/**
	 * Answer a lasso cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor LassoCursor() {
		if (cursors.contains($("lasso")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(LassoCursorImage()), new Point(3, 14), "lasso");
			cursors.put($("lasso"), cursor);
		}
		return (Cursor) cursors.get($("lasso"));
	}
	/**
	 * Answer an image of lasso cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage LassoCursorImage() {
		if (images.contains($("lasso")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000111110000"+
				"0000111000001100"+
				"0001000000000010"+
				"0010000000000010"+
				"0100000000000010"+
				"0100000000001100"+
				"0100000000110000"+
				"0011100011000000"+
				"0110011100000000"+
				"0101010000000000"+
				"0011100000000000"+
				"0000100000000000"+
				"0000100000000000"+
				"0001000000000000"+
				"0000000000000000";
			String maskString =
				"0000000111110000"+
				"0000111111111100"+
				"0001111111111110"+
				"0011111000001111"+
				"0111000000000111"+
				"1110000000001111"+
				"1110000000111110"+
				"1111100011111100"+
				"0111111111110000"+
				"1111111111000000"+
				"1111111100000000"+
				"0111110000000000"+
				"0011110000000000"+
				"0001110000000000"+
				"0011100000000000"+
				"0001000000000000";
			images.put($("lasso"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("lasso"));
	}
	/**
	 * Answer a next cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor NextCursor() {
		if (cursors.contains($("next")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(NextCursorImage()), new Point(7, 7), "next");
			cursors.put($("next"), cursor);
		}
		return (Cursor) cursors.get($("next"));
	}
	/**
	 * Answer an image of next cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage NextCursorImage() {
		if (images.contains($("next")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0001001100000000"+
				"0001001110000000"+
				"0001001111000000"+
				"0001001101100000"+
				"0001001100110000"+
				"0001001100011000"+
				"0001001100011000"+
				"0001001100110000"+
				"0001001101100000"+
				"0001001111000000"+
				"0001001110000000"+
				"0001001100000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0001001100000000"+
				"0011111110000000"+
				"0011111111000000"+
				"0011111111100000"+
				"0011111111110000"+
				"0011111111111000"+
				"0011111110111100"+
				"0011111110111100"+
				"0011111111111000"+
				"0011111111110000"+
				"0011111111100000"+
				"0011111111000000"+
				"0011111110000000"+
				"0001001100000000"+
				"0000000000000000";
			images.put($("next"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("next"));
	}
	/**
	 * Answer a normal cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor NormalCursor() {
		if (cursors.contains($("normal")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(NormalCursorImage()), new Point(7, 7), "normal");
			cursors.put($("normal"), cursor);
		}
		return (Cursor) cursors.get($("normal"));
	}
	/**
	 * Answer an image of normal cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage NormalCursorImage() {
		if (images.contains($("normal")) == false) {
			String imageString =
				"0000000000000000"+
				"0000010000000000"+
				"0000011000000000"+
				"0000011100000000"+
				"0000011110000000"+
				"0000011111000000"+
				"0000011111100000"+
				"0000011111110000"+
				"0000011111000000"+
				"0000011111000000"+
				"0000010011000000"+
				"0000000001100000"+
				"0000000001100000"+
				"0000000000110000"+
				"0000000000110000"+
				"0000000000000000)";
			String maskString =
				"0000110000000000"+
				"0000111000000000"+
				"0000111100000000"+
				"0000111110000000"+
				"0000111111000000"+
				"0000111111100000"+
				"0000111111110000"+
				"0000111111111000"+
				"0000111111100000"+
				"0000111111100000"+
				"0000111111100000"+
				"0000100011110000"+
				"0000000011110000"+
				"0000000001111000"+
				"0000000001111000"+
				"0000000000110000";
			images.put($("normal"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("normal"));
	}
	/**
	 * Answer a paint cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor PaintCursor() {
		if (cursors.contains($("paint")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PaintCursorImage()), new Point(13, 14), "paint");
			cursors.put($("paint"), cursor);
		}
		return (Cursor) cursors.get($("paint"));
	}
	/**
	 * Answer an image of paint cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage PaintCursorImage() {
		if (images.contains($("paint")) == false) {
			String imageString =
				"0000000000000000"+
				"0000011100000000"+
				"0000100110000000"+
				"0000101011000000"+
				"0000110010110000"+
				"0000100010011100"+
				"0001000010001110"+
				"0010000101000110"+
				"0100000010000110"+
				"0100000000001110"+
				"0010000000010110"+
				"0001000000100110"+
				"0000100001000110"+
				"0000010010000110"+
				"0000001100000100"+
				"0000000000000000";
			String maskString =
				"0000011100000000"+
				"0000111110000000"+
				"0001111111000000"+
				"0001111111110000"+
				"0001111111111100"+
				"0001111111111110"+
				"0011111111111111"+
				"0111111111111111"+
				"1111111111111111"+
				"1111111111111111"+
				"0111111111111111"+
				"0011111111111111"+
				"0001111111101111"+
				"0000111111001111"+
				"0000011110001110"+
				"0000001100000100";
			images.put($("paint"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("paint"));
	}
	/**
	 * Answer a pause cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor PauseCursor() {
		if (cursors.contains($("pause")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PauseCursorImage()), new Point(7, 7), "pause");
			cursors.put($("pause"), cursor);
		}
		return (Cursor) cursors.get($("pause"));
	}
	/**
	 * Answer an image of pause cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage PauseCursorImage() {
		if (images.contains($("pause")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000110000110000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0000110000110000"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("pause"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("pause"));
	}
	/**
	 * Answer a pencil cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor PencilCursor() {
		if (cursors.contains($("pencil")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PencilCursorImage()), new Point(3, 14), "pencil");
			cursors.put($("pencil"), cursor);
		}
		return (Cursor) cursors.get($("pencil"));
	}
	/**
	 * Answer an image of pencil cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage PencilCursorImage() {
		if (images.contains($("pencil")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000111100000"+
				"0000000100010000"+
				"0000001000010000"+
				"0000011100100000"+
				"0000010011100000"+
				"0000100001000000"+
				"0000100010000000"+
				"0001000010000000"+
				"0001000100000000"+
				"0001100100000000"+
				"0001111000000000"+
				"0001110000000000"+
				"0001100000000000"+
				"0001000000000000"+
				"0000000000000000";
			String maskString =
				"0000000111100000"+
				"0000001111110000"+
				"0000001111111000"+
				"0000011111111000"+
				"0000111111110000"+
				"0000111111110000"+
				"0001111111100000"+
				"0001111111000000"+
				"0011111111000000"+
				"0011111110000000"+
				"0011111110000000"+
				"0011111100000000"+
				"0011111000000000"+
				"0011110000000000"+
				"0011100000000000"+
				"0001000000000000";
			images.put($("pencil"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("pencil"));
	}
	/**
	 * Answer a play cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor PlayCursor() {
		if (cursors.contains($("play")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PlayCursorImage()), new Point(7, 7), "play");
			cursors.put($("play"), cursor);
		}
		return (Cursor) cursors.get($("play"));
	}
	/**
	 * Answer an image of play cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage PlayCursorImage() {
		if (images.contains($("play")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000011000000000"+
				"0000011100000000"+
				"0000011110000000"+
				"0000011011000000"+
				"0000011001100000"+
				"0000011000110000"+
				"0000011000110000"+
				"0000011001100000"+
				"0000011011000000"+
				"0000011110000000"+
				"0000011100000000"+
				"0000011000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000011000000000"+
				"0000111100000000"+
				"0000111110000000"+
				"0000111111000000"+
				"0000111111100000"+
				"0000111111110000"+
				"0000111101111000"+
				"0000111101111000"+
				"0000111111110000"+
				"0000111111100000"+
				"0000111111000000"+
				"0000111110000000"+
				"0000111100000000"+
				"0000011000000000"+
				"0000000000000000";
			images.put($("play"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("play"));;
	}
	/**
	 * Answer a previous cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor PreviousCursor() {
		if (cursors.contains($("previous")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PreviousCursorImage()), new Point(7, 7), "previous");
			cursors.put($("previous"), cursor);
		}
		return (Cursor) cursors.get($("previous"));
	}
	/**
	 * Answer an image of previous cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage PreviousCursorImage() {
		if (images.contains($("previous")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000011001000"+
				"0000000111001000"+
				"0000001111001000"+
				"0000011011001000"+
				"0000110011001000"+
				"0001100011001000"+
				"0001100011001000"+
				"0000110011001000"+
				"0000011011001000"+
				"0000001111001000"+
				"0000000111001000"+
				"0000000011001000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000011001000"+
				"0000000111111100"+
				"0000001111111100"+
				"0000011111111100"+
				"0000111111111100"+
				"0001111111111100"+
				"0011110111111100"+
				"0011110111111100"+
				"0001111111111100"+
				"0000111111111100"+
				"0000011111111100"+
				"0000001111111100"+
				"0000000111111100"+
				"0000000011001000"+
				"0000000000000000";
			images.put($("previous"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("previous"));
	}
	/**
	 * Answer a prologue cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor PrologueCursor() {
		if (cursors.contains($("prologue")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(PrologueCursorImage()), new Point(7, 7), "prologue");
			cursors.put($("prologue"), cursor);
		}
		return (Cursor) cursors.get($("prologue"));
	}
	/**
	 * Answer an image of prologue cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage PrologueCursorImage() {
		if (images.contains($("prologue")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0001100000011000"+
				"0001100000111000"+
				"0001100001111000"+
				"0001100011011000"+
				"0001100110011000"+
				"0001101100011000"+
				"0001101100011000"+
				"0001100110011000"+
				"0001100011011000"+
				"0001100001111000"+
				"0001100000111000"+
				"0001100000011000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0001100000011000"+
				"0011110000111100"+
				"0011110001111100"+
				"0011110011111100"+
				"0011110111111100"+
				"0011111111111100"+
				"0011111110111100"+
				"0011111110111100"+
				"0011111111111100"+
				"0011110111111100"+
				"0011110011111100"+
				"0011110001111100"+
				"0011110000111100"+
				"0001100000011000"+
				"0000000000000000";
			images.put($("prologue"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("prologue"));
	}
	/**
	 * Answer a quarters cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor QuartersCursor() {
		if (cursors.contains($("quarters")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(QuartersCursorImage()), new Point(7, 7), "quarters");
			cursors.put($("quarters"), cursor);
		}
		return (Cursor) cursors.get($("quarters"));
	}
	/**
	 * Answer an image of quarters cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage QuartersCursorImage() {
		if (images.contains($("quarters")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000100000000"+
				"0000001110000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0010000000000100"+
				"0111110001111110"+
				"0010000000000100"+
				"0000000100000000"+
				"0000000100000000"+
				"0000000100000000"+
				"0000001110000000"+
				"0000000100000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000100000000"+
				"0000001110000000"+
				"0000011111000000"+
				"0000001110000000"+
				"0000001110000000"+
				"0010001110000100"+
				"0111111011111110"+
				"1111110001111111"+
				"0111111011111110"+
				"0010001110000100"+
				"0000001110000000"+
				"0000001110000000"+
				"0000011111000000"+
				"0000001110000000"+
				"0000000100000000"+
				"0000000000000000";
			images.put($("quarters"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("quarters"));
	}
	/**
	 * Answer a quarters cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor ReadCursor() {
		if (cursors.contains($("read")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ReadCursorImage()), new Point(5, 10), "read");
			cursors.put($("read"), cursor);
		}
		return (Cursor) cursors.get($("read"));
	}
	/**
	 * Answer an image of read cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage ReadCursorImage() {
		if (images.contains($("read")) == false) {
			String imageString =
				"0000110000000110" +
				"0001001000001001" +
				"0001001000001001" +
				"0010000000010000" +
				"0100000000100000" +
				"1111101111100000" +
				"1000010000100000" +
				"1000010000100000" +
				"1011010110100000" +
				"0111101111000000" +
				"0000000000000000" +
				"0000000000000000" +
				"0000000000000000" +
				"0000000000000000" +
				"0000000000000000" +
				"0000000000000000";
	        String maskString =
				"0001000000001000" +
				"0010010000010010" +
				"0010010000010010" +
				"0100000000100000" +
				"1000000001000000" +
				"0000000000000000" +
				"0111101111000000" +
				"0111101111000000" +
				"0100101001000000" +
				"0000010000000000" +
				"0111101111000000" +
				"0000000000000000" +
				"0000000000000000" +
				"0000000000000000" +
				"0000000000000000" +
				"0000000000000000";
			images.put($("read"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("read"));
	}
	/**
	 * Answer a repeat cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor RepeatCursor() {
		if (cursors.contains($("repeat")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(RepeatCursorImage()), new Point(7, 7), "repeat");
			cursors.put($("repeat"), cursor);
		}
		return (Cursor) cursors.get($("repeat"));
	}
	/**
	 * Answer an image of repeat cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage RepeatCursorImage() {
		if (images.contains($("repeat")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000100000000"+
				"0000000110000000"+
				"0001111111111000"+
				"0011000110001100"+
				"0010000100000100"+
				"0010000000000100"+
				"0011000000001100"+
				"0001111111111000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000100000000"+
				"0000001110000000"+
				"0001111111111000"+
				"0011111111111100"+
				"0111111111111110"+
				"0111001110001110"+
				"0111000100001110"+
				"0111111111111110"+
				"0011111111111100"+
				"0001111111111000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("repeat"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("repeat"));
	}
	/**
	 * Answer a roundtrip cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor RoundtripCursor() {
		if (cursors.contains($("roundtrip")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(RoundtripCursorImage()), new Point(7, 7), "roundtrip");
			cursors.put($("roundtrip"), cursor);
		}
		return (Cursor) cursors.get($("roundtrip"));
	}
	/**
	 * Answer an image of roundtrip cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage RoundtripCursorImage() {
		if (images.contains($("roundtrip")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000010000"+
				"0000000000011000"+
				"0011111111111100"+
				"0000000000011000"+
				"0000000000010000"+
				"0000100000000000"+
				"0001100000000000"+
				"0011111111111100"+
				"0001100000000000"+
				"0000100000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000010000"+
				"0000000000111000"+
				"0011111111111100"+
				"0111111111111110"+
				"0011111111111100"+
				"0000100000111000"+
				"0001110000010000"+
				"0011111111111100"+
				"0111111111111110"+
				"0011111111111100"+
				"0001110000000000"+
				"0000100000000000"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("roundtrip"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("roundtrip"));
	}
	/**
	 * Answer a scissors cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor ScissorsCursor() {
		if (cursors.contains($("scissors")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(ScissorsCursorImage()), new Point(7, 8), "scissors");
			cursors.put($("scissors"), cursor);
		}
		return (Cursor) cursors.get($("scissors"));
	}
	/**
	 * Answer an image of scissors cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage ScissorsCursorImage() {
		if (images.contains($("scissors")) == false) {
			String imageString =
				"0000000000000000"+
				"0000001010000000"+
				"0000011011000000"+
				"0000011011000000"+
				"0000011011000000"+
				"0000011011000000"+
				"0000001010000000"+
				"0000001110000000"+
				"0000001010000000"+
				"0000001110000000"+
				"0000011011000000"+
				"0000101010100000"+
				"0001001010010000"+
				"0001001010010000"+
				"0000111011100000"+
				"0000000000000000";
			String maskString =
				"0000001010000000"+
				"0000011111000000"+
				"0000111111100000"+
				"0000111111100000"+
				"0000111111100000"+
				"0000111111100000"+
				"0000011111000000"+
				"0000011111000000"+
				"0000011111000000"+
				"0000011111000000"+
				"0000111111100000"+
				"0001111111110000"+
				"0011111111111000"+
				"0011111111111000"+
				"0001111111110000"+
				"0000111011100000";
			images.put($("scissors"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("scissors"));
	}
	/**
	 * Answer a speaker cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor SpeakerCursor() {
		if (cursors.contains($("speaker")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(SpeakerCursorImage()), new Point(7, 7), "speaker");
			cursors.put($("speaker"), cursor);
		}
		return (Cursor) cursors.get($("speaker"));
	}
	/**
	 * Answer an image of speaker cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage SpeakerCursorImage() {
		if (images.contains($("speaker")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000100000000"+
				"0000001100010000"+
				"0000010100001000"+
				"0000100100000100"+
				"0001000100100100"+
				"0110000100100100"+
				"0100000100010010"+
				"0100000100010010"+
				"0110000100100100"+
				"0001000100100100"+
				"0000100100000100"+
				"0000010100001000"+
				"0000001100010000"+
				"0000000100000000"+
				"0000000000000000";
			String maskString =
				"0000000100000000"+
				"0000001110010000"+
				"0000011110111000"+
				"0000111110011100"+
				"0001111110101110"+
				"0111111111111110"+
				"1111111111111110"+
				"1111111110111111"+
				"1111111110111111"+
				"1111111111111110"+
				"0111111111111110"+
				"0001111110101110"+
				"0000111110011100"+
				"0000011110111000"+
				"0000001110010000"+
				"0000000100000000";
			images.put($("speaker"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("speaker"));
	}
	/**
	 * Answer a spuit cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor SpuitCursor() {
		if (cursors.contains($("spuit")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(SpuitCursorImage()), new Point(2, 13), "spuit");
			cursors.put($("spuit"), cursor);
		}
		return (Cursor) cursors.get($("spuit"));
	}
	/**
	 * Answer an image of spuit cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage SpuitCursorImage() {
		if (images.contains($("spuit")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000011100"+
				"0000000000111110"+
				"0000000000111110"+
				"0000000111111110"+
				"0000000011111100"+
				"0000000101110000"+
				"0000001000110000"+
				"0000010001010000"+
				"0000100010000000"+
				"0001000100000000"+
				"0010001000000000"+
				"0010010000000000"+
				"0101100000000000"+
				"0010000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000011100"+
				"0000000000111110"+
				"0000000001111111"+
				"0000011111111111"+
				"0000001111111111"+
				"0000000111111110"+
				"0000001111111100"+
				"0000011111111000"+
				"0000111111111000"+
				"0001111111011000"+
				"0011111110001000"+
				"0111111100000000"+
				"0111111000000000"+
				"1101110000000000"+
				"0111100000000000"+
				"0010000000000000";
			images.put($("spuit"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("spuit"));
	}
	/**
	 * Answer a stop cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public final static Cursor StopCursor() {
		if (cursors.contains($("stop")) == false) {
			Cursor cursor = Toolkit.getDefaultToolkit().createCustomCursor(_ResizeToFit(StopCursorImage()), new Point(7, 7), "stop");
			cursors.put($("stop"), cursor);
		}
		return (Cursor) cursors.get($("stop"));
	}
	/**
	 * Answer an image of stop cursor.
	 * 
	 * @return java.awt.image.BufferedImage
	 */
	public final static BufferedImage StopCursorImage() {
		if (images.contains($("stop")) == false) {
			String imageString =
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000111111110000"+
				"0000111111110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000110000110000"+
				"0000111111110000"+
				"0000111111110000"+
				"0000000000000000"+
				"0000000000000000"+
				"0000000000000000";
			String maskString =
				"0000000000000000"+
				"0000000000000000"+
				"0000111111110000"+
				"0001111111111000"+
				"0001111111111000"+
				"0001111111111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111001111000"+
				"0001111111111000"+
				"0001111111111000"+
				"0001111111111000"+
				"0000111111110000"+
				"0000000000000000"+
				"0000000000000000";
			images.put($("stop"), _CreateImage(_Pixels(imageString, maskString)));
		}
		return (BufferedImage) images.get($("stop"));
	}
}
