package sra.isr.interfaces.display;

import java.awt.*;
import sra.smalltalk.*;

/**
 * IsrDisplayModel class
 * 
 * 	@author    nisinaka
 * 	@created   1998/11/16 (by nisinaka)
 * 	@updated   1999/11/11 (by MATSUDA Ryouichi)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrDisplayModel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrDisplayModel extends StApplicationModel {
	private Object displayObject;
	/**
	 * Answer a default view.
	 *
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		IsrDisplayView aView;
		if (GetDefaultViewMode() == VIEW_AWT) {
			aView = new IsrDisplayViewAwt();
		} else {
			aView = new IsrDisplayViewSwing();
		}
		aView.model_(this);
		aView.adjustSize();
		return aView;
	}
	/**
	 * Create the display model with initial value.
	 * 
	 * @return sra.isr.interfaces.display.IsrDisplayModel
	 * @param anObject java.lang.Object
	 * @param aString java.lang.String
	 */
	public static IsrDisplayModel OpenVisual_label_(Object anObject, String aString) {
		IsrDisplayModel model = new IsrDisplayModel();
		model.visual_(anObject);
		Frame window = model.open();
		window.setTitle(aString);
		return model;
	}
	/**
	 * Answer the visual.
	 *
	 * @return java.lang.Object
	 */
	public Object visual() {
		if (displayObject == null) {
			displayObject = "";
		}
		return displayObject;
	}
	/**
	 * Create the display model with initial value.
	 * 
	 * @return sra.jun.goodies.display.JunDisplayModel
	 * @param anObject java.lang.Object
	 */
	public static IsrDisplayModel Visual_(Object anObject) {
		IsrDisplayModel model = new IsrDisplayModel();
		model.visual_(anObject);
		return model;
	}
	/**
	 * Set the visual.
	 *
	 * @param java.lang.Object
	 */
	public void visual_(Object anObject) {
		if (displayObject != anObject) {
			displayObject = anObject;
			changed_($("visual"));
		}
	}
}
