package sra.isr.interfaces.display;

import java.awt.*;
import sra.smalltalk.*;

/**
 * IsrDisplayViewAwt class
 * 
 * 	@author    nisinaka
 * 	@created   1998/11/16 (by nisinaka)
 * 	@updated   1999/11/11 (by MATSUDA Ryouichi)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrDisplayViewAwt.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrDisplayViewAwt extends StViewCanvas implements IsrDisplayView {
	// property
	private boolean resizable;
	/**
	 * Answer the preferred size of the object.
	 *
	 * @return java.awt.Dimension
	 * @param anObject java.lang.Object
	 */
	protected Dimension _getPreferredSizeOf(Object anObject) {
		if (anObject instanceof Image) {
			Image anImage = (Image) anObject;
			return new Dimension(anImage.getWidth(null), anImage.getHeight(null));
		} else {
			String aString;
			if (anObject instanceof String) {
				aString = (String) anObject;
			} else {
				aString = anObject.toString();
			}
			Point extent = SystemResourceSupport._getStringExtentFor_(aString, null);
			return new Dimension(extent.x, extent.y);
		}
	}
	/**
	 * Adjust size.
	 */
	public void adjustSize() {
		this.setSize(this._getPreferredSizeOf(getModel().visual()));
	}
	/**
	 * Answer a default model.
	 *
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new IsrDisplayModel();
	}
	/**
	 * Display the receiver on aGraphics.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayOn_(Graphics aGraphics) {
		Object displayObject = this.getModel().visual();
		if (displayObject instanceof Image) {
			Image anImage = (Image) displayObject;
			if (this.isResizable()) {
				aGraphics.drawImage(anImage, 0, 0, this.getSize().width, this.getSize().height, null);
			} else {
				int x = (this.getSize().width - anImage.getWidth(null)) / 2;
				int y = (this.getSize().height - anImage.getHeight(null)) / 2;
				aGraphics.drawImage(anImage, x, y, null);
			}
		} else {
			String aString;
			if (displayObject instanceof String) {
				aString = (String) displayObject;
			} else {
				aString = displayObject.toString();
			}
			FontMetrics fontMetrics = aGraphics.getFontMetrics();
			int x = (this.getSize().width - fontMetrics.stringWidth(aString)) / 2;
			int y = (this.getSize().height + fontMetrics.getAscent()) / 2;
			aGraphics.setColor(Color.black);
			aGraphics.drawString(aString, x, y);
		}
	}
	/**
	 * Model property.
	 *
	 * @return sra.isr.interfaces.display.IsrDisplayModel
	 */
	public IsrDisplayModel getModel() {
		return (IsrDisplayModel) this.model();
	}
	/**
	 * Visual property.
	 *
	 * @return java.lang.Object
	 */
	public Object getVisual() {
		return this.getModel().visual();
	}
	/**
	 * Resizable property.
	 *
	 * @return boolean
	 */
	public boolean isResizable() {
		return resizable;
	}
	/**
	 * Model property.
	 *
	 * @return sra.isr.interfaces.display.IsrDisplayModel
	 */
	public void setModel(IsrDisplayModel aModel) {
		this.model_(aModel);
	}
	/**
	 * Resizable property.
	 *
	 * @param newValue boolean
	 */
	public void setResizable(boolean newValue) {
		boolean oldValue = resizable;
		resizable = newValue;
		this.repaint();
		this.firePropertyChange("resizable", new Boolean(oldValue), new Boolean(newValue));
	}
	/**
	 * Visual property.
	 *
	 * @param anObject java.lang.Object
	 */
	public void setVisual(Object anObject) {
		this.getModel().visual_(anObject);
	}
	/*
	 * Update
	 *
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (evt.getAspect() == $("visual")) {
			this.repaint();
			this.firePropertyChange("visual", null, getVisual());
		} else {
			super.update_(evt);
		}
	}
}
