package sra.isr.interfaces.freehand;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;
import sra.isr.interfaces.button.*;
import sra.isr.system.support.*;

/**
 * IsrFreehandApplicationViewAwt class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2000/03/07 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrFreehandApplicationViewAwt.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrFreehandApplicationViewAwt extends StViewPanel implements IsrFreehandApplicationView {
	protected IsrButtonViewAwt scissorsButtonView;
	protected IsrButtonViewAwt brushButtonView;
	protected IsrButtonViewAwt colorButtonView;
	protected IsrButtonViewAwt spuitButtonView;
	protected IsrButtonViewAwt eraserButtonView;
	protected IsrFreehandViewAwt freehandView;
	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		Panel leftPanel, leftTopPanel, centerPanel;
		this.setLayout(new BorderLayout());

		//leftTopPanel
		leftTopPanel = new Panel(new GridLayout(5, 1));
		//
		scissorsButtonView = new IsrButtonViewAwt();
		scissorsButtonView.setSize(23, 23);
		try {
			SystemInterface.Current();
		} catch (Throwable e) {
			scissorsButtonView.setEnabled(false);
		}
		leftTopPanel.add(scissorsButtonView);
		//
		brushButtonView = new IsrButtonViewAwt() {
			protected StController defaultController() {
				return new IsrButtonController() {
					public void mousePressed(MouseEvent e) {
						IsrButtonViewAwt view = (IsrButtonViewAwt) this.getView();
						IsrButtonModel model = this.getModel();
						Rectangle box = new Rectangle(0, 0, view.getBounds().width, view.getBounds().height);
						if (!box.contains(e.getPoint())) {
							return;
						}
						StBlockClosure action = model.action();
						if (action != null) {
							switch (action.numArgs()) {
								case 0 :
									action.value();
									break;
								case 1 :
									action.value_(model);
									break;
								case 2 :
									action.value_value_(model, e);
									break;
							}
						}
						e.consume();
					}
					public void mouseReleased(MouseEvent e) {
					}
					public void mouseEntered(MouseEvent e) {
					}
					public void mouseExited(MouseEvent e) {
					}
				};
			}
		};
		brushButtonView.setSize(23, 23);
		leftTopPanel.add(brushButtonView);
		//
		colorButtonView = new IsrButtonViewAwt();
		colorButtonView.setSize(23, 23);
		leftTopPanel.add(colorButtonView);
		//
		spuitButtonView = new IsrButtonViewAwt();
		spuitButtonView.setSize(23, 23);
		try {
			SystemInterface.Current();
		} catch (Throwable e) {
			spuitButtonView.setEnabled(false);
		}
		leftTopPanel.add(spuitButtonView);
		//
		eraserButtonView = new IsrButtonViewAwt();
		eraserButtonView.setSize(23, 23);
		leftTopPanel.add(eraserButtonView);

		//leftPanel
		leftPanel = new Panel(new BorderLayout());
		//
		leftPanel.add(leftTopPanel, "North");

		//centerPanel
		centerPanel = StApplicationModel._PanelWithBorder();
		centerPanel.setLayout(new BorderLayout());
		//
		freehandView = new IsrFreehandViewAwt();
		freehandView.setSize(this.getModel().defaultSize() + 20, this.getModel().defaultSize() + 20);
		freehandView.model_(this.getModel());
		centerPanel.add(freehandView, "Center");
		this.add(leftPanel, "West");
		this.add(centerPanel, "Center");
	}
	/**
	 * Answer a default model.
	 *
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new IsrFreehandModel();
	}
	/**
	 * Answer a Model.
	 *
	 * @return sra.isr.interfaces.freehand.IsrFreehandModel
	 */
	public IsrFreehandModel getModel() {
		return (IsrFreehandModel) this.model();
	}
	/**
	 * Rebuild this component.
	 *
	 * @param oldModel sra.smalltalk.StModel
	 * @param newModel sra.smalltalk.StModel
	 */
	protected void rebuildComponent(StModel oldModel, StModel newModel) {
		IsrFreehandModel model = (IsrFreehandModel) this.model();
		freehandView.model_(model);
		//
		scissorsButtonView.model_(model.scissorsButton());
		brushButtonView.model_(model.brushButton());
		colorButtonView.model_(model.colorButton());
		spuitButtonView.model_(model.spuitButton());
		eraserButtonView.model_(model.eraserButton());
	}
	/**
	 * Model property accessing.
	 *
	 * @param aModel sra.isr.interfaces.freehand.IsrFreehandModel
	 */
	public void setModel(IsrFreehandModel aModel) {
		this.model_(aModel);
	}
}
