package sra.isr.interfaces.freehand;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;

/**
 * IsrFreehandController class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2000/02/18 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrFreehandController.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrFreehandController extends StController implements MouseListener, MouseMotionListener {
	protected Point previousPoint = new Point(0, 0);
	protected Point currentPoint = new Point(0, 0);
	/**
	 * Add listeners.
	 * 
	 * @param newView sra.mvctest.StView
	 */
	protected void buildListener(StView newView) {
		Component aView = (Component) newView;
		aView.addMouseListener(this);
		aView.addMouseMotionListener(this);
	}
	/**
	 * Answer a model.
	 *
	 * @return sra.isr.interfaces.freehand.IsrFreehandModel
	 */
	protected final IsrFreehandModel getModel() {
		return (IsrFreehandModel) this.model();
	}
	/**
	 * Answer this view.
	 *
	 * @return sra.isr.interfaces.freehand.IsrFreehandView interface
	 */
	protected IsrFreehandView getView() {
		return (IsrFreehandView) this.view();
	}
	/**
	 * The General dragging action.
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseDragged(MouseEvent event) {
		if (this.isMenuButtonPressed_(event)) {
			return;
		}
		IsrFreehandView view = this.getView();
		IsrFreehandModel model = this.getModel();
		Rectangle box = view.plateArea();
		currentPoint = event.getPoint();
		currentPoint.translate(0 - box.x, 0 - box.y);
		Graphics gc = model.currentPixmap().getGraphics();
		model.gstate_(gc);
		int lineWidth = model.lineWidth();
		if (previousPoint != null) {
			for (int width = - (lineWidth / 2); width < (- (lineWidth / 2) + lineWidth); width++) {
				gc.drawLine(previousPoint.x + width, previousPoint.y, currentPoint.x + width, currentPoint.y);
				gc.drawLine(previousPoint.x, previousPoint.y + width, currentPoint.x, currentPoint.y + width);
			}
		}
		if (box.contains(event.getPoint())) {
			gc.fillOval(currentPoint.x - (lineWidth / 2), currentPoint.y - (lineWidth / 2), lineWidth, lineWidth);
		}
		model.setImage(model.currentPixmap());
		previousPoint = currentPoint;
		gc.dispose();
	}
	/**
	 * Invoked when the mouse button has been moved on a component
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseMoved(MouseEvent event) {
		IsrFreehandView view = this.getView();
		if (view.plateArea().contains(event.getPoint())) {
			((Component) view).setCursor(this.getModel().cursor());
		} else {
			((Component) view).setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		}
	}
	/**
	 * Invoked when a mouse button has been pressed. 
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent event) {
		if (this.isMenuButtonPressed_(event)) {
			return;
		}
		IsrFreehandView view = this.getView();
		IsrFreehandModel model = this.getModel();
		Rectangle box = view.plateArea();
		if (box.contains(event.getPoint())) {
			currentPoint = event.getPoint();
			currentPoint.translate(0 - box.x, 0 - box.y);
			Graphics gc = model.currentPixmap().getGraphics();
			model.gstate_(gc);
			int lineWidth = model.lineWidth();
			gc.fillOval(currentPoint.x - (lineWidth / 2), currentPoint.y - (lineWidth / 2), lineWidth, lineWidth);
			previousPoint = currentPoint;
			model.setImage(model.currentPixmap());
			gc.dispose();
		} else {
			previousPoint = null;
		}
	}
}
