package sra.isr.interfaces.freehand;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * IsrFreehandImageJPanel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/06/23 (by Mitsuhiro Asada)
 * 	@updated   2001/06/15 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrFreehandImageJPanel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrFreehandImageJPanel extends JPanel {
	private IsrFreehandModel freehandModel = null;
	private IsrFreehandApplicationView freehandView = null;
	private JCheckBox freehandSwitch = null;
	/**
	 * IsrFreehandImageJPanel constructor.
	 */
	public IsrFreehandImageJPanel() {
		super();
		initialize();
	}
	/**
	 * IsrFreehandImageJPanel constructor.
	 * 
	 * @param layout java.awt.LayoutManager
	 */
	public IsrFreehandImageJPanel(LayoutManager layout) {
		super(layout);
		initialize();
	}
	/**
	 * IsrFreehandImageJPanel constructor.
	 *
	 * @param layout java.awt.LayoutManager
	 * @param isDoubleBuffered boolean
	 */
	public IsrFreehandImageJPanel(LayoutManager layout, boolean isDoubleBuffered) {
		super(layout, isDoubleBuffered);
		initialize();
	}
	/**
	 * IsrFreehandImageJPanel constructor.
	 * 
	 * @param isDoubleBuffered boolean
	 */
	public IsrFreehandImageJPanel(boolean isDoubleBuffered) {
		super(isDoubleBuffered);
		initialize();
	}
	/**
	 * connFreehandViewSetTarget:  (FreehandImageModel.this <--> FreehandView.model)
	 */
	private void _connFreehandViewSetTarget() {
		try {
			this.getFreehandView().setModel(this.getFreehandModel());
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Called whenever the part throws an exception.
	 * 
	 * @param exception java.lang.Throwable
	 */
	private void _handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}
	/**
	 * Initializes connections
	 *
	 * @exception java.lang.Exception
	 */
	private void _initConnections() throws Exception {
		this._connFreehandViewSetTarget();
	}
	/**
	 * Return the freehand image.
	 *
	 * @return java.awt.Image
	 */
	public Image freehandImage() {
		return this.getFreehandModel().currentPixmap();
	}
	/**
	 * Set the freehand image.
	 *
	 * @param image java.awt.Image
	 */
	public void freehandImage_(Image image) {
		this.getFreehandModel().setImage(image);
	}
	/**
	 * Return the freehand switch.
	 *
	 * @return boolean
	 */
	public boolean freehandSwitch() {
		return this.getFreehandSwitch().isSelected();
	}
	/**
	 * Set the freehand switch.
	 *
	 * @param aBoolean boolean
	 */
	public void freehnadSwitch_(boolean aBoolean) {
		this.getFreehandSwitch().setSelected(aBoolean);
	}
	/**
	 * Return the FreehandImageModel property value.
	 * 
	 * @return sra.isr.interfaces.freehand.IsrFreehandModel
	 */
	public IsrFreehandModel getFreehandModel() {
		if (freehandModel == null) {
			try {
				freehandModel = new IsrFreehandModel();
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return freehandModel;
	}
	/**
	 * Return the FreehandSwitch property value.
	 *
	 * @return javax.swing.JCheckBox
	 */
	public JCheckBox getFreehandSwitch() {
		if (freehandSwitch == null) {
			try {
				freehandSwitch = new JCheckBox();
				freehandSwitch.setName("FreehandSwitch");
				freehandSwitch.setText("freehand");
				freehandSwitch.setBounds(2, 2, 123, 25);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return freehandSwitch;
	}
	/**
	 * Return the FreehandView property value.
	 * 
	 * @return sra.isr.interfaces.freehand.IsrFreehandApplicationView
	 */
	protected IsrFreehandApplicationView getFreehandView() {
		if (freehandView == null) {
			try {
				freehandView = new IsrFreehandApplicationViewSwing();
				((Component) freehandView).setName("FreehandViewSwing");
				((Component) freehandView).setBounds(2, 28, 201, 178);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return freehandView;
	}
	/**
	 * Initialize the class.
	 */
	private void initialize() {
		try {
			this.setName("IsrFreehandJPanel");
			this.setLayout(null);
			this.setSize(208, 206);
			this.add(this.getFreehandSwitch(), this.getFreehandSwitch().getName());
			this.add((Component) this.getFreehandView(), ((Component) this.getFreehandView()).getName());
			this._initConnections();
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Set the FreehandImageModel property value.
	 * 
	 * @param model sra.isr.interfaces.freehand.IsrFreehandModel
	 */
	public void setFreehandModel(IsrFreehandModel model) {
		freehandModel = model;
		this.getFreehandView().setModel(model);
	}
}
