package sra.isr.interfaces.freehand;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.image.support.*;
import sra.isr.interfaces.colors.*;
import sra.isr.interfaces.button.*;
import sra.isr.interfaces.cursors.*;
import sra.isr.system.support.*;

/**
 * IsrFreehandModel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2002/01/04 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrFreehandModel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrFreehandModel extends StApplicationModel {
	protected Image currentPixmap;
	protected int lineWidth = 4;
	protected IsrColorChoiceModel colorModel;
	protected IsrButtonModel colorButton;
	private boolean accepted = false;
	//
	protected PopupMenu brushMenu;
	/**
	 * Construsctor
	 */
	public IsrFreehandModel() {
		super();
	}
	/**
	 * Answer the icon image which disable button.
	 * 
	 * @param icon java.awt.Image
	 * @return java.awt.Image
	 */
	public Image _disableIconImage(Image icon) {
		int width = 23;
		int height = 23;
		int[] pixcels = new int[width * height];
		for (int i = 0; i < pixcels.length; i++) {
			pixcels[i] = 0x00000000;
		}
		StImage anImage = new StImage(width, height, pixcels);
		anImage.copy_from_in_rule_(new Rectangle(4, 4, 16, 16), new Point(0, 0), new StImage(icon), StImage.Over);
		int black = Color.black.getRGB();
		int color = (new Color((float) 0.7, (float) 0.7, (float) 0.7)).getRGB();
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				if (anImage.getPixel(x, y) == black) {
					anImage.setPixel(x, y, color);
				}
			}
		}
		return anImage.image();
	}
	/**
	 * Set line width with argument.
	 *
	 * @param number int 
	 */
	public void _lineWidth_(int number) {
		lineWidth= number;
	}
	/**
	 * Set line width 1.
	 */
	public void _lineWidth1() {
		this._lineWidth_(1);
	}
	/**
	 * Set line width 10.
	 */
	public void _lineWidth10() {
		this._lineWidth_(10);
	}
	/**
	 * Set line width 16.
	 */
	public void _lineWidth16() {
		this._lineWidth_(16);
	}
	/**
	 * Set line width 2.
	 */
	public void _lineWidth2() {
		this._lineWidth_(2);
	}
	/**
	 * Set line width 24.
	 */
	public void _lineWidth24() {
		this._lineWidth_(24);
	}
	/**
	 * Set line width 3.
	 */
	public void _lineWidth3() {
		this._lineWidth_(3);
	}
	/**
	 * Set line width 32.
	 */
	public void _lineWidth32() {
		this._lineWidth_(32);
	}
	/**
	 * Set line width 4.
	 */
	public void _lineWidth4() {
		this._lineWidth_(4);
	}
	/**
	 * Set line width 5.
	 */
	public void _lineWidth5() {
		this._lineWidth_(5);
	}
	/**
	 * Set line width 6.
	 */
	public void _lineWidth6() {
		this._lineWidth_(6);
	}
	/**
	 * Set line width 7.
	 */
	public void _lineWidth7() {
		this._lineWidth_(7);
	}
	/**
	 * Set line width 8.
	 */
	public void _lineWidth8() {
		this._lineWidth_(8);
	}
	/**
	 * Set line width 9.
	 */
	public void _lineWidth9() {
		this._lineWidth_(9);
	}
	/**
	 * Defines OK/Cancel dialog buttons.
	 *
	 * @return java.awt.Panel
	 * @param target java.awt.Dialog
	 */
	public Panel _okCancelButtons(final Dialog target) {
		Panel panel = new Panel();
		final IsrFreehandModel this_ = this;
		panel.add("West", new Button("OK") {
			public boolean action(Event ev, Object what) {
				this_.accept();
				target.dispose();
				return true;
			}
		});
		panel.add("East", new Button("Cancel") {
			public boolean action(Event ev, Object what) {
				this_.cancel();
				target.dispose();
				return true;
			}
		});
		return panel;
	}
	/**
	 * Accept button pressed action(in dialog mode)
	 */
	public void accept() {
		accepted = true;
	}
	/**
	 * Answer my brush button.
	 * 
	 * @return sra.isr.interfaces.button.IsrButtonModel
	 */
	public IsrButtonModel brushButton() {
		final IsrFreehandModel self = this;
		StBlockClosure block = new StBlockClosure() {
			public Object value_value_(Object o1, Object o2) {
				IsrButtonModel model = (IsrButtonModel) o1;
				MouseEvent event = (MouseEvent) o2;
				self.brushButtonAction(event);
				return null;
			}
		};
		return IsrButtonModel.Value_visual_action_(false, IsrCursors.BrushCursorImage(), block);
	}
	/**
	 * Set the brush button action.
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void brushButtonAction(MouseEvent event) {
		Point point = event.getPoint();
		Component component = event.getComponent();
		if (brushMenu == null) {
			brushMenu = this.brushMenu();
			component.add(brushMenu);
		}
		brushMenu.show(component, point.x, point.y);
	}
	/**
	 * Create a popup menu of brush button.
	 *
	 * @return java.awt.PopupMenu
	 */
	protected PopupMenu brushMenu() {
		StMenuModel rootMenu = new StMenuModel();
		//
		StMenu lineWidthMenu = new StMenu("LineWidthMenu");
		lineWidthMenu.add(this.createMenuItem("1", "_lineWidth1"));
		lineWidthMenu.add(this.createMenuItem("2", "_lineWidth2"));
		lineWidthMenu.add(this.createMenuItem("3", "_lineWidth3"));
		lineWidthMenu.add(this.createMenuItem("4", "_lineWidth4"));
		lineWidthMenu.add(this.createMenuItem("5", "_lineWidth5"));
		lineWidthMenu.add(this.createMenuItem("6", "_lineWidth6"));
		lineWidthMenu.add(this.createMenuItem("7", "_lineWidth7"));
		lineWidthMenu.add(this.createMenuItem("8", "_lineWidth8"));
		lineWidthMenu.add(this.createMenuItem("9", "_lineWidth9"));
		lineWidthMenu.add(this.createMenuItem("10", "_lineWidth10"));
		lineWidthMenu.add(this.createMenuItem("16", "_lineWidth16"));
		lineWidthMenu.add(this.createMenuItem("24", "_lineWidth24"));
		lineWidthMenu.add(this.createMenuItem("32", "_lineWidth32"));
		rootMenu.add(lineWidthMenu);
		//
		StMenu menu = (StMenu) rootMenu.getItem("LineWidthMenu");
		PopupMenu popupMenu = StMenuViewForAwt.CreatePopupMenu(menu);
		return popupMenu;
	}
	/**
	 * Accept button pressed action(in dialog mode). do nothing.
	 */
	public void cancel() {
	}
	/**
	 * Clear plate.
	 */
	public void clear() {
		Graphics gc = this.currentPixmap.getGraphics();
		try {
			gc.setColor(Color.white);
			gc.fillRect(0, 0, this.defaultSize(), this.defaultSize());
			this.commit();
			this.changed_(StObject.$("plate"));
		} finally {
			gc.dispose();
		}
	}
	/**
	 * Answer my selection color.
	 * 
	 * @return java.awt.Color
	 */
	public Color color() {
		return this.colorModel().color();
	}
	/**
	 * Answer my color button.
	 * 
	 * @return sra.isr.interfaces.button.IsrButtonModel
	 */
	public IsrButtonModel colorButton() {
		if (colorButton == null) {
			final IsrFreehandModel self = this;
			StBlockClosure block = new StBlockClosure() {
				public Object value_(Object o) {
					self.colorButtonAction();
					return null;
				}
			};
			colorButton = IsrButtonModel.Value_visual_action_(false, this.colorImage(), block);
		}
		return colorButton;
	}
	/**
	 * Set the color button action.
	 */
	public void colorButtonAction() {
		IsrColorChoiceModel model = this.colorModel();
		Frame frames[] = new Frame[model.builder().frames().length];
		frames = model.builder().frames();
		for (int i = 0; i < frames.length; i++) {
			if (frames[i].isShowing()) {
				frames[i].toFront();
			} else {
				frames[i].show();
			}
		}
		if (frames.length == 0) {
			model.open();
		}
	}
	/**
	 * Answer default color image.
	 * 
	 * @return java.awt.Image
	 */
	public Image colorImage() {
		StImage image = new StImage(16, 16);
		Rectangle bounds = image.bounds();
		image = IsrImageProcessor.Fill_rectangle_color_(image, bounds, Color.white);
		bounds = new Rectangle(1, 1, bounds.width - 2, bounds.height - 2);
		image = IsrImageProcessor.Fill_rectangle_color_(image, bounds, this.color());
		return image.image();
	}
	/**
	 * Answer my color model.
	 * 
	 * @return sra.isr.interfaces.colors.IsrColorChoiceModel
	 */
	public IsrColorChoiceModel colorModel() {
		if (colorModel == null) {
			colorModel = new IsrColorChoiceSBH();
			colorModel.color_(Color.black);
			colorModel.addDependent_(this);
		}
		return colorModel;
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 */
	public void commit() {
		// this.freehandImage = new StImage(this.currentPixmap());
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @return java.awt.Image
	 */
	public Image currentPixmap() {
		if (currentPixmap == null) {
			currentPixmap = new StImage(this.defaultSize(), this.defaultSize()).image();
			this.clear();
		}
		return currentPixmap;
	}
	/**
	 * Answer brush cursor.
	 * 
	 * @return java.awt.Cursor
	 */
	public Cursor cursor() {
		return IsrCursors.BrushCursor();
	}
	/**
	 * Answer default plate size.
	 * 
	 * @return int
	 */
	public int defaultSize() {
		return 150;
	}
	/**
	 * Answer a default view.
	 *
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new IsrFreehandApplicationViewAwt();
		} else {
			return new IsrFreehandApplicationViewSwing();
		}
	}
	/**
	 * Answer my eraser button.
	 * 
	 * @return sra.isr.interfaces.button.IsrButtonModel
	 */
	public IsrButtonModel eraserButton() {
		final IsrFreehandModel self = this;
		StBlockClosure block = new StBlockClosure() {
			public Object value_(Object o) {
				self.eraserButtonAction();
				return null;
			}
		};
		return IsrButtonModel.Value_visual_action_(false, IsrCursors.EraserCursorImage(), block);
	}
	/**
	 * Set the eraser button action.
	 */
	public void eraserButtonAction() {
		this.clear();
	}
	/**
	 * Answer freehand image.
	 * 
	 * @return sra.smalltalk.StImage
	 */
	public StImage getImage() {
		this.commit();
		return new StImage(this.currentPixmap());
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @param gc java.awt.Graphics
	 */
	public void gstate_(Graphics gc) {
		gc.setColor(this.color());
		gc.setPaintMode();
	}
	/**
	 * Answer line width.
	 * 
	 * @return int
	 */
	public int lineWidth() {
		if (lineWidth == 0) {
			lineWidth = 4;
		}
		return lineWidth;
	}
	/**
	 * Answer the plate image.
	 *
	 * @return java.awt.Image
	 */
	public Image plate() {
		return this.currentPixmap();
	}
	/**
	 * Cleanup this object.
	 */
	public void release() {
		if (currentPixmap != null) {
			this.commit();
			currentPixmap.flush();
			currentPixmap = null;
		}
	}
	/**
	 * Create the Dialog for IsrFreehandModel with initial image.
	 *
	 * @return sra.smalltalk.StImage
	 */
	public static StImage Request() {
		return Request_(new StImage(16, 16).image());
	}
	/**
	 * Create the Dialog for IsrFreehandModel with image.
	 *
	 * @return StImage
	 * @param anImage java.awt.Image
	 */
	public static StImage Request_(Image anImage) {
		StImage result = null;
		final IsrFreehandModel freehandModel = new IsrFreehandModel();
		if (anImage == null) {
			freehandModel.setImage(new StImage(freehandModel.defaultSize(), freehandModel.defaultSize()).image());
		} else {
			StImage resizeImaeg = IsrImageProcessor.Icon_size_(new StImage(anImage), freehandModel.defaultSize());
			freehandModel.setImage(resizeImaeg.image());
		}
		Frame parent = new Frame();
		IsrFreehandApplicationView view = (IsrFreehandApplicationView) freehandModel.defaultView();
		view.setModel(freehandModel);
		Dialog dialog = new Dialog(parent, "Gauge", false);
		dialog.setResizable(false);
		dialog.add("Center", (Component) view);
		dialog.add("South", freehandModel._okCancelButtons(dialog));
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				freehandModel.cancel();
				e.getWindow().dispose();
			};
		});
		dialog.pack();
		try {
			Thread.sleep(0);
		} catch (Exception e) {
		}
		freehandModel.changed();
		_ShowAtMousePoint(dialog);
		if (freehandModel.accepted) {
			result = (StImage) freehandModel.getImage().copy();
		}
		return result;
	}
	/**
	 * Answer my scissors button.
	 * 
	 * @return sra.isr.interfaces.button.JunButtonModel
	 */
	public IsrButtonModel scissorsButton() {
		final IsrFreehandModel self = this;
		StBlockClosure block = new StBlockClosure() {
			public Object value_(Object o) {
				self.scissorsButtonAction();
				return null;
			}
		};
		Image icon = null;
		try {
			SystemInterface.Current();
			icon = IsrCursors.ScissorsCursorImage();
		} catch (Throwable e) {
			icon = this._disableIconImage(IsrCursors.ScissorsCursorImage());
		}
		return IsrButtonModel.Value_visual_action_(false, icon, block);
	}
	/**
	 * Set the scissors button action.
	 */
	public void scissorsButtonAction() {
		int size = this.defaultSize();
		StImage image = StImage._FromUser();
		Image iconImage = IsrImageProcessor.Icon_size_(image, size).image();
		Image anImage = this.currentPixmap();
		anImage.getGraphics().drawImage(iconImage, 0, 0, size, size, Color.white, null);
		this.setImage(anImage);
	}
	/**
	 * Set freehand image.
	 * 
	 * @param anImage java.awt.Image
	 */
	public void setImage(Image anImage) {
		this.currentPixmap = anImage;
		this.commit();
		this.changed_(StObject.$("plate"));
	}
	/**
	 * Answer my spuit button.
	 * 
	 * @return sra.isr.interfaces.button.JunButtonModel
	 */
	public IsrButtonModel spuitButton() {
		final IsrFreehandModel self = this;
		StBlockClosure block = new StBlockClosure() {
			public Object value_(Object o) {
				self.spuitButtonAction();
				return null;
			}
		};
		Image icon = null;
		try {
			SystemInterface.Current();
			icon = IsrCursors.SpuitCursorImage();
		} catch (Throwable e) {
			icon = this._disableIconImage(IsrCursors.SpuitCursorImage());
		}
		return IsrButtonModel.Value_visual_action_(false, icon, block);
	}
	/**
	 * Set the spuit button action.
	 */
	public void spuitButtonAction() {
		Color color = IsrColorSpuit.Request();
		if (color == null) {
			return;
		}
		this.colorModel().color_(color);
	}
	/**
	 * Update the receiver according to the change notification from color button.
	 *
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		StSymbol aSymbol = evt.getAspect();
		if ((aSymbol == $("xy")) || (aSymbol == $("z")) || (aSymbol == $("color"))) {
			this.colorButton().visual_(this.colorImage());
		}
	}
	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Freehand";
	}
}
