package sra.isr.interfaces.freehand;

import java.awt.*;
import sra.smalltalk.*;
import sra.isr.system.support.*;

/**
 * IsrFreehandViewAwt class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2000/02/18 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrFreehandViewAwt.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrFreehandViewAwt extends StViewCanvas implements IsrFreehandView {

	/**
	 * Answer my default controller
	 *
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new IsrFreehandController();
	}
	/**
	 * Answer a default model.
	 *
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new IsrFreehandModel();
	}
	/**
	 * Paint this border rectangle.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayOn_(Graphics aGraphics) {
		this.displayPlateOn_(aGraphics);
	}
	/**
	 * Display the plate using the Graphics.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayPlateOn_(Graphics aGraphics) {
		if (!this.isShowing())
			return;
		Rectangle box = this.plateArea();
		aGraphics.clipRect(box.x - 1, box.y - 1, box.width + 2, box.height + 2);
		aGraphics.drawImage(this.plate(), box.x, box.y, this);
		aGraphics.setColor(Color.black);
		aGraphics.drawRect(box.x - 1, box.y - 1, box.width + 1, box.height + 1);
	}
	/**
	 * Answer a Model.
	 *
	 * @return sra.isr.interfaces.freehand.IsrFreehandModel
	 */
	public IsrFreehandModel getModel() {
		return (IsrFreehandModel) this.model();
	}
	/**
	 * Get the plate image.
	 *
	 * @return java.awt.Image
	 */
	public Image plate() {
		return this.getModel().plate();
	}
	/**
	 * Get the plate bounds.
	 *
	 * @return java.awt.Rectangle
	 */
	public Rectangle plateArea() {
		Rectangle imageRect = new Rectangle(0, 0, this.plate().getWidth(this), this.plate().getHeight(this));
		imageRect.translate(10, 10);
		return imageRect;
	}
	/**
	 * Cleanup this object.
	 */
	public void release() {
		this.getModel().release();
		this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
	/**
	 * Model property accessing.
	 *
	 * @param aModel sra.isr.interfaces.freehand.IsrFreehandModel
	 */
	public void setModel(IsrFreehandModel aModel) {
		this.model_(aModel);
	}
	/**
	 * Update the receiver according to the change notification from the model.
	 *
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (!this.isShowing())
			return;
		if (evt.getAspect() == $("plate")) {
			Graphics gc = this.getGraphics();
			try {
				this.displayPlateOn_(gc);
			} finally {
				gc.dispose();
			}
		}
	}
}
