package sra.isr.interfaces.gauge;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import sra.smalltalk.*;
import sra.isr.system.support.*;

/**
 * IsrGaugeModel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2000/02/17 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrGaugeModel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrGaugeModel extends StApplicationModel {
	protected double gaugeValue;
	protected double[] gaugeRange;
	private boolean accepted = false;
	/**
	 * Construsctor
	 */
	public IsrGaugeModel() {
		super();
		initialize();
	}
	/**
	 * Defines OK/Cancel dialog buttons.
	 *
	 * @return java.awt.Panel
	 * @param target java.awt.Dialog
	 */
	public Panel _okCancelButtons(final Dialog target) {
		Panel panel = new Panel();
		final IsrGaugeModel this_ = this;
		panel.add("West", new Button("OK") {
			public boolean action(Event ev, Object what) {
				this_.accept();
				target.dispose();
				return true;
			}
		});
		panel.add("East", new Button("Cancel") {
			public boolean action(Event ev, Object what) {
				this_.cancel();
				target.dispose();
				return true;
			}
		});
		return panel;
	}
	/**
	 * Accept button pressed action(in dialog mode)
	 */
	public void accept() {
		accepted = true;
	}
	/**
	 * Accept button pressed action(in dialog mode). do nothing.
	 */
	public void cancel() {
	}
	/**
	 * Create a gauge view.
	 *
	 * @return sra.smalltalk.StView
	 */
	public StView createGaugeView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return IsrGaugeViewAwt.On_get_put_valuesMinMaxDivideRound_(this, StObject.$("value"), StObject.$("value_"), this.range());
		} else {
			IsrGaugeView view = new IsrGaugeViewSwing();
			view.on_get_put_valuesMinMaxDivideRound_(this, StObject.$("value"), StObject.$("value_"), this.range());
			return view;
		}
	}
	/**
	 * Answer a default view.
	 *
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		return this.createGaugeView();
	}
	/**
	 * Initialize this object.
	 */
	public IsrGaugeModel initialize() {
		return this;
	}
	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @exception java.io.IOException
	 */
	public void printOn_(Writer aWriter) throws IOException {
		super.printOn_(aWriter);
		aWriter.write(" on: ");
		aWriter.write(Double.toString(this.value()));
	}
	/**
	 * Answer the range values
	 *
	 * @return double[]
	 */
	public double[] range() {
		if ((gaugeRange == null) || (gaugeRange == new double[] {0.0, 0.0, 0.0, 0.0})) {
			gaugeRange = new double[] {0.0, 100.0, 10.0, 1.0};
		}
		return gaugeRange;
	}
	/**
	 * Set the range values
	 *
	 * @param anArray double[]
	 */
	public void range_(double[] anArray) {
		gaugeRange = anArray;
	}
	/**
	 * Create the Dialog for IsrGaugeModel with value.
	 *
	 * @return double
	 * @param aNumber double
	 */
	public static double Request_(double aNumber) {
		return Request_range_(aNumber, null);
	}
	/**
	 * Create the Dialog for IsrGaugeModel with value and range.
	 *
	 * @return double
	 * @param aNumber double
	 * @param anArray double[]
	 */
	public static double Request_range_(double aNumber, double[] anArray) {
		double result = 0;
		final IsrGaugeModel gaugeModel = IsrGaugeModel.Value_range_(aNumber, anArray);
		Frame parent = new Frame();
		IsrGaugeView view = (IsrGaugeView) gaugeModel.createGaugeView();
		Dialog dialog = new Dialog(parent, "Gauge", true);
		dialog.setResizable(false);
		dialog.add("Center", (Component) view);
		dialog.add("South", gaugeModel._okCancelButtons(dialog));
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				gaugeModel.cancel();
				e.getWindow().dispose();
			};
		});
		dialog.pack();
		try {
			Thread.sleep(0);
		} catch (Exception e) {
		}
		gaugeModel.changed();
		_ShowAtMousePoint(dialog);
		if (gaugeModel.accepted) {
			result = gaugeModel.value();
		}
		return result;
	}
	/**
	 * Answer the gauge value.
	 *
	 * @return double
	 */
	public double value() {
		return gaugeValue;
	}
	/**
	 * Create the gauge model with value.
	 *
	 * @return sra.isr.interfaces.gauge.IsrGaugeModel
	 * @param aNumber double
	 */
	public static IsrGaugeModel Value_(double aNumber) {
		return Value_range_(aNumber, null);
	}
	/**
	 * Set the gauge value.
	 *
	 * @param aNumber double
	 */
	public void value_(double aNumber) {
		gaugeValue = aNumber;
		this.changed_(StObject.$("value"));
	}
	/**
	 * Create the gauge model with value and range.
	 *
	 * @return sra.isr.interfaces.gauge.IsrGaugeModel
	 * @param aNumber double
	 * @param anArray double[]
	 */
	public static IsrGaugeModel Value_range_(double aNumber, double[] anArray) {
		IsrGaugeModel gaugeModel = new IsrGaugeModel();
		gaugeModel.value_(aNumber);
		gaugeModel.range_(anArray);
		return gaugeModel;
	}
	/**
	 * Answer a window title.
	 *
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Gauge";
	}
}
