package sra.isr.interfaces.gauge;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;

/**
 * IsrGaugePanel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/06/23 (by Mitsuhiro Asada)
 * 	@updated   2001/06/15 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrGaugePanel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrGaugePanel extends Panel {
	private IsrGaugeModel gaugeModel = null;
	private IsrGaugeView gaugeView = null;
	private Checkbox gaugeSwitch = null;
	/**
	 * IsrGaugePanel constructor.
	 */
	public IsrGaugePanel() {
		super();
		initialize();
	}
	/**
	 * IsrGaugePanel constructor.
	 * 
	 * @param layout java.awt.LayoutManager
	 */
	public IsrGaugePanel(LayoutManager layout) {
		super(layout);
		initialize();
	}
	/**
	 * connGaugeViewSetTarget:  (GaugeModel.this <--> GaugeView.model)
	 */
	private void _connGaugeViewSetTarget() {
		try {
			this.getGaugeView().setModel(this.getGaugeModel());
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Called whenever the part throws an exception.
	 * 
	 * @param exception java.lang.Throwable
	 */
	private void _handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}
	/**
	 * Initializes connections
	 * 
	 * @exception java.lang.Exception
	 */
	private void _initConnections() throws Exception {
		this._connGaugeViewSetTarget();
	}
	/**
	 * Return the gauge switch.
	 *
	 * @return boolean
	 */
	public boolean gaugeSwitch() {
		return this.getGaugeSwitch().getState();
	}
	/**
	 * Set the gauge switch.
	 *
	 * @param aBoolean boolean
	 */
	public void gaugeSwitch_(boolean aBoolean) {
		this.getGaugeSwitch().setState(aBoolean);
	}
	/**
	 * Return the gauge value.
	 *
	 * @return double
	 */
	public double gaugeValue() {
		return this.getGaugeModel().value();
	}
	/**
	 * Set the gauge value.
	 *
	 * @param value double
	 */
	public void gaugeValue_(double value) {
		this.getGaugeModel().value_(value);
	}
	/**
	 * Return the GaugeModel property value.
	 *
	 * @return sra.isr.interfaces.gauge.IsrGaugeModel
	 */
	public IsrGaugeModel getGaugeModel() {
		if (gaugeModel == null) {
			try {
				this.setGaugeModel(new IsrGaugeModel());
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return gaugeModel;
	}
	/**
	 * Return the GaugeSwitch property value.
	 * 
	 * @return java.awt.Checkbox
	 */
	public Checkbox getGaugeSwitch() {
		if (gaugeSwitch == null) {
			try {
				gaugeSwitch = new Checkbox();
				gaugeSwitch.setName("GaugeSwitch");
				gaugeSwitch.setLabel("gauge");
				gaugeSwitch.setBounds(2, 2, 130, 25);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return gaugeSwitch;
	}
	/**
	 * Return the GaugeView property value.
	 * 
	 * @return sra.isr.interfaces.gauge.IsrGaugeView
	 */
	protected IsrGaugeView getGaugeView() {
		if (gaugeView == null) {
			try {
				gaugeView = new IsrGaugeViewAwt();
				((Component) gaugeView).setName("GaugeViewAwt");
				((Component) gaugeView).setBounds(17, 26, 100, 180);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return gaugeView;
	}
	/**
	 * Initialize the class.
	 */
	private void initialize() {
		try {
			this.setName("IsrGaugePanel");
			this.setLayout(null);
			this.setSize(127, 212);
			this.add(this.getGaugeSwitch(), this.getGaugeSwitch().getName());
			this.add((Component) this.getGaugeView(), ((Component) this.getGaugeView()).getName());
			this._initConnections();
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Set the GaugeModel property value.
	 *
	 * @param model sra.isr.interfaces.gauge.IsrGaugeModel
	 */
	public void setGaugeModel(IsrGaugeModel model) {
		gaugeModel = model;
		this.getGaugeView().on_get_put_valuesMinMaxDivideRound_(model, StObject.$("value"), StObject.$("value_"), model.range());
	}
}
