package sra.isr.interfaces.gauge;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import sra.smalltalk.*;

/**
 * IsrGaugeView interface
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/02/17 (by Mitsuhiro Asada)
 * 	@updated   2000/03/13 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrGaugeView.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public interface IsrGaugeView extends StView {
	/**
	 * Display the border using the Graphics.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayBorderOn_(Graphics aGraphics);
	/**
	 * Display the scale using the Graphics.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayScalesOn_(Graphics aGraphics);
	/**
	 * Display the value using the Graphics.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayValueOn_(Graphics aGraphics);
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @return java.awt.Rectangle
	 */
	public Rectangle gaugeBorder();
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @return java.util.Vector
	 */
	public Vector gaugeScales();
	/**
	 * Answer a Model.
	 *
	 * @return sra.isr.interfaces.gauge.IsrGaugeModel
	 */
	public IsrGaugeModel getModel();
	/**
	 * Answer the gauge value.
	 *
	 * @return double
	 */
	public double getValue();
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @param aModel sra.isr.interfaces.gauge.IsrGaugeModel
	 * @param aSymbol sra.smalltalk.StSymbol
	 * @param aSymbol sra.smalltalk.StSymbol
	 * @param anArray double[]
	 */
	public void on_get_put_valuesMinMaxDivideRound_(IsrGaugeModel aModel, StSymbol getSymbol, StSymbol putSymbol, double[] anArray);
	/**
	 * Model property accessing.
	 *
	 * @param aModel sra.isr.interfaces.gauge.IsrGaugeModel
	 */
	public void setModel(IsrGaugeModel aModel);
	/**
	 * Set the gauge balue.
	 *
	 * @param cursorPoint Point
	 */
	public void setValue_(Point cursorPoint);
}
