package sra.isr.interfaces.nineColors;

import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.interfaces.colors.*;

/**
 * Isr9ColorsController class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2000/02/17 (by Mitsuhiro Asada)
 * 	@updated   2002/04/26 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: Isr9ColorsController.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class Isr9ColorsController extends StController implements MouseListener, MouseMotionListener {
	/**
	 * Add listeners.
	 *
	 * @param newView sra.smalltalk.StView
	 */
	protected void buildListener(StView newView) {
		Component aView = (Component) newView;
		aView.addMouseListener(this);
		aView.addMouseMotionListener(this);
	}
	/**
	 * Answer this model.
	 *
	 * @return sra.isr.interfaces.nineColors.Isr9ColorsModel
	 */
	protected Isr9ColorsModel getModel() {
		return (Isr9ColorsModel) this.model();
	}
	/**
	 * Answer this view.
	 *
	 * @return sra.isr.interfaces.nineColors.Isr9ColorsView interface
	 */
	protected Isr9ColorsView getView() {
		return (Isr9ColorsView) this.view();
	}
	/**
	 * Invoked when the mouse button has been moved on a component
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mouseMoved(MouseEvent event) {
		super.mouseMoved(event);
	}
	/**
	 * Invoked when a mouse button has been pressed. 
	 *
	 * @param event java.awt.event.MouseEvent
	 */
	public void mousePressed(MouseEvent event) {
		Isr9ColorsView view = this.getView();
		Point point = event.getPoint();
		// popup menu
		if (event.isMetaDown()) {
			view.showPopupMenu(point);
			return;
		}
		Isr9ColorsModel colorsModel = this.getModel();
		if (view.plateArea().contains(point)) {
			double x = ((double) (point.x - view.plateArea().getLocation().x)) / (view.plateArea().width - 1.0);
			double y = ((double) (point.y - view.plateArea().getLocation().y)) / (view.plateArea().height - 1.0);
			Isr2dPoint pointXy = new Isr2dPoint(x, y);
			int index = colorsModel.indexAt_(pointXy);
			if (index == -1)
				return;
			Color color = null;
			if (event.isShiftDown()) {
				color = IsrColorChoiceSBH.Request();
				if (color == null)
					return;
			} else {
				try {
					color = IsrColorSpuit.Request();
				} catch (Throwable e) {
					color = colorsModel.at_(index);
					if (color == null) {
						color = IsrColorChoiceSBH.Request();
					} else {
						color = IsrColorChoiceSBH.Request_(color);
					}
				}
				if (color == null)
					return;
			}
			colorsModel.at_put_(index, color);
			event.consume();
		}
	}
}
