package sra.isr.interfaces.nineColors;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import sra.smalltalk.*;

/**
 * Isr9ColorsPanel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/06/23 (by Mitsuhiro Asada)
 * 	@updated   2001/06/15 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: Isr9ColorsPanel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class Isr9ColorsPanel extends Panel {
	private Isr9ColorsModel nineColorsModel = null;
	private Isr9ColorsView nineColorsView = null;
	private Checkbox nineColorsSwitch = null;
	/**
	 * Isr9ColorsPanel constructor.
	 */
	public Isr9ColorsPanel() {
		super();
		initialize();
	}
	/**
	 * Isr9ColorsPanel constructor.
	 * 
	 * @param layout java.awt.LayoutManager
	 */
	public Isr9ColorsPanel(LayoutManager layout) {
		super(layout);
		initialize();
	}
	/**
	 * connPtoP1SetTarget:  (NineColorsModel.this <--> NineColorsView.model)
	 */
	private void _connNineColorsViewSetTarget() {
		try {
			this.getNineColorsView().setModel(this.getNineColorsModel());
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Called whenever the part throws an exception.
	 * 
	 * @param exception java.lang.Throwable
	 */
	private void _handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}
	/**
	 * Initializes connections
	 *
	 * @exception java.lang.Exception 
	 */
	private void _initConnections() throws Exception {
		this._connNineColorsViewSetTarget();
	}
	/**
	 * Return the nine colors property value.
	 *
	 * @return java.awt.Color[]
	 */
	public Color[] colors() {
		return this.getNineColorsModel().colors();
	}
	/**
	 * Set the nine colors model property value.
	 *
	 * @param colors java.awt.Color[]
	 */
	public void colors_(Color[] colors) {
		this.getNineColorsModel().colors_(colors);
	}
	/**
	 * Return the NineColorsModel property value.
	 *
	 * @return Isr9ColorsModel
	 */
	public Isr9ColorsModel getNineColorsModel() {
		if (nineColorsModel == null) {
			try {
				nineColorsModel = new Isr9ColorsModel();
			} catch (java.lang.Throwable e) {
				this._handleException(e);
			}
		}
		return nineColorsModel;
	}
	/**
	 * Return the NineColorsSwitch property value.
	 * 
	 * @return java.awt.Checkbox
	 */
	public Checkbox getNineColorsSwitch() {
		if (nineColorsSwitch == null) {
			try {
				nineColorsSwitch = new Checkbox();
				nineColorsSwitch.setName("NineColorsSwitch");
				nineColorsSwitch.setLabel("nine colors");
				nineColorsSwitch.setBounds(2, 2, 123, 25);
			} catch (java.lang.Throwable e) {
				this._handleException(e);
			}
		}
		return nineColorsSwitch;
	}
	/**
	 * Return the NineColorsView property value.
	 * 
	 * @return sra.isr.interfaces.nineColors.Isr9ColorsView
	 */
	protected Isr9ColorsView getNineColorsView() {
		if (nineColorsView == null) {
			try {
				nineColorsView = new Isr9ColorsViewAwt();
				((Component) nineColorsView).setName("NineColorsViewAwt");
				((Component) nineColorsView).setBounds(2, 26, 123, 123);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return nineColorsView;
	}
	/**
	 * Initialize the class.
	 */
	private void initialize() {
		try {
			this.setName("Isr9ColorsPanel");
			this.setLayout(null);
			this.setSize(127, 151);
			this.add(this.getNineColorsSwitch(), this.getNineColorsSwitch().getName());
			this.add((Component) this.getNineColorsView(), ((Component) this.getNineColorsView()).getName());
			this._initConnections();
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Return the nine colors switch property value.
	 * 
	 * @return boolean
	 */
	public boolean nineColorsSwitch() {
		return this.getNineColorsSwitch().getState();
	}
	/**
	 * Set the nine colors swich property value.
	 *
	 * @param boolean aBoolean
	 */
	public void nineColorsSwitch_(boolean aBoolean) {
		this.getNineColorsSwitch().setState(aBoolean);
	}
	/**
	 * Set the NineColorsModel property value.
	 *
	 * @param model Isr9ColorsModel
	 */
	public void setNineColorsModel(Isr9ColorsModel model) {
		nineColorsModel = model;
		this.getNineColorsView().setModel(model);
	}
}
