package sra.isr.interfaces.nineColors;

import java.awt.*;
import java.util.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.system.support.*;

/**
 * Isr9ColorsViewAwt class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2001/07/03 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: Isr9ColorsViewAwt.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class Isr9ColorsViewAwt extends StViewCanvas implements Isr9ColorsView {
	protected PopupMenu popupMenu;
	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setSize(this.plateArea().width + 22, this.plateArea().height + 22);
	}
	/**
	 * Answer the nine colors.
	 *
	 * @return java.awt.Color[]
	 */
	public Color[] colors() {
		return this.getModel().colors();
	}
	/**
	 * Answer my default controller
	 *
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new Isr9ColorsController();
	}
	/**
	 * Answer a default model.
	 *
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new Isr9ColorsModel();
	}
	/**
	 * Paint this border rectangle.
	 *
	 * @param graphics java.awt.Graphics
	 */
	public void displayOn_(Graphics aGraphics) {
		this.displayPlateOn_(aGraphics);
	}
	/**
	 * Display the plate using the Graphics.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayPlateOn_(Graphics aGraphics) {
		if (!this.isShowing())
			return;
		Rectangle box = this.plateArea();
		aGraphics.clipRect(box.x - 1, box.y - 1, box.width + 2, box.height + 2);
		this.plate().displayOn_at_(aGraphics, box.getLocation());
		aGraphics.setColor(Color.black);
		aGraphics.drawRect(box.x - 1, box.y - 1, box.width + 1, box.height + 1);
	}
	/**
	 * Answer a Model.
	 *
	 * @return sra.isr.interfaces.nineColors.Isr9ColorsModel
	 */
	public Isr9ColorsModel getModel() {
		return (Isr9ColorsModel) this.model();
	}
	/**
	 * Get the plate image.
	 *
	 * @return sra.smalltalk.StImage
	 */
	public StImage plate() {
		return this.getModel().plate();
	}
	/**
	 * Get the plate bounds.
	 *
	 * @return java.awt.Rectangle
	 */
	public Rectangle plateArea() {
		Rectangle imageRect = this.plate().bounds();
		imageRect.translate(10, 10);
		return imageRect;
	}
	/**
	 * Rebuild this component.
	 * 
	 * @param oldModel sra.smalltalk.StModel
	 * @param newModel sra.smalltalk.StModel
	 */
	protected void rebuildComponent(StModel oldModel, StModel newModel) {
		if (popupMenu != null) {
			this.remove(popupMenu);
		}
		StMenuModel menuModel = ((Isr9ColorsModel) newModel).defaultMenu();
		if (menuModel != null) {
			StMenu editMenu = (StMenu) menuModel.getItem("NineColorsMenu");
			popupMenu = StMenuViewForAwt.CreatePopupMenu(editMenu);
			this.add(popupMenu);
		}
	}
	/**
	 * Model property accessing.
	 *
	 * @param aModel sra.isr.interfaces.nineColors.Isr9ColorsModel
	 */
	public void setModel(Isr9ColorsModel aModel) {
		this.model_(aModel);
	}
	/**
	 * Open the menu at the specified point.
	 *
	 * @param aPoint java.awt.Point
	 */
	public void showPopupMenu(Point aPoint) {
		if (popupMenu != null) {
			if (this.plateArea().contains(aPoint)) {
				double x = ((double) (aPoint.x - this.plateArea().getLocation().x)) / (this.plateArea().width - 1.0);
				double y = ((double) (aPoint.y - this.plateArea().getLocation().y)) / (this.plateArea().height - 1.0);
				int index = this.getModel().indexAt_(new Isr2dPoint(x, y));
				popupMenu.getItem(0).setEnabled(this.getModel().at_(index) != null);
				popupMenu.getItem(1).setEnabled(this.getModel().at_(index) != null);
				popupMenu.getItem(2).setEnabled(this.getModel().bufferColor() != null);
				this.getModel().selectedArea_(index);
			} else {
				this.getModel().selectedArea_(-1);
				popupMenu.getItem(0).setEnabled(false);
				popupMenu.getItem(1).setEnabled(false);
				popupMenu.getItem(2).setEnabled(false);
			}
			popupMenu.show(this, aPoint.x, aPoint.y);
		}
	}
	/**
	 * Update the receiver according to the change notification from the model.
	 *
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (!this.isShowing())
			return;
		if (evt.getAspect() == $("colors")) {
			Graphics gc = this.getGraphics();
			try {
				this.displayPlateOn_(gc);
			} finally {
				gc.dispose();
			}
		}
	}
}
