package sra.isr.interfaces.progress;

import java.awt.*;
import sra.smalltalk.*;

/**
 * IsrProgress class
 * 
 * 	@author    nisinaka
 * 	@created   1998/11/06 (by nisinaka)
 * 	@updated   1999/08/05 (by nisinaka)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrProgress.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrProgress extends StApplicationModel {
	/**
	 * The progress value.
	 */
	protected float progressValue = 0;

	/**
	 * The progress message.
	 */
	protected String progressMessage = new String();

	/**
	 * The stop condition.
	 */
	protected boolean stopCondition;

	/**
	 * The rasing flag.
	 */
	protected boolean raisingFlag = false;
	/**
	 * Answer a default view.
	 * 
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new IsrProgressViewAwt();
		} else {
			return new IsrProgressViewSwing();
		}
	}
	/**
	 * Evaluate the BlockClosure.
	 *
	 * @return java.lang.Object
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public Object do_(StBlockClosure aBlock) {
		if (this.myDependents() == null || this.myDependents().isEmpty()) {
			IsrProgressView aView = (IsrProgressView) this.defaultView();
			aView.setPopupDialog(true);
			aView.model_(this);
		}
		Object result = null;
		//
		this.changed_($("start"));
		raisingFlag = true;
		try {
			result = this.run_(aBlock);
		} finally {
			this.changed_($("end"));
			raisingFlag = false;
		}
		return result;
	}
	/**
	 * Answer the progress message.
	 *
	 * @return java.lang.String
	 */
	public String message() {
		return progressMessage;
	}
	/**
	 * Set the progress message and update the receiver.
	 *
	 * @param aString java.lang.String
	 */
	public void message_(String aString) {
		String oldString = progressMessage;
		progressMessage = aString;
		if (!progressMessage.equals(oldString)) {
			this.changed_($("message"));
		}
	}
	/**
	 * Answer the current raising flag.
	 *
	 * @return boolean
	 */
	protected final boolean raisingFlag() {
		return raisingFlag;
	}
	/**
	 * Evaluate the block while in progress.
	 *
	 * @return java.lang.Object
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public Object run_(StBlockClosure aBlock) {
		Object result = null;
		if (aBlock.numArgs() == 0) {
			result = aBlock.value();
		} else {
			result = aBlock.value_(this);
		}
		return result;
	}
	/**
	 * Answer the progress value.
	 *
	 * @return float
	 */
	public float value() {
		return progressValue;
	}
	/**
	 * Set the progress value and update the receiver.
	 *
	 * @param normalizedNumber float
	 */
	public void value_(float normalizedNumber) {
		if (0 <= normalizedNumber && normalizedNumber <= 1) {
			float oldValue = progressValue;
			progressValue = normalizedNumber;
			if (progressValue != oldValue) {
				this.changed_($("value"));
			}
		}
	}
}
