package sra.isr.interfaces.spot;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.system.support.*;

/**
 * IsrSpotModel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2000/02/16 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrSpotModel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrSpotModel extends StApplicationModel {
	protected Isr2dPoint xy;
	protected static StImage Plate = null;
	private boolean accepted = false;
	/**
	 * Construsctor
	 */
	public IsrSpotModel() {
		super();
		initialize();
	}
	/**
	 * Defines OK/Cancel dialog buttons.
	 *
	 * @return java.awt.Panel
	 * @param target java.awt.Dialog
	 */
	public Panel _okCancelButtons(final Dialog target) {
		Panel panel = new Panel();
		final IsrSpotModel this_ = this;
		panel.add("West", new Button("OK") {
			public boolean action(Event ev, Object what) {
				this_.accept();
				target.dispose();
				return true;
			}
		});
		panel.add("East", new Button("Cancel") {
			public boolean action(Event ev, Object what) {
				this_.cancel();
				target.dispose();
				return true;
			}
		});
		return panel;
	}
	/**
	 * Accept button pressed action(in dialog mode)
	 */
	public void accept() {
		accepted = true;
	}
	/**
	 * Accept button pressed action(in dialog mode). do nothing.
	 */
	public void cancel() {
	}
	/**
	 * Answer a default view.
	 *
	 * @return sra.smalltalk.StView
	 */
	public StView defaultView() {
		if (GetDefaultViewMode() == VIEW_AWT) {
			return new IsrSpotViewAwt();
		} else {
			return new IsrSpotViewSwing();
		}
	}
	/**
	 * Initialize this object.
	 *
	 * @return sra.isr.geometry.basic.IsrSpotModel
	 */
	public IsrSpotModel initialize() {
		this.xy();
		return this;
	}
	/**
	 * Answer the plate image.
	 *
	 * @return sra.smalltalk.StImage
	 */
	public StImage plate() {
		return this.Plate();
	}
	/**
	 * Answer the plate image.
	 *
	 * @return sra.smalltalk.StImage
	 */
	static StImage Plate() {
		if (Plate == null) {
			int x, y;
			Color color;
			int interval = 101;
			int width, height;
			StImage image = new StImage(interval, interval);
			width = image.width();
			height = image.height();
			color = Color.getHSBColor((float) 0.0, (float) 0.0, (float) 0.82);
			for (y = 0; y < height; y++) {
				for (x = 0; x < width; x++) {
					image.setPixel(x, y, color.getRGB());
				}
			}
			color = Color.black;
			for (y = 0; y < height; y++) {
				for (x = 0; x < width; x++) {
					if ((x % 10 == 0) && (y % 10 == 0)) {
						image.setPixel(x, y, color.getRGB());
						if ((x == ((width - 1) / 2)) && (y == ((height - 1) / 2))) {
							Point[] array = {new Point(-1, 0), new Point(0, 1), new Point(1, 0), new Point(0, -1)};
							for (int i = 0; i < array.length; i++) {
								image.setPixel(x + array[i].x, y + array[i].y, color.getRGB());
							}
						}
					}
				}
			}
			Plate = image;
		}
		return Plate;
	}
	/**
	 * Print my string representation on aWriter.
	 * 
	 * @param aWriter java.io.Writer
	 * @exception java.io.IOException
	 */
	public void printOn_(Writer aWriter) throws IOException {
		super.printOn_(aWriter);
		aWriter.write(" on: ");
		aWriter.write(this.xy().toString());
	}
	/**
	 * Create the Dialog for IsrSpotModel with initial value.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint or null
	 */
	public static Isr2dPoint Request_() {
		return Request_(new Isr2dPoint(0.0, 0.0));
	}
	/**
	 * Create the Dialog for IsrSpotModel with value.
	 *
	 * @return sra.isr.geometry.basic.Jun2dPoint or null
	 * @param aPoint sra.isr.geometry.basic.Jun2dPoint
	 */
	public static Isr2dPoint Request_(Isr2dPoint aPoint) {
		final IsrSpotModel spotModel = new IsrSpotModel();
		if (aPoint == null) {
			spotModel.xy_(new Isr2dPoint(0.0, 0.0));
		} else {
			spotModel.xy_(aPoint);
		}
		Isr2dPoint result = null;
		Frame parent = new Frame();
		IsrSpotView view = (IsrSpotView) spotModel.defaultView();
		view.setModel(spotModel);
		Dialog dialog = new Dialog(parent, "Area", true);
		dialog.setResizable(false);
		dialog.add("Center", (Component) view);
		dialog.add("South", spotModel._okCancelButtons(dialog));
		dialog.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				spotModel.cancel();
				e.getWindow().dispose();
			};
		});
		dialog.pack();
		try {
			Thread.sleep(0);
		} catch (Exception e) {
		}
		spotModel.changed();
		_ShowAtMousePoint(dialog);
		if (spotModel.accepted) {
			result = spotModel.xy();
		}
		return result;
	}
	/**
	 * Answer the spot view.
	 *
	 * @return sra.isr.interfaces.spot.IsrSpotViewAwt
	 */
	public IsrSpotViewAwt spotView() {
		IsrSpotViewAwt view = new IsrSpotViewAwt();
		view.model_(this);
		return view;
	}
	/**
	 * Answer a window title.
	 * 
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return "Spot";
	}
	/**
	 * Answer the spot value.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint
	 */
	public Isr2dPoint xy() {
		if (xy == null) {
			xy = new Isr2dPoint(0, 0);
		}
		return xy;
	}
	/**
	 * Create the spot model with spot value.
	 *
	 * @return sra.isr.interfaces.spot.IsrSpotModel
	 * @param aPoint sra.isr.geometry.basic.Isr2dPoint
	 */
	public static IsrSpotModel Xy_(Isr2dPoint aPoint) {
		IsrSpotModel model = new IsrSpotModel();
		model.xy_(aPoint);
		return model;
	}
	/**
	 * Set the spot value.
	 *
	 * @param aPoint sra.isr.geometry.basic.Isr2dPoint
	 */
	public void xy_(Isr2dPoint aPoint) {
		if (xy != aPoint) {
			xy = aPoint;
			this.changed_(StObject.$("xy"));
		}
	}
}
