package sra.isr.interfaces.spot;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import sra.isr.geometry.basic.*;

/**
 * IsrSpotPanel class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/06/23 (by Mitsuhiro Asada)
 * 	@updated   2001/06/15 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrSpotPanel.java,v 4.2 2002/07/11 09:25:04 m-asada Exp $
 */
public class IsrSpotPanel extends Panel {
	private IsrSpotModel spotModel = null;
	private IsrSpotView spotView = null;
	private Checkbox spotSwitch = null;
	/**
	 * IsrSpotPanel constructor.
	 */
	public IsrSpotPanel() {
		super();
		initialize();
	}
	/**
	 * IsrSpotPanel constructor.
	 *
	 * @param layout java.awt.LayoutManager
	 */
	public IsrSpotPanel(LayoutManager layout) {
		super(layout);
		initialize();
	}
	/**
	 * connPtoP1SetTarget:  (SpotModel.this <--> SpotView.model)
	 */
	private void _connSpotViewSetTarget() {
		try {
			this.getSpotView().setModel(this.getSpotModel());
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Called whenever the part throws an exception.
	 * 
	 * @param exception java.lang.Throwable
	 */
	private void _handleException(Throwable exception) {
		exception.printStackTrace(System.out);
	}
	/**
	 * Initializes connections
	 * 
	 * @exception java.lang.Exception
	 */
	private void _initConnections() throws Exception {
		this._connSpotViewSetTarget();
	}
	/**
	 * Return the AreaModel property value.
	 *
	 * @return sra.isr.interfaces.spot.IsrSpotModel
	 */
	public IsrSpotModel getSpotModel() {
		if (spotModel == null) {
			try {
				spotModel = new IsrSpotModel();
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return spotModel;
	}
	/**
	 * Return the SpotSwitch property value.
	 * 
	 * @return java.awt.Checkbox
	 */
	public Checkbox getSpotSwitch() {
		if (spotSwitch == null) {
			try {
				spotSwitch = new Checkbox();
				spotSwitch.setName("SpotSwitch");
				spotSwitch.setLabel("spot");
				spotSwitch.setBounds(2, 2, 123, 25);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return spotSwitch;
	}
	/**
	 * Return the SpotView property value.
	 * 
	 * @return sra.isr.interfaces.spot.IsrSpotView
	 */
	protected IsrSpotView getSpotView() {
		if (spotView == null) {
			try {
				spotView = new IsrSpotViewAwt();
				((Component) spotView).setName("SpotViewAwt");
				((Component) spotView).setBounds(2, 26, 123, 123);
			} catch (Throwable e) {
				this._handleException(e);
			}
		}
		return spotView;
	}
	/**
	 * Initialize the class.
	 */
	private void initialize() {
		try {
			this.setName("IsrSpotPanel");
			this.setLayout(null);
			this.setSize(127, 151);
			this.add(this.getSpotSwitch(), this.getSpotSwitch().getName());
			this.add((Component) this.getSpotView(), ((Component) this.getSpotView()).getName());
			this._initConnections();
		} catch (Throwable e) {
			this._handleException(e);
		}
	}
	/**
	 * Set the AreaModel property value.
	 *
	 * @param model sra.isr.interfaces.spot.IsrSpotModel
	 */
	public void setSpotModel(IsrSpotModel model) {
		spotModel = model;
		this.getSpotView().setModel(model);
	}
	/**
	 * Return the spot.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint
	 */
	public Isr2dPoint spot() {
		return this.xy();
	}
	/**
	 * Set the spot.
	 *
	 * @param point sra.isr.geometry.basic.Isr2dPoint
	 */
	public void spot_(Isr2dPoint point) {
		this.xy_(point);
	}
	/**
	 * Return the spot switch.
	 *
	 * @return boolean
	 */
	public boolean spotSwitch() {
		return this.getSpotSwitch().getState();
	}
	/**
	 * Set the spot switch.
	 *
	 * @param aBoolean boolean
	 */
	public void spotSwitch_(boolean aBoolean) {
		this.getSpotSwitch().setState(aBoolean);
	}
	/**
	 * Return the spot value.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint
	 */
	public Isr2dPoint xy() {
		return this.getSpotModel().xy();
	}
	/**
	 * Set the spot value.
	 *
	 * @param point sra.isr.geometry.basic.Isr2dPoint
	 */
	public void xy_(Isr2dPoint point) {
		this.getSpotModel().xy_(point);
	}
}
