package sra.isr.interfaces.spot;

import java.awt.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.interfaces.button.*;
import sra.isr.system.support.*;

/**
 * IsrSpotViewSwing class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/03/13 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrSpotViewSwing.java,v 4.2 2002/07/11 09:25:05 m-asada Exp $
 */
public class IsrSpotViewSwing extends StViewJPanel implements IsrSpotView {
	/**
	 * Build this component.
	 */
	protected void buildComponent() {
		this.setPreferredSize(new Dimension(this.plateArea().width + 22, this.plateArea().height + 22));
	}
	/**
	 * Answer my default controller
	 *
	 * @return sra.smalltalk.StController
	 */
	protected StController defaultController() {
		return new IsrSpotController();
	}
	/**
	 * Answer a default model.
	 *
	 * @return sra.smalltalk.StModel
	 */
	protected StModel defaultModel() {
		return new IsrSpotModel();
	}
	/**
	 * Paint this border rectangle.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayOn_(Graphics aGraphics) {
		this.displayPlateOn_(aGraphics);
		this.displayXYOn_(aGraphics);
	}
	/**
	 * Display the plate using the Graphics.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayPlateOn_(Graphics aGraphics) {
		if (!this.isShowing())
			return;
		Rectangle box = this.plateArea();
		aGraphics.clipRect(box.x - 1, box.y - 1, box.width + 2, box.height + 2);
		this.plate().displayOn_at_(aGraphics, box.getLocation());
		aGraphics.setColor(Color.black);
		aGraphics.drawRect(box.x - 1, box.y - 1, box.width + 1, box.height + 1);
	}
	/**
	 * Display the point using the Graphics.
	 *
	 * @param aGraphics java.awt.Graphics
	 */
	public void displayXYOn_(Graphics aGraphics) {
		if (!this.isShowing())
			return;
		Rectangle box = this.plateArea();
		aGraphics.clipRect(box.x - 1, box.y - 1, box.width + 2, box.height + 2);
		Point point = new Point((int) (this.xy().x() * (box.width - 1)), (int) (this.xy().y() * (box.height - 1)));
		Point globalPoint = new Point(point);
		globalPoint.translate(box.x, box.y);
		aGraphics.setColor(Color.white);
		for (int i = 0; i < 360; i += 15) {
			aGraphics.drawArc(globalPoint.x - 4, globalPoint.y - 4, 8, 8, i, 15);
			if (aGraphics.getColor() == Color.white) {
				aGraphics.setColor(Color.black);
			} else {
				aGraphics.setColor(Color.white);
			}
		}
	}
	/**
	 * Answer a Model.
	 *
	 * @return sra.isr.interfaces.spot.IsrSpotModel
	 */
	public IsrSpotModel getModel() {
		return (IsrSpotModel) this.model();
	}
	/**
	 * Get the plate image.
	 *
	 * @return sra.smalltalk.StImage
	 */
	public StImage plate() {
		return this.getModel().plate();
	}
	/**
	 * Get the plate bounds.
	 *
	 * @return java.awt.Rectangle
	 */
	public Rectangle plateArea() {
		Rectangle imageRect = this.plate().bounds();
		imageRect.translate(10, 10);
		return imageRect;
	}
	/**
	 * Model property accessing.
	 *
	 * @param aModel sra.isr.interfaces.spot.IsrSpotModel
	 */
	public void setModel(IsrSpotModel aModel) {
		this.model_(aModel);
	}
	/**
	 * Update the receiver according to the change notification from the model.
	 *
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (!this.isShowing())
			return;
		if (evt.getAspect() == $("xy")) {
			Graphics gc = this.getGraphics();
			try {
				this.displayPlateOn_(gc);
				this.displayXYOn_(gc);
			} finally {
				gc.dispose();
			}
		}
	}
	/**
	 * Answer the spot value.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint
	 */
	public Isr2dPoint xy() {
		return this.getModel().xy();
	}
}
