package sra.isr.servlet.basic;

import java.io.*;
import java.util.*;
import javax.servlet.http.*;
import com.oreilly.servlet.multipart.*;

/**
 * IsrParamParser class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2001/06/16 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrParamParser.java,v 4.2 2002/07/11 09:25:05 m-asada Exp $
 */
public class IsrParamParser {
	private static int MAX_POST_SIZE = 1024 * 1024 * 5; //5MB
	protected static String ENCODING = null;
	protected String encoding;
	protected Hashtable parameters;
	protected Hashtable files;
	/**
	 * The default constructor.
	 *
	 * @param request javax.servlet.http.HttpServletRequest
	 * @exception sra.isr.servlet.basic.IsrServletException
	 */
	public IsrParamParser(HttpServletRequest request) throws IsrServletException {
		this(request, null, MAX_POST_SIZE);
	}
	/**
	 * The default constructor with limits the upload size.
	 *
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param maxSize int
	 * @exception sra.isr.servlet.basic.IsrServletException
	 */
	public IsrParamParser(HttpServletRequest request, int maxSize) throws IsrServletException {
		this(request, null, maxSize);
	}
	/**
	 * The default constructor with encoding string.
	 *
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param encoding java.lang.String
	 * @exception sra.isr.servlet.basic.IsrServletException
	 */
	public IsrParamParser(HttpServletRequest request, String encoding) throws IsrServletException {
		this(request, encoding, MAX_POST_SIZE);
	}
	/**
	 * The default constructor with encoding string and limits the upload size.
	 *
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param encoding java.lang.String
	 * @param maxSize int
	 * @exception sra.isr.servlet.basic.IsrServletException
	 */
	public IsrParamParser(HttpServletRequest request, String encoding, int maxSize) throws IsrServletException {
		super();
		this.setEncoding(encoding);
		String contentType = request.getContentType();
		try {
			if (contentType == null || contentType.indexOf("boundary=") == -1) {
				this.parseSinglePart(request);
			} else {
				this.parseMultiPart(request, maxSize);
			}
		} catch (IOException e) {
			throw new IsrServletException(e);
		}
	}
	/**
	 * Answer the default encoding string.
	 *
	 * @return java.lang.String
	 */
	public static String DefaultEncoding() {
		// return "JISAutoDetect";
		return "ISO-8859-1";
	}
	/**
	 * Answer the current encoding string.
	 * 
	 * @return java.lang.String
	 */
	public static String Encoding() {
		if (ENCODING == null) {
			ENCODING = System.getProperty("default.client.encoding");
			if (ENCODING == null) {
				ENCODING = DefaultEncoding();
			}
		}
		return ENCODING;
	}
	/**
	 * Answer the current encoding string.
	 * 
	 * @return java.lang.String
	 */
	private String getEncoding() {
		if (encoding == null) {
			encoding = this.Encoding();
		}
		return encoding;
	}
	/**
	 * Answer the file attribute of form.
	 *
	 * @return java.util.Hashtable
	 * @param name java.lang.String
	 */
	public Hashtable getFile(String name) {
		try {
			return (Hashtable) this.getFiles().get(name);
		} catch (Exception e) {
			return null;
		}
	}
	/**
	 * Answer the file names of form.
	 *
	 * @return java.util.Enumeration
	 */
	public Enumeration getFileNames() {
		return this.getFiles().keys();
	}
	/**
	 * Answer the file attributes of form.
	 *
	 * @return java.util.Hashtable
	 */
	public Hashtable getFiles() {
		if (files == null) {
			files = new Hashtable();
		}
		return files;
	}
	/**
	 * Answer the text attribute of form.
	 *
	 * @return java.lang.String
	 * @param name java.lang.String
	 */
	public String getParameter(String name) {
		try {
			return (String) this.getParameters().get(name);
		} catch (Exception e) {
			return null;
		}
	}
	/**
	 * Answer the text attributes of form.
	 *
	 * @return java.util.Hashtable
	 */
	public Hashtable getParameters() {
		if (parameters == null) {
			parameters = new Hashtable();
		}
		return parameters;
	}
	/**
	 * Answer the text names of form.
	 *
	 * @return java.util.Enumeration
	 */
	public Enumeration getParametersNames() {
		return this.getParameters().keys();
	}
	/**
	 * The parse of Multi Part From data.
	 *
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param maxSize int
	 * @exception java.io.IOException The exception description.
	 */
	private void parseMultiPart(HttpServletRequest request, int maxSize) throws IOException {
		MultipartParser parser = new MultipartParser(request, maxSize, true, true);
		Part part;
		while ((part = parser.readNextPart()) != null) {
			String name = part.getName();
			if (part.isParam() == true) {
				ParamPart paramPart = (ParamPart) part;
				String value = paramPart.getStringValue(this.getEncoding());
				this.getParameters().put(name, value);
			} else if (part.isFile()) {
				FilePart filePart = (FilePart) part;
				if (filePart.getFileName() != null && filePart.getFileName().length() != 0) {
					byte[] file = null;
					ByteArrayOutputStream byteStream = null;
					try {
						byteStream = new ByteArrayOutputStream();
						filePart.writeTo(byteStream);
						byteStream.flush();
						file = byteStream.toByteArray();
					} finally {
						if (byteStream != null) {
							byteStream.close();
							byteStream = null;
						}
					}
					Hashtable table = new Hashtable();
					table.put("name", name);
					table.put("filename", filePart.getFileName());
					table.put("content-type", filePart.getContentType());
					table.put("content", file);
					this.getFiles().put(name, table);
				}
			}
		}
	}
	/**
	 * Insert the method's description here.
	 * 
	 * @param request javax.servlet.http.HttpServletRequest
	 * @exception java.io.IOException
	 */
	private void parseSinglePart(HttpServletRequest request) throws IOException {
		Enumeration e = request.getParameterNames();
		while (e.hasMoreElements()) {
			String key = (String) e.nextElement();
			String[] values = request.getParameterValues(key);
			this.getParameters().put(key, new String(values[0].getBytes("8859_1"), this.getEncoding()));
		}
	}
	/**
	 * Set the current encoding string.
	 * 
	 * @param newEncoding java.lang.String
	 */
	private void setEncoding(String newEncoding) {
		encoding = newEncoding;
	}
}
