package sra.isr.servlet.basic;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.sun.mail.util.BASE64DecoderStream;
import sra.isr.image.database.*;
import sra.isr.image.objects.*;
import sra.isr.system.support.*;

/**
 * IsrServlet class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/12/11 (by Mitsuhiro Asada)
 * 	@updated   2001/10/22 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrServlet.java,v 4.2 2002/07/11 09:25:05 m-asada Exp $
 */
public abstract class IsrServlet extends HttpServlet {
	protected static String ENCODING = null;
	protected IsrImageDatabase database = null;
	/**
	 * Answer the content type.
	 *
	 * @return java.lang.String
	 */
	public String contentType() {
		return "text/html; charset=" + this.Encoding();
	}
	/**
	 * Get username and password from Http Authorization String.
	 * 
	 * @return java.lang.String[]
	 * @param auth java.lang.String
	 */
	public String[] decodeAuthorizationString(String auth) {
		String[] result = new String[2];
		if (auth != null) {
			byte[] bytes = BASE64DecoderStream.decode(auth.substring(auth.indexOf(' ') + 1).getBytes());
			auth = new String(bytes);
			int length = auth.indexOf(0);
			if (length >= 0) {
				auth = auth.substring(0, length);
			}
			int position = auth.indexOf(':');
			result[0] = auth.substring(0, position);
			result[1] = auth.substring(position + 1);
		}
		return result;
	}
	/**
	 * Answer the default encoding string.
	 *
	 * @return java.lang.String
	 */
	public static String DefaultEncoding() {
		// return "UTF-8";
		return "ISO-8859-1";
	}
	/**
	 * Answer the default DOCTYPE string.
	 *
	 * @return java.lang.String
	 */
	public String doctype() {
		return "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">";
	}
	/**
	 * Process incoming HTTP GET requests 
	 * 
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param response javax.servlet.http.HttpServletResponse
	 * @exception javax.servlet.ServletException
	 * @exception java.io.IOException
	 */
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			this.doGetProcess(request, response);
		} catch (IsrServletException exception) {
			this.setError(response, exception);
		}
	}
	/**
	 * Process the HTTP GET request.
	 * 
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param response javax.servlet.http.HttpServletResponse
	 * @exception sra.isr.servlet.IsrServletException
	 * @exception javax.servlet.ServletException
	 * @exception java.io.IOException
	 */
	protected void doGetProcess(HttpServletRequest request, HttpServletResponse response) throws IsrServletException, ServletException, IOException {
	}
	/**
	 * Process incoming HTTP POST request.
	 * 
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param response javax.servlet.http.HttpServletResponse
	 * @exception javax.servlet.ServletException
	 * @exception java.io.IOException
	 */
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			this.doPostProcess(request, response);
		} catch (IsrServletException exception) {
			this.setError(response, exception);
		}
	}
	/**
	 * Process the HTTP POST request.
	 * 
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param response javax.servlet.http.HttpServletResponse
	 * @exception sra.isr.servlet.IsrServletException
	 * @exception javax.servlet.ServletException
	 * @exception java.io.IOException
	 */
	protected void doPostProcess(HttpServletRequest request, HttpServletResponse response) throws IsrServletException, ServletException, IOException {
	}
	/**
	 * Answer the current encoding string.
	 *
	 * @return java.lang.String
	 */
	public static String Encoding() {
		if (ENCODING == null) {
			ENCODING = System.getProperty("file.encoding");
			if (ENCODING == null) {
				ENCODING = DefaultEncoding();
			}
		}
		return ENCODING;
	}
	/**
	 * Get an instance of image database.
	 * 
	 * @return sra.isr.web.database.IsrImageDatabase
	 */
	protected IsrImageDatabase getDatabase() {
		if (database == null) {
			database = IsrImageDatabase.GetInstance(this.loadProperties());
		}
		return database;
	}
	/**
	 * Get this package name.
	 * 
	 * @return java.lang.String
	 */
	public String getPackageName() {
		String name = this.getClass().getName();
		int index = name.lastIndexOf('.');
		return name.substring(0, index);
	}
	/**
	 * Get the specified parameter from the request and decode it.
	 *
	 * @return java.lang.String
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param name java.lang.String
	 * @exception java.io.IOException
	 * @exception javax.servlet.ServletException
	 */
	protected String getParameter(HttpServletRequest request, String name) throws IOException, ServletException {
		String aString = request.getParameter(name);
		if (aString == null || aString.length() == 0) {
			throw new ServletException("The property '" + name + "' is missing");
		}
		return new String(aString.getBytes("8859_1"), IsrParamParser.Encoding());
	}
	/**
	 * Get the param perser.
	 *
	 * @return sra.isr.servlet.basic.IsrParamParser
	 * @param request javax.servlet.http.HttpServletRequest
	 * @exception sra.isr.servlet.basic.IsrServletException
	 */
	protected IsrParamParser getParamParser(HttpServletRequest request) throws IsrServletException {
		return new IsrParamParser(request);
	}
	/**
	 * Initialize this servlet.
	 *
	 * @exception javax.servlet.ServletException
	 */
	public void init() throws ServletException {
		super.init();

		// J2SDK 1.4 supports headless operation by setting the 
		// "java.awt.headless" system property to value "true".
		System.setProperty("java.awt.headless", "true");
	}
	/**
	 * Answer <code>true</code> if authenticate the user;
	 * <code>false</code> if other cases.
	 *
	 * @return boolean
	 * @param user java.lang.String
	 * @param password java.lang.String
	 */
	protected boolean isAuthenticate(String user, String password) {
		/*
		if (user != null && user.equals("isr4java") && password != null && password.equals("isr4java")) {
			return true;
		}
		return false;
		*/

		return true;
	}
	/**
	 * Authenticate the user with Http Authorization.
	 *
	 * @return boolean
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param response javax.servlet.http.HttpServletResponse
	 */
	protected boolean isAuthenticate(HttpServletRequest request, HttpServletResponse response) {
		String[] strings = this.decodeAuthorizationString(request.getHeader("Authorization"));
		String user = strings[0];
		String password = strings[1];

		if (this.isAuthenticate(user, password) == false) {
			response.setHeader("WWW-Authenticate", "BASIC realm=\"" + IsrSystem.System() + IsrSystem.Version() + "\"");
			response.setStatus(response.SC_UNAUTHORIZED);
			return false;
		}
		return true;
	}
	/**
	 * Answer the debug mode.
	 *
	 * @return boolean
	 */
	public boolean isDebug() {
		return false;
	}
	/**
	 * Get an properties for the servlet context.
	 *
	 * @return java.util.Properties
	 */
	public Properties loadProperties() {
		Properties properties = new Properties();
		InputStream stream = null;
		try {
			stream = this.getServletContext().getResourceAsStream("WEB-INF/isr.properties");
			properties.load(stream);
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (stream != null) {
					stream.close();
					stream = null;
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return properties;
	}
	/**
	 * Logging Exception.
	 * 
	 * @param message java.lang.String
	 * @param exception java.lang.Exception
	 */
	public void log(String message, Exception exception) {
		exception.printStackTrace();
		this.getServletContext().log(message, exception);
	}
	/**
	 * Set the body of the response.
	 * This method should be called after setting the status.
	 *
	 * @param response javax.servlet.http.HttpServletResponse
	 * @param bytes byte[]
	 * @exception java.io.IOException
	 */
	protected void setBody(HttpServletResponse response, byte[] bytes) throws IOException {
		response.setContentType(this.contentType());
		//
		InputStream in = null;
		OutputStream out = null;
		try {
			out = new BufferedOutputStream(response.getOutputStream());
			out.write(bytes);
		} finally {
			if (in != null) {
				in.close();
			}
			if (out != null) {
				out.flush();
				out.close();
			}
		}
	}
	/**
	 * Set the body of the response.
	 * This method should be called after setting the status.
	 *
	 * @param response javax.servlet.http.HttpServletResponse
	 * @param body java.lang.String
	 * @exception java.io.IOException
	 */
	protected void setBody(HttpServletResponse response, File file) throws IOException {
		response.setContentType(this.contentType());
		//
		InputStream in = null;
		OutputStream out = null;
		try {
			in = new BufferedInputStream(new FileInputStream(file));
			out = new BufferedOutputStream(response.getOutputStream());
			//
			byte[] buffer = new byte[4096];
			int length;
			while ((length = in.read(buffer)) != -1) {
				out.write(buffer, 0, length);
			}
		} finally {
			if (in != null) {
				in.close();
			}
			if (out != null) {
				out.flush();
				out.close();
			}
		}
	}
	/**
	 * Set the body of the response.
	 * This method should be called after setting the status.
	 *
	 * @param response javax.servlet.http.HttpServletResponse
	 * @param body java.lang.String
	 * @exception java.io.IOException
	 */
	protected void setBody(HttpServletResponse response, String body) throws IOException {
		response.setContentType(this.contentType());
		String encoding = response.getCharacterEncoding();
		//
		OutputStreamWriter osw = new OutputStreamWriter(response.getOutputStream(), encoding);
		PrintWriter pw = new PrintWriter(osw);
		try {
			pw.print(body);
		} finally {
			pw.flush();
			pw.close();
		}
	}
	/**
	 * Set the body of the response.
	 * This method should be called after setting the status.
	 *
	 * @param response javax.servlet.http.HttpServletResponse
	 * @param imageInfo sra.isr.image.objects.IsrImageFileInfo
	 * @exception java.io.IOException
	 */
	protected void setBody(HttpServletResponse response, IsrImageFileInfo imageInfo) throws IOException {
		response.setContentType(imageInfo.getContentType());
		//
		InputStream in = null;
		OutputStream out = null;
		try {
			out = new BufferedOutputStream(response.getOutputStream());
			out.write(imageInfo.getBytes());
		} finally {
			if (in != null) {
				in.close();
			}
			if (out != null) {
				out.flush();
				out.close();
			}
		}
	}
	/**
	 * Set the error message of the response.
	 *
	 * @param response javax.servlet.http.HttpServletResponse
	 * @param exception sra.isr.servlet.IsrServletException
	 * @exception java.io.IOException
	 */
	protected void setError(HttpServletResponse response, IsrServletException exception) throws IOException {
		// Logging exception.
		if (this.isDebug() == true) {
			this.log(exception.getMessage(), exception);
		}

		// Set status code.
		int statusCode = exception.getStatusCode();
		if (statusCode != 0) {
			response.setStatus(statusCode);
		} else {
			response.setStatus(exception.STATUS_INTERNAL_SERVER_ERROR);
		}

		// Set header of no-cache.
		this.setNoCache(response);

		// Set encoding type.
		response.setContentType("text/html; charset=" + this.Encoding());
		String encoding = response.getCharacterEncoding();

		// Set body.
		OutputStreamWriter osw = new OutputStreamWriter(response.getOutputStream(), encoding);
		PrintWriter pw = new PrintWriter(osw);
		try {
			pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">");
			pw.println("<HTML>");
			pw.println("<HEAD>");
			pw.println("<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=" + encoding + "\">");
			pw.println("<TITLE>ImageSearcher for Web [Fatal Error]</TITLE>");
			pw.println("</HEAD>");
			pw.println("");
			pw.println("<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\">");
			pw.println("<H4>The '" + this.getServletName() + "' returns the following message.</H4>");
			pw.println("<P>" + exception.getMessage() + "</P>");
			if (this.isDebug() == true) {
				pw.println("");
				pw.println("<BLOCKQUOTE><PRE>");
				exception.printStackTrace(pw);
				pw.println("</PRE></BLOCKQUOTE>");
			}
			pw.println("</BODY>");
			pw.println("</HTML>");
		} finally {
			pw.flush();
			pw.close();
		}
	}
	/**
	 * Set header of no-cache.
	 *
	 * @param response javax.servlet.http.HttpServletResponse
	 */
	protected void setNoCache(HttpServletResponse response) {
		response.setHeader("Pragma", "no-cache");
		response.setHeader("Cache-Control", "no-cache");
		response.setDateHeader("Expires", 0);
	}
}
