package sra.isr.servlet.basic;

import java.io.*;

/**
 * IsrServletException class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2001/02/04 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrServletException.java,v 4.2 2002/07/11 09:25:05 m-asada Exp $
 */
public class IsrServletException extends Exception {
	protected Throwable exception;
	protected int statusCode = 0;

	// Server status codes; see RFC 2068.
	public final static int STATUS_OK = 200;
	public final static int STATUS_BAD_REQUEST = 400;
	public final static int STATUS_INTERNAL_SERVER_ERROR = 500;
	/**
	 * Create a new instance of <code>IsrServletException</code>.
	 */
	public IsrServletException() {
		super();
	}
	/**
	 * Create a new instance of <code>IsrServletException</code>.
	 *
	 * @param message java.lang.String
	 */
	public IsrServletException(String message) {
		super(message);
	}
	/**
	 * Create a new instance of <code>IsrServletException</code> with <code>status</code>.
	 *
	 * @param message java.lang.String
	 * @param status int
	 */
	public IsrServletException(String message, int status) {
		super(message);
		statusCode = status;
	}
	/**
	 * Create a new instance of <code>IsrServletException</code>.
	 *
	 * @param message java.lang.String
	 * @param throwable java.lang.Throwable
	 */
	public IsrServletException(String message, Throwable throwable) {
		super(message);
		exception = throwable;
	}
	/**
	 * Create a new instance of <code>IsrServletException</code> with <code>status</code>.
	 *
	 * @param message java.lang.String
	 * @param throwable java.lang.Throwable
	 * @param status int
	 */
	public IsrServletException(String message, Throwable throwable, int status) {
		super(message);
		exception = throwable;
		statusCode = status;
	}
	/**
	 * Create a new instance of <code>IsrServletException</code>.
	 * 
	 * @param throwable java.lang.Throwable
	 */
	public IsrServletException(Throwable throwable) {
		super(throwable.getMessage());
		exception = throwable;
	}
	/**
	 * Create a new instance of <code>IsrServletException</code> with <code>status</code>.
	 * 
	 * @param throwable java.lang.Throwable
	 * @param status int
	 */
	public IsrServletException(Throwable throwable, int status) {
		super(throwable.getMessage());
		exception = throwable;
		statusCode = status;
	}
	/**
	 * Produces the message, include the message from the nested
	 * exception if there is one.
	 * 
	 * @return java.lang.String
	 */
	public String getMessage() {
		return super.getMessage();
	}
	/**
	 * Answer the status code.
	 *
	 * @return int
	 */
	public int getStatusCode() {
		return statusCode;
	}
	/**
	 * Prints this <code>Throwable</code> and its backtrace to the 
	 * standard error stream.
	 */
	public void printStackTrace() {
		this.printStackTrace(System.err);
	}
	/**
	 * Prints the composite message and the embedded stack trace to
	 * the specified stream <code>ps</code>.
	 *
	 * @param ps java.io.PrintStream
	 */
	public void printStackTrace(PrintStream ps) {
		if (exception == null) {
			super.printStackTrace(ps);
		} else {
			synchronized (ps) {
				ps.println(this);
				exception.printStackTrace(ps);
			}
		}
	}
	/**
	 * Prints the composite message and the embedded stack trace to
	 * the specified print writer <code>pw</code>
	 *
	 * @param pw java.io.PrintWriter
	 */
	public void printStackTrace(PrintWriter pw) {
		if (exception == null) {
			super.printStackTrace(pw);
		} else {
			synchronized (pw) {
				pw.println(this);
				exception.printStackTrace(pw);
			}
		}
	}
}
