package sra.isr.servlet.image;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import sra.isr.image.objects.*;
import sra.isr.servlet.basic.*;

/**
 * IsrGetImageServlet class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/12/29 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrGetImageServlet.java,v 4.2 2002/07/11 09:25:05 m-asada Exp $
 */
public abstract class IsrGetImageServlet extends IsrServlet {
	/**
	 * Answer the default content type.
	 *
	 * @return java.lang.String
	 */
	public String contentType() {
		return "image/jpeg";
	}
	/**
	 * Process the HTTP GET request.
	 * 
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param response javax.servlet.http.HttpServletResponse
	 * @exception sra.isr.web.servlet.IsrServletException
	 * @exception javax.servlet.ServletException
	 * @exception java.io.IOException
	 */
	protected void doGetProcess(HttpServletRequest request, HttpServletResponse response) throws IsrServletException, ServletException, IOException {
		int pkey = Integer.parseInt(this.getParameter(request, "pkey"));
		this.setBody(response, this.getImageFileInfo(pkey));
	}
	/**
	 * Get the image information.
	 * 
	 * @return sra.isr.image.objects.IsrImageFileInfo
	 * @param pkey int
	 * @exception sra.isr.web.servlet.IsrServletException
	 */
	protected abstract IsrImageFileInfo getImageFileInfo(int pkey) throws IsrServletException;
}
