package sra.isr.servlet.image;

import java.awt.*;
import java.io.*;
import java.sql.SQLException;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import sra.smalltalk.*;
import sra.isr.geometry.basic.*;
import sra.isr.image.database.*;
import sra.isr.servlet.basic.*;

/**
 * IsrImageSearcherServlet class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2001/02/15 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrImageSearcherServlet.java,v 4.2 2002/07/11 09:25:05 m-asada Exp $
 */
public class IsrImageSearcherServlet extends IsrServletWithSession {
	/**
	 * Process the HTTP GET request with a session.
	 * 
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param response javax.servlet.http.HttpServletResponse
	 * @exception sra.isr.servlet.IsrServletException
	 * @param session javax.servlet.http.HttpSession
	 * @exception javax.servlet.ServletException
	 * @exception java.io.IOException
	 */
	protected void doGetProcess(HttpServletRequest request, HttpServletResponse response, HttpSession session) throws IsrServletException, ServletException, IOException {
		Hashtable[] results = null;
		IsrParamParser parser = this.getParamParser(request);

		// parse parameters.
		Color typicalColor = this.getTypicalColor(parser.getParameter("typical"));
		double luminance = this.getAverageLuminance(parser.getParameter("luminance"));
		Color[] nineColors = this.getNineColors(parser.getParameters());
		Isr2dPoint iconArea = this.getIconArea(parser.getParameter("extent"));
		Isr2dPoint centerSpectra = this.getCenterSpectra(parser.getParameter("center"));
		double cutOff = this.getCutOff(parser.getParameter("cutoff"));

		// search image from database.
		try {
			results = this.getDatabase().search(typicalColor, luminance, nineColors, iconArea, centerSpectra, cutOff);
		} catch (SQLException e) {
			throw new IsrServletException(e);
		}

		// write image browser by jsp.
		session.setAttribute("results", results);
		response.sendRedirect(request.getContextPath() + "/imageBrowser.jsp?page=1");
	}
	/**
	 * Answer the average luminance with parameter.
	 *
	 * @return double
	 * @param parameter java.lang.String
	 */
	protected double getAverageLuminance(String parameter) {
		double luminance = -1.0f;
		if (parameter != null && parameter.length() != 0) {
			luminance = new Double(parameter).doubleValue();
		}
		return luminance;
	}
	/**
	 * Answer the center spectra with parameter.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint
	 * @param parameter java.lang.String
	 */
	protected Isr2dPoint getCenterSpectra(String parameter) {
		Isr2dPoint centerSpectra = null;
		if (parameter != null && parameter.length() != 0) {
			int position = parameter.indexOf('@');
			double x = new Double(parameter.substring(0, position)).doubleValue();
			double y = new Double(parameter.substring(position + 1)).doubleValue();
			centerSpectra = new Isr2dPoint(x, y);
		}
		return centerSpectra;
	}
	/**
	 * Answer the cut off with parameter.
	 *
	 * @return double
	 * @param parameter java.lang.String
	 */
	protected double getCutOff(String parameter) {
		double cutOff = -1.0f;
		if (parameter != null && parameter.length() != 0) {
			cutOff = new Double(parameter).doubleValue();
		}
		return cutOff;
	}
	/**
	 * Answer the icon area with parameter.
	 *
	 * @return sra.isr.geometry.basic.Isr2dPoint
	 * @param parameter java.lang.String
	 */
	protected Isr2dPoint getIconArea(String parameter) {
		Isr2dPoint iconArea = null;
		if (parameter != null && parameter.length() != 0) {
			int position = parameter.indexOf('@');
			double x = new Double(parameter.substring(0, position)).doubleValue();
			double y = new Double(parameter.substring(position + 1)).doubleValue();
			iconArea = new Isr2dPoint(x, y);
		}
		return iconArea;
	}
	/**
	 * Answer the nine colors with parameter.
	 *
	 * @return java.awt.Color[9]
	 * @param parameters java.util.Hashtable
	 */
	protected Color[] getNineColors(Hashtable parameters) {
		Color[] nineColors = null;
		for (int i = 0; i < 9; i++) {
			String colorValue = (String) parameters.get("color" + (i + 1));
			if (colorValue != null && colorValue.length() != 0) {
				Color color = new Color(Integer.parseInt(colorValue));
				if (nineColors == null) {
					nineColors = new Color[9];
				}
				nineColors[i] = color;
			}
		}
		return nineColors;
	}
	/**
	 * Answer the typical color with parameter.
	 *
	 * @return java.awt.Color
	 * @param parameter java.lang.String
	 */
	protected Color getTypicalColor(String parameter) {
		Color typicalColor = null;
		if (parameter != null && parameter.length() != 0) {
			typicalColor = new Color(Integer.parseInt(parameter));
		}
		return typicalColor;
	}
	/**
	 * Answer <code>true</code> if create new <code>HttpSession</code>
	 * with the HTTP GET request; <code>false</code> if <code>HttpSession</code>
	 * associated with the HTTP GET request.
	 *
	 * @return boolean
	 */
	protected boolean isAssociatedSessionOfGet() {
		return true;
	}
}
