package sra.isr.servlet.image;

import java.awt.*;
import java.io.*;
import java.sql.SQLException;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import sra.smalltalk.*;
import sra.isr.image.database.*;
import sra.isr.image.objects.*;
import sra.isr.image.streams.*;
import sra.isr.servlet.basic.*;

/**
 * IsrUploadImageServlet class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/12/27 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrUploadImageServlet.java,v 4.2 2002/07/11 09:25:05 m-asada Exp $
 */
public class IsrUploadImageServlet extends IsrServlet {
	/**
	 * Create the thumbnail image info from an image object.
	 *
	 * @return sra.isr.image.objects.IsrImageFileInfo
	 * @param imageObject sra.isr.image.objects.IsrImageObject
	 * @exception java.io.IOException
	 */
	public IsrImageFileInfo createThumbnailImageInfo(IsrImageObject imageObject) throws IOException {
		Rectangle iconArea = imageObject.iconArea();
		StImage iconImage = new StImage(iconArea.width, iconArea.height);
		iconImage = iconImage.copy_from_in_rule_(iconImage.bounds(), iconArea.getLocation(), imageObject.iconImage(), StImage.Over);
		IsrImageFileInfo imageInfo = new IsrImageFileInfo();
		//
		IsrImageStream imageStream = null;
		ByteArrayOutputStream byteStream = null;
		try {
			byteStream = new ByteArrayOutputStream();
			imageStream = IsrJpegImageStream.On_(byteStream);
			imageStream.nextPutImage_(iconImage);
			imageStream.flush();
			imageInfo.setBytes(byteStream.toByteArray());
			imageInfo.setContentType("image/jpeg");
		} finally {
			if (byteStream != null) {
				byteStream.close();
				byteStream = null;
			}
			if (imageStream != null) {
				imageStream.close();
				imageStream = null;
			}
		}
		return imageInfo;
	}
	/**
	 * Process the HTTP POST request.
	 * 
	 * @param request javax.servlet.http.HttpServletRequest
	 * @param response javax.servlet.http.HttpServletResponse
	 * @exception sra.isr.servlet.IsrServletException
	 * @exception javax.servlet.ServletException
	 * @exception java.io.IOException
	 */
	protected void doPostProcess(HttpServletRequest request, HttpServletResponse response) throws IsrServletException, ServletException, IOException {
		IsrImageObject imageObject = null;
		IsrImageFileInfo originalImage = null;
		IsrImageFileInfo thumbnailImage = null;

		// checked upload file.
		IsrParamParser parser = this.getParamParser(request);
		Hashtable imageInfo = parser.getFile("image");
		if (imageInfo == null) {
			throw new IsrServletException("The upload of an image file is empty.");
		}
		String filename = (String) imageInfo.get("filename");
		String contentType = (String) imageInfo.get("content-type");
		byte[] imageByte = (byte[]) imageInfo.get("content");

		// create image object and image file info objects.
		try {
			originalImage = new IsrImageFileInfo(filename, contentType, imageByte);
			imageObject = originalImage.asImageObject();
			thumbnailImage = this.createThumbnailImageInfo(imageObject);
		} catch (IOException e) {
			throw new IsrServletException(e);
		}

		// store image object on database.
		int pkey = this.storeImage(imageObject, originalImage, thumbnailImage);

		// write result of upload image by jsp.
		response.sendRedirect(request.getContextPath() + "/resultUpload.jsp?pkey=" + pkey + "&width=" + imageObject.iconArea().width + "&height=" + imageObject.iconArea().height);
	}
	/**
	 * Store an image object on database.
	 *
	 * @return int
	 * @param imageObject sra.isr.image.objects.IsrImageObject
	 * @param contentType java.lang.String
	 * @param originalImage sra.isr.image.objects.IsrImageFileInfo
	 * @param thumbnailImage sra.isr.image.objects.IsrImageFileInfo
	 * @exception sra.isr.web.servlet.IsrServletException
	 */
	protected int storeImage(IsrImageObject imageObject, IsrImageFileInfo originalImage, IsrImageFileInfo thumbnailImage) throws IsrServletException {
		try {
			return this.getDatabase().storeImage(imageObject, originalImage, thumbnailImage);
		} catch (SQLException e) {
			throw new IsrServletException(e);
		}
	}
}
