package sra.isr.system.support;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import sra.smalltalk.*;

/**
 * IsrJLauncher class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   2000/03/10 (by Mitsuhiro Asada)
 * 	@updated   N/A
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrJLauncher.java,v 4.2 2002/07/11 09:25:05 m-asada Exp $
 */
public class IsrJLauncher extends IsrLauncher {
	/**
	 * Create a default menu bar of the IsrJLauncher.
	 *
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu defaultHelpMenu() {
		StMenu helpMenu = new StMenu("Help");
		helpMenu.add(createMenuItem("About...", "openAbout"));
	    helpMenu.add(createMenuItem("Copyright", "openCopyright"));
	    helpMenu.add(createMenuItem("License", "openLicense"));
		helpMenu.addSeparator();
		StMenu viewMenu = new StMenu("Default View Mode");
		StButtonGroup viewGroup = new StButtonGroup();
		viewMenu.add(createRadioButtonMenuItem("Awt", "viewAwt", viewGroup));
		viewMenu.add(createRadioButtonMenuItem("Swing Only", "viewSwingOnly", viewGroup));
		((StRadioButtonMenuItem) viewMenu.getItem("Swing Only")).setSelected(true);
		helpMenu.add(viewMenu);
		//
		return helpMenu;
	}
	/**
	 * Open IsrJLauncher.
	 * 
	 * @param args java.lang.String[]
	 */
	public static void main(String[] args) {
		StApplicationModel.SetDefaultViewMode(StApplicationModel.VIEW_SWING_ONLY);
		//
		_closeToExit = true;
		//
		new IsrJLauncher();
	}
	/**
	 * Open a copyright dialog.
	 */
	public void openCopyright() {
		JTextArea aTextArea = new JTextArea(IsrSystem.Copyright(), 0, 0);
		aTextArea.setFont(new Font("Courier", Font.PLAIN, 12));
		aTextArea.setLineWrap(true);
		aTextArea.setWrapStyleWord(true);
		aTextArea.setEditable(false);

		JScrollPane aScrollPane = new JScrollPane(aTextArea);
		aScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		aScrollPane.setPreferredSize(new Dimension(500, 160));

		JFrame aFrame = new JFrame("Copyright");
		aFrame.getContentPane().add(aScrollPane);
		aFrame.pack();
		aFrame.setResizable(false);
		aFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});

		this._showOnCenter(aFrame);
	}
	/**
	 * Open a license dialog.
	 */
	public void openLicense() {
		JTextArea aTextArea = new JTextArea(IsrSystem.License(), 0, 0);
		aTextArea.setFont(new Font("Courier", Font.PLAIN, 12));
		aTextArea.setLineWrap(true);
		aTextArea.setWrapStyleWord(true);
		aTextArea.setEditable(false);

		JScrollPane aScrollPane = new JScrollPane(aTextArea);
		aScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		aScrollPane.setPreferredSize(new Dimension(600, 360));

		JFrame aFrame = new JFrame("The modified BSD license");
		aFrame.getContentPane().add(aScrollPane);
		aFrame.pack();
		aFrame.setResizable(false);
		aFrame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				e.getWindow().dispose();
			}
		});

		this._showOnCenter(aFrame);
	}
}
