package sra.isr.system.support;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.Calendar;
import sra.smalltalk.*;

/**
 * IsrSystem class
 * 
 * 	@author    Mitsuhiro Asada
 * 	@created   1999/12/21 (by Mitsuhiro Asada)
 * 	@updated   2000/11/13 (by Mitsuhiro Asada)
 * 	@version   Isr023b
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 2000-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: IsrSystem.java,v 4.3 2002/07/11 09:25:05 m-asada Exp $
 */
public class IsrSystem extends StModel {
	/**
	 * Answer the string of the copyright.
	 *
	 * @return java.lang.String
	 */
	public final static String Copyright() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = null;
		try {
			pw = new PrintWriter(sw);
			pw.println("\"ImageSearcher for Java\" package is being published as open source software under the terms of \"The modified BSD license\".");
			pw.println();
			pw.println(CopyrightSRA());
			pw.println(CopyrightKTL());
			pw.println("All rights reserved.");
			pw.println();
			pw.println(JavaTM());
		} finally {
			if (pw != null) {
				pw.flush();
				pw.close();
			}
		}
		return sw.toString();
	}
	/**
	 * Answer the string of the KTL copyright.
	 *
	 * @return java.lang.String
	 */
	public final static String CopyrightKTL() {
		int startYear = 2002;
		int thisYear = Calendar.getInstance().get(Calendar.YEAR);
		String yearString = null;
		if (startYear < thisYear) {
			yearString = String.valueOf(startYear) + "-" + String.valueOf(thisYear);
		} else {
			yearString = String.valueOf(startYear);
		}
		return "Copyright (C) " + yearString + " SRA/KTL (SRA Key Technology Laboratory, Inc.)";
	}
	/**
	 * Answer the string of the SRA copyright.
	 *
	 * @return java.lang.String
	 */
	public final static String CopyrightSRA() {
		int thisYear = Calendar.getInstance().get(Calendar.YEAR);
		return "Copyright (C) 2000-" + thisYear + " SRA (Software Research Associates, Inc.)";
	}
	/**
	 * Answer the string of the Java trademark legend.
	 *
	 * @return java.lang.String
	 */
	public final static String JavaTM() {
		return "Java and all Java-based trademarks and logos are trademarks or registered trademarks of Sun Microsystems, Inc. in the U.S. or other countries, and are used under license. ";
	}
	/**
	 * Answer the string of the license term.
	 *
	 * @return java.lang.String
	 */
	public final static String License() {
		StringWriter sw = new StringWriter();
		PrintWriter pw = null;
		try {
			pw = new PrintWriter(sw);
			pw.println(CopyrightSRA());
			pw.println(CopyrightKTL());
			pw.println("All rights reserved.");
			pw.println();
			pw.println("Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:");
			pw.println();
			pw.println("1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. ");
			pw.println();
			pw.println("2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. ");
			pw.println();
			pw.println("3. The name of the author may not be used to endorse or promote products derived from this software without specific prior written permission. ");
			pw.println();
			pw.println("THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.");
			pw.println();
			pw.println(JavaTM());
		} finally {
			if (pw != null) {
				pw.flush();
				pw.close();
			}
		}
		return sw.toString();
	}
	/**
	 * Create a eMMa logo image.
	 *
	 * @return java.awt.Image
	 */
	public final static Image LogoImage() {
		return SystemResourceSupport.createImage("/sra/isr/system/support/LogoImage.jpg");
	}
	/**
	 * Return Isr system string
	 *
	 * @return string
	 */
	public final static String System() {
		return "Isr";
	}
	/**
	 * Return Isr version string
	 *
	 * @return string
	 */
	public final static String Version() {
		return "023b";
	}
}
