package sra.smalltalk;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import java.lang.reflect.*;
import java.util.Vector;
import javax.swing.*;

/**
 * StApplicationModel class
 * 
 * 	@author    he weijie
 * 	@created   1998/08/18 (by he weijie)
 * 	@updated   1999/10/01 (by MATSUDA Ryouichi)
 * 	@updated   2002/04/09 (by nisinaka)
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StApplicationModel.java,v 4.5 2002/07/11 09:18:29 m-asada Exp $
 */
public abstract class StApplicationModel extends StModel implements ActionListener, ItemListener {
	/**
	 * view mode
	 */
	public final static int VIEW_AWT = 0;
	public final static int VIEW_SWING = 1;
	public final static int VIEW_SWING_ONLY = 2;
	protected static int DefaultViewMode = VIEW_AWT;

	/**
	 *
	 */
	protected StUIBuilder builder = new StUIBuilder(this);
	/**
	 * Create a JPanel with border.
	 *
	 * @return javax.swing.JPanel
	 */
	public static JPanel _JPanelWithBorder() {
	    return new JPanel() {
	        public Insets getInsets() {
	            return new Insets(4, 4, 4, 4);
	        }
	        public void paint(Graphics graphics) {
	            super.paint(graphics);
	            Dimension size = this.getSize();
	            // top
	            graphics.setColor(Color.black);
	            graphics.fillRect(1, 1, size.width - 4, 1);
	            graphics.setColor(Color.gray);
	            graphics.fillRect(2, 2, size.width - 4, 1);
	            // bottom
	            graphics.setColor(Color.lightGray);
	            graphics.fillRect(1, size.height - 3, size.width - 4, 1);
	            // left
	            graphics.setColor(Color.black);
	            graphics.fillRect(1, 1, 1, size.height - 4);
	            graphics.setColor(Color.gray);
	            graphics.fillRect(2, 2, 1, size.height - 4);
	            // right
	            graphics.setColor(Color.lightGray);
	            graphics.fillRect(size.width - 3, 1, 1, size.height - 4);
	        }
	    };
	}
	/**
	 * create a panel with the border.
	 * @return java.awt.Panel 
	 */
	public static Panel _PanelWithBorder() {
		Panel aPanel = new Panel() {
			public Insets getInsets() {
				return new Insets(4, 4, 4, 4);
			}
			public void paint(Graphics graphics) {
				super.paint(graphics);
				Dimension size = this.getSize();
				// top
				graphics.setColor(Color.black);
				graphics.fillRect(1, 1, size.width - 4, 1);
				graphics.setColor(Color.gray);
				graphics.fillRect(2, 2, size.width - 4, 1);
				// bottom
				graphics.setColor(Color.lightGray);
				graphics.fillRect(1, size.height - 3, size.width - 4, 1);
				// left
				graphics.setColor(Color.black);
				graphics.fillRect(1, 1, 1, size.height - 4);
				graphics.setColor(Color.gray);
				graphics.fillRect(2, 2, 1, size.height - 4);
				// right
				graphics.setColor(Color.lightGray);
				graphics.fillRect(size.width - 3, 1, 1, size.height - 4);
			}
		};
		return aPanel;
	}
	/**
	 * Show the frame at the current mouse point.
	 *
	 * @param aWindow java.awt.Window
	 */
	public static void _ShowAtMousePoint(Window aWindow) {
	    try {
	        Point aPoint = SystemInterface._MousePoint();
	        aWindow.setLocation(
	            Math.max(0, aPoint.x - aWindow.getWidth() / 2),
	            Math.max(0, aPoint.y - aWindow.getHeight() / 2));
	    } catch (UnsatisfiedLinkError e) {
	    } catch (NoClassDefFoundError e) {
	    }
	    aWindow.show();
	}
	/**
	 * Invoked when a menu item has been selected. 
	 *
	 * @param e java.awt.event.ActionEvent 
	 */
	public void actionPerformed(ActionEvent ev) {
		try {
			this.perform_(ev.getActionCommand());
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}
	/**
	 * 
	 * @return sra.smalltalk.StUIBuilder
	 */
	public final StUIBuilder builder() {
		return builder;
	}
	/**
	 * Close all of the application's window.
	 */
	public void closeRequest() {
	    Frame[] frames = this.builder().frames();
	    for (int i = 0; i < frames.length; i++) {
	        frames[i].dispose();
	    }
	}
	/**
	 * Create a StCheckBoxMenuItem and set a command.
	 *
	 * @param displayString java.lang.String
	 * @param commandString java.lang.String
	 * @return sra.smalltalk.StCheckBoxMenuItem
	 */
	protected StCheckBoxMenuItem createCheckBoxMenuItem(String displayString, String commandString) {
		StCheckBoxMenuItem item = new StCheckBoxMenuItem(displayString);
		item.setActionCommand(commandString);
		//item.addItemListener(this);
		item.addActionListener(this);
		return item;
	}
	/**
	 * Create a StMenuItem and set a command.
	 *
	 * @param displayString java.lang.String
	 * @param commandString java.lang.String
	 * @return sra.smalltalk.StMenuItem
	 */
	protected StMenuItem createMenuItem(String displayString, String commandString) {
		StMenuItem item = new StMenuItem(displayString);
		item.setActionCommand(commandString);
		item.addActionListener(this);
		return item;
	}
	/**
	 * Create a StRadioButtonMenuItem and set a command.
	 *
	 * @param displayString java.lang.String
	 * @param commandString java.lang.String
	 * @param radioGroup sra.smalltalk.StMenuRadioButtonGroup
	 * @return sra.smalltalk.StRadioButtonMenuItem
	 */
	protected StRadioButtonMenuItem createRadioButtonMenuItem(String displayString, String commandString, StButtonGroup radioGroup) {
		StRadioButtonMenuItem item = new StRadioButtonMenuItem(displayString);
		item.setActionCommand(commandString);
		//item.addItemListener(this);
		item.addActionListener(this);
		radioGroup.add(item);
		return item;
	}
	/**
	 * Create a view from class name.
	 *
	 * @deprecated since Jun340
	 * @return sra.smalltalk.StView
	 */
	protected static final StView CreateView(String className) {
		try {
			return (StView) Class.forName(className).newInstance();
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}
	/**
	 * Answer a default view.
	 * @return sra.smalltalk.StView
	 */
	public abstract StView defaultView();
	/**
	 * Answer the default view mode.
	 *
	 * @return int
	 */
	public final static int GetDefaultViewMode() {
		return DefaultViewMode;
	}
	/**
	 * Is swing enabled.
	 *
	 * @deprecated since Jun340
	 * @return boolean
	 */
	public final static boolean IsSwingInstalled() {
		return true;
	}
	/**
	 * Invoked when a checkbox item has been selected. 
	 *
	 * @param e java.awt.event.ActionEvent 
	 */
	public void itemStateChanged(ItemEvent ev) {
		String command;
		if (ev.getSource() instanceof StCheckBoxMenuItem) {
			command = ((StCheckBoxMenuItem) ev.getSource()).getActionCommand();
		} else {
			command = ((CheckboxMenuItem) ev.getSource()).getActionCommand();
		}
		try {
			this.perform_(command);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}
	/**
	 * Answer a menu model.
	 * @return sra.smalltalk.StMenuModel
	 */
	public StMenuModel menuModel() {
		return null;
	}
	/**
	 * Create a frame of the default view and open it.
	 */
	public Frame open() {
	    return this.openView_((StView) this.defaultView());
	}
	/**
	 * Create a frame of the view and open it.
	 * 
	 * @param param sra.smalltalk.StView
	 */
	public Frame openView_(StView aView) {
	    aView.model_(this);

	    Frame aFrame =
	        (aView instanceof JComponent)
	            ? StMenuViewForSwing.CreateJFrame(this, aView)
	            : StMenuViewForAwt.CreateFrame(this, aView);

	    _ShowAtMousePoint(aFrame);
	    return aFrame;
	}
	/**
	 * 
	 * @return boolean
	 */
	protected boolean requestForWindowClose() {
		return true;
	}
	/**
	 * Set the default view mode.
	 *
	 * @param mode int
	 */
	public final static boolean SetDefaultViewMode(int mode) {
		DefaultViewMode = mode;
		return true;
	}
	/**
	 * Invoked when a window is in the process of being closed.
	 * The close operation can be overridden at this point.
	 */
	public void windowClosing(WindowEvent e) {
		this.builder().removeFrame_((Frame) e.getWindow());
		e.getWindow().setVisible(false);
		Thread.yield();
		e.getWindow().dispose();
	}
	/**
	 * Answer a window resizable.
	 * @return boolean
	 */
	protected boolean windowResizable() {
		return true;
	}
	/**
	 * Answer a window title.
	 * @return java.lang.String
	 */
	protected String windowTitle() {
		return this.toString();
	}
}
