package sra.smalltalk;

/**
 * StBlockValue class
 * 
 * 	@author    nisinaka
 * 	@created   1998/10/15 (by nisinaka)
 * 	@updated   1998/11/09 (by nisinaka)
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StBlockValue.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StBlockValue extends StComputedValue {
	/**
	 * The block closure to evaluate.
	 */
	protected StBlockClosure block = null;

	/**
	 * The array of StObject, each of the object has a dependency.
	 */
	protected StObject[] arguments = null;

	/**
	 * The number of arguments block takes.
	 */
	protected int numArgs;
	/**
	 * Create a new StBlockValue with specified block and arguments.
	 *
	 * @return sra.smalltalk.StBlockValue
	 * @param aClass java.lang.Class
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param anArray sra.smalltalk.StObject[]
	 */
	public static StBlockValue Block_arguments_(Class aClass, StBlockClosure aBlock, StObject[] anArray) {
		StBlockValue aBlockValue = (StBlockValue) _New(aClass);
		aBlockValue.setBlock_arguments_(aBlock, anArray);
		return aBlockValue;
	}
	/**
	 * Create a new StBlockValue with specified block and arguments.
	 *
	 * @return sra.smalltalk.StBlockValue
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param anArray sra.smalltalk.StObject[]
	 */
	public static StBlockValue Block_arguments_(StBlockClosure aBlock, StObject[] anArray) {
		return Block_arguments_(StBlockValue.class, aBlock, anArray);
	}
	/**
	 * Compute a value for the receiver.
	 *
	 * @return java.lang.Object
	 */
	public Object computeValue() {
		try {
			switch (numArgs) {
				case 0 :
					return block.value();
				case 1 :
					return block.value_(arguments[0].perform_("value"));
				case 2 :
					return block.value_value_(arguments[0].perform_("value"), arguments[1].perform_("value"));
				case 3 :
					return block.value_value_value_(arguments[0].perform_("value"), arguments[1].perform_("value"), arguments[2].perform_("value"));
			}
			Object[] anArray = new StObject[Math.min(numArgs, arguments.length)];
			for (int i = 0; i < anArray.length; i++) {
				anArray[i] = arguments[i].perform_("value");
			}
			return block.valueWithArguments_(anArray);
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}
	/**
	 * Set the receiver's BlockClosure.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	protected void setBlock_(StBlockClosure aBlock) {
		block = aBlock;
		numArgs = aBlock.numArgs();
		cachedValue = UnassignedValue;
		arguments = new StObject[0];
	}
	/**
	 * Set the receiver's block and the arguments.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param anArray sra.smalltalk.StObject[]
	 */
	protected void setBlock_arguments_(StBlockClosure aBlock, StObject[] anArray) {
		this.setBlock_(aBlock);
		arguments = anArray;
		for (int i = 0; i < arguments.length; i++) {
			arguments[i].addDependent_(this);
		}
	}
}
