package sra.smalltalk;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

/**
 * StCheckBoxMenuItem class
 * 
 * 	@author    MATSUDA Ryouichi
 * 	@created   1999/09/16 (by MATSUDA Ryouichi)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StCheckBoxMenuItem.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StCheckBoxMenuItem extends StMenuItem implements ItemSelectable {
	private Vector itemListeners;
	protected boolean selected = false;
	/**
	 * Creates a menuItem with text.
	 *
	 * @param text java.lang.String
	 */
	public StCheckBoxMenuItem(String text) {
		this(text, false);
	}
	/**
	 * Creates a menuItem with text.
	 *
	 * @param text java.lang.String
	 * @param selected boolean
	 */
	public StCheckBoxMenuItem(String text, boolean selected) {
		super(text);
		this.selected = selected;
	}
	/**
	 * adds an ItemListener to the checkbox
	 */
	public synchronized void addItemListener(ItemListener l) {
		if (itemListeners == null) {
			itemListeners = new Vector();
		}
		itemListeners.addElement(l);
	}
	/**
	 * Invoked when an item has been selected or deselected.
	 */    
	protected void fireItemStateChanged(ItemEvent event) {
		if (itemListeners == null) {
			return;
		}
		Vector copy;
		synchronized (itemListeners) {
			copy = (Vector) itemListeners.clone();
		}
		for (int i = 0; i < copy.size(); i++) {
			ItemListener l = (ItemListener) copy.elementAt(i);
			l.itemStateChanged(event);
		}
	}
	/**
	 * Returns the an array (length 1) containing the checkbox menu item
	 * label or null if the checkbox is not selected.
	 * @see ItemSelectable
	 */
	public synchronized Object[] getSelectedObjects() {
		if (isSelected()) {
			Object[] items = new Object[1];
			items[0] = label;
			return items;
		}
		return null;
	}
	/**
	 * @return boolean
	 */
	public boolean isSelected() {
		return selected;
	}
	/**
	 * removes an ItemListener from the button
	 */
	public void removeItemListener(ItemListener l) {
		itemListeners.removeElement(l);
	}
	/**
	 * @param aBoolean boolean
	 */
	protected void selected_(boolean aBoolean) {
		if (selected != aBoolean) {
			selected = aBoolean;
			StSymbol aspect = selected ? $("selected") : $("deselected");
			StMenuItem root = getRoot();
			if (root instanceof StMenuModel) {
				((StMenuModel) root).changed_with_(aspect, this);
			}
			this.fireItemStateChanged(new ItemEvent(
				this, ItemEvent.ITEM_STATE_CHANGED, this,
				selected ? ItemEvent.SELECTED : ItemEvent.DESELECTED));
		}
	}
	/**
	 * @param aBoolean boolean
	 */
	public void setSelected(boolean aBoolean) {
		this.selected_(aBoolean);
	}
}
