package sra.smalltalk;

import java.io.*;
import java.util.*;
import java.awt.Color;
import java.awt.image.ColorModel;

/**
 * StColorValue class
 * 
 * 	@author    He Weijie
 * 	@created   1998/10/10 (by xiaolai)
 * 	@updated   2000/01/06 (by Mitsuhiro Asada)
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StColorValue.java,v 4.3 2002/07/11 09:18:29 m-asada Exp $
 */
public class StColorValue extends StObject {
    /**
     * The RGB numbers.
     */
    protected int red, green, blue;

    //
    // Class variavles
    //
    public static int _MaxDistanceSquared = -1;
    public static int _MaxValue = 0xFF;
    public static int _MaxLuminanceValue = _MaxValue * 1000;
    public static double _LuminanceScaleValue = 1.0f / _MaxLuminanceValue;

    //
    // Constant color values.
    //
    public final static Color Black = Color.black;
    public final static Color Blue = Color.blue;
    public final static Color Brown = new Color(127, 63, 63);
    public final static Color Chartreuse = new Color(127, 255, 0);
    public final static Color Cyan = new Color(0, 255, 255);
    public final static Color DarkCyan = new Color(0, 127, 127);
    public final static Color DarkGray = new Color(95, 95, 95);
    public final static Color DarkGreen = new Color(0, 127, 0);
    public final static Color DarkMagenta = new Color(127, 0, 127);
    public final static Color DarkRed = new Color(127, 0, 0);
    public final static Color Gray = new Color(127, 127, 127);
    public final static Color Green = new Color(0, 255, 0);
    public final static Color LightGray = new Color(170, 170, 170);
    public final static Color LightCyan = new Color(127, 255, 255);
    public final static Color LightYellow = new Color(255, 255, 127);
    public final static Color Magenta = new Color(255, 0, 255);
    public final static Color Navy = new Color(0, 0, 127);
    public final static Color Olive = new Color(127, 127, 0);
    public final static Color Orange = new Color(255, 127, 0);
    public final static Color Orchid = new Color(255, 127, 255);
    public final static Color PaleGreen = new Color(127, 255, 127);
    public final static Color Pink = new Color(255, 191, 191);
    public final static Color Purple = new Color(159, 31, 223);
    public final static Color Red = Color.red;
    public final static Color RoyalBlue = new Color(127, 127, 255);
    public final static Color Salmon = new Color(255, 127, 127);
    public final static Color SpringGreen = new Color(0, 255, 127);
    public final static Color VeryDarkGray = new Color(63, 63, 63);
    public final static Color VeryLightGray = new Color(212, 212, 212);
    public final static Color White = Color.white;
    public final static Color Yellow = new Color(255, 255, 0);

    /**
     * Colors with its name as a key.
     */
    protected static Hashtable NamedColors = new Hashtable(31);;

    static {
        NamedColors.put("Black", Black);
        NamedColors.put("Blue", Blue);
        NamedColors.put("Brown", Brown);
        NamedColors.put("Chartreuse", Chartreuse);
        NamedColors.put("Cyan", Cyan);
        NamedColors.put("DarkCyan", DarkCyan);
        NamedColors.put("DarkGray", DarkGray);
        NamedColors.put("DarkGreen", DarkGreen);
        NamedColors.put("DarkMagenta", DarkMagenta);
        NamedColors.put("DarkRed", DarkRed);
        NamedColors.put("Gray", Gray);
        NamedColors.put("Green", Green);
        NamedColors.put("LightGray", LightGray);
        NamedColors.put("LightCyan", LightCyan);
        NamedColors.put("LightYellow", LightYellow);
        NamedColors.put("Magenta", Magenta);
        NamedColors.put("Navy", Navy);
        NamedColors.put("Olive", Olive);
        NamedColors.put("Orange", Orange);
        NamedColors.put("Orchid", Orchid);
        NamedColors.put("PaleGreen", PaleGreen);
        NamedColors.put("Pink", Pink);
        NamedColors.put("Purple", Purple);
        NamedColors.put("Red", Red);
        NamedColors.put("RoyalBlue", RoyalBlue);
        NamedColors.put("Salmon", Salmon);
        NamedColors.put("SpringGreen", SpringGreen);
        NamedColors.put("VeryDarkGray", VeryDarkGray);
        NamedColors.put("VeryLightGray", VeryLightGray);
        NamedColors.put("White", White);
        NamedColors.put("Yellow", Yellow);
    }
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return boolean
	 * @param aColor java.awt.Color
	 */
	public static double _DistanceFrom(Color color1, Color color2) {
		return Math.sqrt(_DistanceSquaredFrom(color1, color2));
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return boolean
	 * @param aColor java.awt.Color
	 */
	public static int _DistanceSquaredFrom(Color color1, Color color2) {
		int rdiff = color1.getRed() - color2.getRed();
		int gdiff = color1.getGreen() - color2.getGreen();
		int bdiff = color1.getBlue() - color2.getBlue();
		return rdiff * rdiff + gdiff * gdiff + bdiff * bdiff;
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return java.lang.Integer or null
	 * @param aColor java.awt.Color
	 */
	public static Integer _DistanceSquaredFrom_ifLessThan_(Color color1, Color color2, int dist) {
		int diff, rdiff, gdiff, bdiff;
		rdiff = color1.getRed() - color2.getRed();
		diff = rdiff * rdiff;
		if (diff >= dist) {
			return null;
		}
		gdiff = color1.getGreen() - color2.getGreen();
		diff = diff + gdiff * gdiff;
		if (diff >= dist) {
			return null;
		}
		bdiff = color1.getBlue() - color2.getBlue();
		diff = diff + bdiff * bdiff;
		if (diff >= dist) {
			return null;
		}
		return new Integer(diff);
	}
	/**
	 * @param color java.awt.Color
	 * @return int
	 */
	public static int _GetAlpha(Color color) {
		ColorModel rgbColorModel = ColorModel.getRGBdefault();
		return rgbColorModel.getAlpha(color.getRGB());
	}
	/**
	 * @param color java.awt.Color
	 * @return float
	 */
	public static float _GetBrightness(Color color) {
		float[] values = new float[3];
		Color.RGBtoHSB(color.getRed(), color.getGreen(), color.getBlue(), values);
		return values[2];
	}
	/**
	 * Get the Coler by name.
	 * @param aColorName java.lang.String
	 * @return java.awt.Color
	 */
	public static Color _GetColorByName_(String colorName) {
	    Color color = (Color) NamedColors.get(colorName);

	    if (color == null) {
	        char[] chars = colorName.toCharArray();
	        if (Character.isLowerCase(chars[0])) {
	            chars[0] = Character.toUpperCase(chars[0]);
	            color = (Color) NamedColors.get(new String(chars));
	        }
	    }

	    return color;
	}
	/**
	 * @param color java.awt.Color
	 * @return float
	 */
	public static float _GetHue(Color color) {
		float[] values = new float[3];
		Color.RGBtoHSB(color.getRed(), color.getGreen(), color.getBlue(), values);
		return values[0];
	}
	/**
	 * @param color java.awt.Color
	 * @return double
	 */
	public static double _GetLuminance(Color color) {
		return _LuminanceScaleValue * _ScaledLuminance_(color);
	}
	/**
	 * @param color java.awt.Color
	 * @return float
	 */
	public static float _GetSaturation(Color color) {
		float[] values = new float[3];
		Color.RGBtoHSB(color.getRed(), color.getGreen(), color.getBlue(), values);
		return values[1];
	}
	/**
	 * @param color java.awt.Color
	 * @return booleanfloat
	 */
	public static boolean _IsGray(Color color) {
		return ((color.getRed() == color.getGreen()) && (color.getGreen() == color.getBlue()) && ((color.getRGB() >> 24 & 0xFF) != 0x00));
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 * 
	 * @return double
	 */
	public static int _MaxDistanceSquared() {
		if (_MaxDistanceSquared == -1) {
			_MaxDistanceSquared = _DistanceSquaredFrom(Color.black, Color.white);
		}
		return _MaxDistanceSquared;
	}
	/**
	 * @param float,float,float,float
	 * @return Color
	 */
	public static Color _NewColor(float red, float green, float blue, float alpha) {
		return _NewColor(red * 255, green * 255, blue * 255, alpha * 255);
	}
	/**
	 * @param int,int,int,int
	 * @return Color
	 */
	public static Color _NewColor(int red, int green, int blue, int alpha) {
		int pixel = alpha << 24 | red << 16 | green << 8 | blue << 0;
		return new Color(pixel);
	}
	/**
	 * @param color java.awt.Color
	 * @return float
	 */
	public static int _ScaledLuminance_(Color color) {
		return (299 * color.getRed()) + (587 * color.getGreen()) + (114 * color.getBlue());
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @param b double
	 * @return java.awt.Color
	 */
	public static Color Brightness_(double b) {
		int sb = Math.round((float) (b * 255));
		if (sb >= 0 && sb <= 255) {
			return new Color(sb, sb, sb);
		} else {
			return Color.black;
		}
	}
	/**
	 * Return the names of all the constant color
	 *
	 * @return java.util.Vector
	 */
	public static Vector ConstantNames() {
	    Vector colorNames = new Vector();
	    Enumeration keys = NamedColors.keys();
	    while (keys.hasMoreElements()) {
	        colorNames.addElement($((String) keys.nextElement()));
	    }
	    return colorNames;
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @param cyan double
	 * @param magenta double
	 * @param yellow double
	 * @return java.awt.Color
	 */
	public static Color Cyan_Magenta_Yellow_(double cyan, double magenta, double yellow) {
		return new Color((float) (1 - cyan), (float) (1 - magenta), (float) (1 - yellow));
	}
	/**
	 * Please refer to the corresponding method in Smalltalk.
	 *
	 * @param r int
	 * @param g int
	 * @param b int
	 * @return sra.smalltalk.StColorValue
	 */
	public static Color ScaledRed_ScaledGreen_ScaledBlue_(int r, int g, int b) {
		int scaledRed = Math.min(Math.max((r / 32), 0), 255);
		int scaledGreen = Math.min(Math.max((g / 32), 0), 255);
		int scaledBlue = Math.min(Math.max((b / 32), 0), 255);
		return new Color(scaledRed, scaledGreen, scaledBlue);
	}

	/**
	 * Answer the store string of the specified color.
	 *
	 * @return java.lang.String
	 * @param aColor java.awt.Color
	 */
	public static String _StoreStringOf(Color aColor) {
	    int MaxValue = 0x1fff;
	    float ScaleValue = (float) MaxValue /  0xff;
	    StringBuffer buffer = new StringBuffer();
	    buffer.append("(ColorValue scaledRed: ");
	    buffer.append(Math.round(aColor.getRed() * ScaleValue));
	    buffer.append(" scaledGreen: ");
	    buffer.append(Math.round(aColor.getGreen() * ScaleValue));
	    buffer.append(" scaledBlue: ");
	    buffer.append(Math.round(aColor.getBlue() * ScaleValue));
	    buffer.append(")");
	    return buffer.toString();
	}
}
