package sra.smalltalk;

/**
 * StComputedValue class
 * 
 * 	@author    nisinaka
 * 	@created   1998/10/15 (by nisinaka)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StComputedValue.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StComputedValue extends StValueModel {
	/**
	 * A cached value of the computation result.
	 */
	protected Object cachedValue = UnassignedValue;

	/**
	 * A flag whether to wait until the receiver is asked to perform the computation.
	 */
	protected boolean eagerEvaluation = true;

	/**
	 * A unique object which represents an unassigned value.
	 */
	protected static final Object UnassignedValue = new Object();
	/**
	 * Compute a value for the receiver.
	 *
	 * @return java.lang.Object
	 */
	public Object computeValue() {
		throw SmalltalkException.SubclassResponsibility();
	}
	/**
	 * Set the receiver's value to unknown.
	 * Propagate the change to my dependents.
	 */
	protected void resetValue() {
		if (eagerEvaluation) {
			cachedValue = this.computeValue();
		} else {
			cachedValue = UnassignedValue;
		}
		this.changed_($("value"));
	}
	/**
	 * If a model is propagating a change, then reset the receiver and
	 * propagate the change to my dependents.
	 *
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		this.resetValue();
	}
	/**
	 * Answer the cached value for the receiver.
	 *
	 * @return java.lang.Object
	 */
	public Object value() {
		if (cachedValue == UnassignedValue) {
			cachedValue = this.computeValue();
		}
		return cachedValue;
	}
	/**
	 * Set the currently stored value, and notify dependents.
	 *
	 * @param newValue java.lang.Object
	 */
	public void value_(Object newValue) {
		throw SmalltalkException.ShouldNotImplement();
	}
}
