package sra.smalltalk;

/**
 * StDependencyTransformer class
 * 
 * 	@author    nisinaka
 * 	@created   1998/10/14 (by nisinaka)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StDependencyTransformer.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StDependencyTransformer extends StObject {
	protected StObject receiver = null;
	protected StSymbol aspect = null;
	protected String selector = null;
	protected int numArguments;
	/**
	 * Answer the number of arguments of a selector aSymbol.
	 *
	 * @return int
	 * @param aSymbol java.lang.String
	 */
	public static int _NumArgs(String aSymbol) {
		int number = 0;
		for (int i = 0; i < aSymbol.length(); i++) {
			if (aSymbol.charAt(i) == '_') {
				number++;
			}
		}
		return number;
	}
	/**
	 * Answer true if the receiver matches the condition specified with the arguments.
	 *
	 * @return boolean
	 * @param anObject sra.smalltalk.StObject
	 * @param anAspect sra.smalltalk.StSymbol
	 */
	public boolean matches_forAspect_(StObject anObject, StSymbol anAspect) {
		return (receiver == anObject && aspect == anAspect);
	}
	/**
	 * Initialize the StDependencyTransformer.
	 *
	 * @param aReceiver sra.smalltalk.StObject
	 * @param anAspect sra.smalltalk.StSymbol
	 * @param aSymbol java.lang.String
	 */
	public void setReceiver_aspect_selector_(StObject aReceiver, StSymbol anAspect, String aSymbol) {
		receiver = aReceiver;
		aspect = anAspect;
		selector = aSymbol;
		numArguments = _NumArgs(selector);
		if (numArguments > 2) {
			throw SmalltalkException.Error("selector expects too many arguments");
		}
	}
	/**
	 * Receive a change notice from an object.
	 *
	 * @param evt sra.smalltalk.DependentEvent
	 */
	public void update_(DependentEvent evt) {
		if (aspect != evt.getAspect()) {
			return;
		}
		try {
			switch (numArguments) {
				case 0 :
					receiver.perform_(selector);
					return;
				case 1 :
					receiver.perform_with_(selector, evt.getParameter());
					return;
				case 2 :
					receiver.perform_with_with_(selector, evt.getParameter(), evt.getSource());
					return;
			}
		} catch (Exception e) {
			throw new SmalltalkException(e);
		}
	}
}
