package sra.smalltalk;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

/**
 * StImageTestExamples class
 * 
 * 	@author    Hoshi Takanori
 * 	@created   2001/11/30 (by Hoshi Takanori)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StImageTestExamples.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StImageTestExamples extends SmalltalkTestExamples {
	/**
	 * 
	 * @return boolean
	 */
	public static boolean Example1() {
		StImage image = new StImage(128, 96);
		if (image != null) {
			image._display();
			return true;
		}
		return false;
	}
	/**
	 * 
	 * @return boolean
	 */
	public static boolean Example2() {
		StImage image = StImage._FromUser();
		if (image != null) {
			image._display();
			return true;
		}
		return false;
	}
	/**
	 * 
	 * @return boolean
	 */
	public static boolean Example3() {
		Rectangle rect = SystemInterface._RectangleFromUser();
		if (rect != null) {
			System.out.println("rect = " + rect);
			StImage image = StImage._OfArea(rect);
			if (image != null) {
				image._display();
				return true;
			}
		}
		return false;
	}
	/**
	 * 
	 * @return boolean
	 */
	public static boolean Example4() {
		StImage image = StImage._OfWholeScreen();
		if (image != null) {
			image._display();
			return true;
		}
		return false;
	}
	/**
	 * 
	 * @return boolean
	 */
	public static boolean Example5() {
		StImage image1 = StImage._FromUser();
		StImage image2 = null;
		try {
			ByteArrayOutputStream bout = new ByteArrayOutputStream();
			ObjectOutputStream oout = new ObjectOutputStream(bout);
			oout.writeObject(image1);
			oout.flush();
			//
			ObjectInputStream oin = new ObjectInputStream(new ByteArrayInputStream(bout.toByteArray()));
			image2 = (StImage) oin.readObject();
			//
			oout.close();
			oin.close();
		} catch (java.io.IOException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
			return false;
		} catch (ClassNotFoundException e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
			return false;
		}
		image1._display();
		image2._display();

		int[] pixels1 = image1.getPixels();
		int[] pixels2 = image2.getPixels();
		for (int i = 0; i < pixels1.length; i++) {
			if (pixels1[i] != pixels2[i]) {
				System.out.println("pixels[" + i + "] image1 = " + pixels1[i] + " image2 = " + pixels2[i]);
				return false;
			}
		}
		byte[] bytes1 = image1.bits();
		byte[] bytes2 = image2.bits();
		for (int i = 0; i < bytes1.length; i++) {
			if (bytes1[i] != bytes2[i]) {
				System.out.println("bits[" + i + "] image1 = " + bytes1[i] + " image2 = " + bytes2[i]);
				return false;
			}
		}

		return true;
	}
	/**
	 * 
	 * @return sra.smalltalk.StImage
	 */
	protected static StImage Image1() {
	    return new StImage(
	        17,
	        19,
	        StByteArray.FromPackedString_(
	            "2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT@L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5L#P5@CH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MP@2MCT2MCTP$IBP$IBP$IB@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@2MCT2MCT2MCT@L#P5L#P5DIBP-[V5-[V5-[V5 @@@@@@@@@@@J6-+Z6-+Z6-+P@@@J6-+P@@@L#P5L#P5@CH4MSH4MQBP$KV5-Z-+Z6-+Z4@@@@@@@@@@@B-+Z6-+Z6-+Z4@@@B-+Z6-+Z4@@@CH4MP@2MCT2MCTP$IB5-[V+Z6-+Z6-@@@@@@@@@@@@+Z6-+Z6-+Z6-@@@@+Z6-+Z6-@@@@2MCT@L#P5L#P5DIBP-[V5*6-+Z6-+P@@@@@@@@@@@@@@@@@@@@@@@J6-+Z6-+Z6-+P@@@L#P5@CH4MSH4MQBP$KV5-Z-+Z6-+Z6-+Z6-+Z6-+Z6-+Z6-+Z6-+Z6-+Z6-+Z6-+Z4@@@CH4MP@2MCT2MCTP$IB5-[V+Z6-X6M#X6M#X6M#X6M#X6M#X6M#X6M#X6M#+Z6-+Z6-@@@@2MCT@L#P5L#P5DIBP-[V5&M#X?????????????????????????????_&+VM#X:6-+P@@@L#P5@CH4MSH4MQBP$KV5-Y#X6O????????????????????????79*7???=#X6N-+Z4@@@CH4MP@2MCT2MCTP$IB5-[VX6M#????????????????????=>Z-????=>Z-X6M#+Z6-@@@@2MCT@L#P5L#P5DIBP-[V5&M#X?????????????????_&+_????_&+__&+VM#X:6-+P@@@L#P5@CH4MSH4MQBP$KV5-Y#X6O????????????79*7????79*779*779*5#X6L@@@@@@@CH4MP@2MCT2MCTP$IB5-[VX6M#????????=>Z-????=>Z-=>Z-=>Z-=>Z-X6M#????@@@@2MCT@L#P5L#P5@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@L#P5@CH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MSH4MP@2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT2MCT@@@a"),
	        new DirectColorModel(24, 255 << 0, 255 << 8, 255 << 16));
	}
	/**
	 * 
	 * @return sra.smalltalk.StImage
	 */
	protected static StImage Image2() {
	    return new StImage(
	        17,
	        19,
	        StByteArray.FromPackedString_(
	            "@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@P$IB@DIBP ABP$H@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@2MCT@L#P5@CH4MP@2MCT@L#P5@ABP$H@5-[V@M[V5 CV5-X@@@@@@@@@@@@@@@@@+Z6-@J6-+PB-+Z4@@@@@@J6-+P@@@@@@2MCT@L#P5@CH4MP@2MCT@DIBP CV5-X@+Z6-@J6-+P@@@@@@@@@@@@@@@@B-+Z4@+Z6-@J6-+P@@@@@@+Z6-@J6-+P@@@@@@2MCT@L#P5@CH4MP@P$IB@M[V5 B-+Z4@+Z6-@@@@@@@@@@@@@@@@@J6-+PB-+Z4@+Z6-@@@@@@B-+Z4@+Z6-@@@@@@CH4MP@2MCT@L#P5@ABP$H@5-[V@J6-+PB-+Z4@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@+Z6-@J6-+PB-+Z4@@@@@@L#P5@CH4MP@2MCT@DIBP CV5-X@+Z6-@J6-+PB-+Z4@+Z6-@J6-+PB-+Z4@+Z6-@J6-+PB-+Z4@+Z6-@J6-+P@@@@@@2MCT@L#P5@CH4MP@P$IB@M[V5 B-+Z4@X6M#@FM#X0A#X6L@X6M#@FM#X0A#X6L@X6M#@FM#X0B-+Z4@+Z6-@@@@@@CH4MP@2MCT@L#P5@ABP$H@5-[V@FM#X0C???<@????@O???0C???<@????@O???0C???<@=>Z-@FM#X0B-+Z4@@@@@@L#P5@CH4MP@2MCT@DIBP CV5-X@X6M#@O???0C???<@????@O???0C???<@????@O_&+PC???<@X6M#@J6-+P@@@@@@2MCT@L#P5@CH4MP@P$IB@M[V5 A#X6L@????@O???0C???<@????@O???0C79*4@????@O_&+PA#X6L@+Z6-@@@@@@CH4MP@2MCT@L#P5@ABP$H@5-[V@FM#X0C???<@????@O???0C???<@=>Z-@O???0C79*4@=>Z-@FM#X0B-+Z4@@@@@@L#P5@CH4MP@2MCT@DIBP CV5-X@X6M#@O???0C???<@????@O_&+PC???<@=>Z-@O_&+PC79*4@X6M#@@@@@@@@@@@@2MCT@L#P5@CH4MP@P$IB@M[V5 A#X6L@????@O???0C79*4@????@O_&+PC79*4@=>Z-@O_&+PA#X6L@????@@@@@@CH4MP@2MCT@L#P5@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MP@2MCT@L#P5@CH4MPb"),
	        new DirectColorModel(32, 0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000));
	}
	/**
	 * 
	 * @return sra.smalltalk.StImage
	 */
	protected static StImage Image3() {
	    byte[] rgb = new byte[] {(byte) 255, (byte) 0 };
	    return new StImage(
	        16,
	        16,
	        new byte[] {
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0,
	            (byte) 255,
	            (byte) 0 },
	        new IndexColorModel(1, 2, rgb, rgb, rgb));
	}
	/**
	 * Execute all examples.
	 * @param args java.lang.String[]
	 */
	public static void main(String args[]) {
	    new StImageTestExamples();
	}
}
