package sra.smalltalk;

import java.io.Writer;
import java.io.IOException;
import java.lang.IndexOutOfBoundsException;

/**
 * StInterval class
 * 
 * 	@author    NISHIHARA Satoshi
 * 	@created   2000/01/13 (by NISHIHARA Satoshi)
 * 	@updated   2000/01/21 (by NISHIHARA Satoshi)
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StInterval.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StInterval extends StObject {
	/**
	 * initial number in the progression.
	 */
	private double start;
	/**
	 * last number in the progression.
	 */
	private double stop;
	/**
	 * increment number for determining the next number in the progression.
	 */
	private double step = 1.0;
	public StInterval() {
		super();
	}
	/**
	 * Create a new StInterval.
	 * @param startNumber double
	 * @param stopNumber double
	 */
	public StInterval(double startNumber, double stopNumber) {
		super();
		this.setFrom_to_by_(startNumber, stopNumber, 1);
	}
	/**
	 * Create a new StInterval.
	 * @param startNumber double
	 * @param stopNumber double
	 * @param stepNumber double
	 */
	public StInterval(double startNumber, double stopNumber, double stepNumber) {
		super();
		this.setFrom_to_by_(startNumber, stopNumber, stepNumber);
	}
	/**
	 * Computes the number of the index of this progression.
	 *
	 * @param int
	 * @return double
	 */
	public double at_(int index) {
		if ((index >= 1) && (index <= this.size())) {
			return start + (step * (index - 1));
		}
		throw new IndexOutOfBoundsException("size: " + this.size() + " index: " + index);
	}
	/**
	 * Enumerate all elements of the receiver and evaluate the StBlockClosure.
	 *
	 * @param aBlock,sra.smalltalk.StBlockClosure
	 * @return double[]
	 */
	public double[] collect_(StBlockClosure aBlock) {
		double[] result = new double[this.size()];
		double nextValue = start;
		for (int i = 0; i < result.length; i++) {
			result[i] = ((Double) (aBlock.value_(new Double(nextValue)))).doubleValue();
			nextValue += step;
		}
		return result;
	}
	/**
	 * Enumerate all elements of the receiver and evaluate the StBlockClosure.
	 *
	 * @param aBlock,sra.smalltalk.StBlockClosure
	 */
	public void do_(StBlockClosure aBlock) {
		double value = start;
		if (step < 0) {
			while (stop <= value) {
				aBlock.value_(new Double(value));
				value += step;
			}
		} else {
			while (stop >= value) {
				aBlock.value_(new Double(value));
				value += step; 
			}
		}
	}
	/**
	 * Answer true if the Object is equal to the receiver, otherwise false.
	 *
	 * @return boolean
	 * @param anObject java.lang.Object
	 * @see java.util.Hashtable
	 */
	public boolean equals(Object other) {
		if (other.getClass() == this.getClass()) {
			StInterval aStInterval = (StInterval) other;
			return (
				(this.start == (aStInterval.start)) && 
				(this.step == (aStInterval.increment())) && 
				(this.size() == aStInterval.size())
			);
		}
		return false;
	}
	/**
	 * initinal number of this progression.
	 *
	 * @return double
	 */
	public double first() {
		return start;
	}
	/**
	 * Create a new StInterval.
	 * @param startNumber double
	 * @param stopNumber double
	 */
	public static StInterval From_to_(double startNumber, double stopNumber) {
		StInterval interval = new StInterval();
		interval.setFrom_to_by_(startNumber, stopNumber, 1);
		return interval;
	}
	/**
	 * Create a new StInterval.
	 * @param startNumber double
	 * @param stopNumber double
	 */
	public static StInterval From_to_by_(double startNumber, double stopNumber, double stepNumber) {
		StInterval interval = new StInterval();
		interval.setFrom_to_by_(startNumber, stopNumber, stepNumber);
		return interval;
	}
	// Hash Function

	/**
	 * Computes a hash code for this object.
	 *
	 * @return int
	 * @see java.util.Hashtable
	 */
	public int hashCode() {
		long bitsStart = Double.doubleToLongBits(start);
		int startHash = (int) (bitsStart ^ (bitsStart >> 32));
		long bitsStop = Double.doubleToLongBits(stop);
		int stopHash = (int) (bitsStop ^ (bitsStop >> 32));
		return (((startHash >> 2) | stopHash) >> 1 | this.size());
	}
	/**
	 * increment number of this progression.
	 *
	 * @return double
	 */
	public double increment() {
		return step;
	}
	/**
	 * Computes the last number of this progression.
	 *
	 * @return double
	 */
	public double last() {
		return stop - ((stop - start) % step);
	}
	/**
	 * Print my string representation on aWriter.
	 *
	 * @param aWriter java.io.Writer
	 */
	public void printOn_(Writer aWriter) throws IOException {
		aWriter.write("(" + start + " to: " + stop);
		if (step != 1) {
			aWriter.write(" by: " + step);
		}
		aWriter.write(")");
	}
	/**
	 * Enumerate all elements of the receiver and evaluate the StBlockClosure.
	 *
	 * @param aBlock,sra.smalltalk.StBlockClosure
	 */
	public void reverseDo_(StBlockClosure aBlock) {
		double value = stop;
		if (step < 0) {
			while (start >= value) {
				aBlock.value_(new Double(value));
				value -= step;
			}
		} else {
			while (start <= value) {
				aBlock.value_(new Double(value));
				value -= step; 
			}
		}
	}
	/**
	 * private
	 *
	 * @param double
	 * @param double
	 * @param double
	 * @return sra.smalltalk.StInterval
	 */
	private StInterval setFrom_to_by_(double startNumber, double stopNumber, double stepNumber) {
		this.start = startNumber;
		this.stop = stopNumber;
		this.step = stepNumber;
		return this;
	}
	/**
	 * Computes the size of this progression.
	 *
	 * @return int
	 */
	public int size() {
		int size = 0;
		if (step < 0) {
			if (!(start < stop)) {
				size = (int) ((stop - start) / step) + 1;
			}
		} else {
			if (!(stop < start)) {
				size = (int) ((stop - start) / step) + 1;
			}
		}
		return size;
	}
	/**
	 * get initial number in the progression.
	 *
	 * @return double
	 */
	public double start() {
		return this.start;
	}
	/**
	 * set initial number in the progression.
	 *
	 * @param double
	 * @return double
	 */
	public double start_(double startValue) {
		return this.start = startValue;
	}
	/**
	 * get increment number for determining the next number in the progression.
	 *
	 * @return double
	 */
	public double step() {
		return this.step;
	}
	/**
	 * set increment number for determining 
	 * the next number in the progression.
	 *
	 * @param double
	 * @return double
	 */
	public double step_(double stepValue) {
		return this.step = stepValue;
	}
	/**
	 * get last number in the progression.
	 *
	 * @return double
	 */
	public double stop() {
		return this.stop;
	}
	/**
	 * set last number in the progression.
	 *
	 * @param double
	 * @return double
	 */
	public double stop_(double stopValue) {
		return this.stop = stopValue;
	}
	/**
	 * Store my string representation on the writer.
	 * @param aWriter java.io.Writer
	 */
	public void storeOn_(Writer aWriter) throws IOException {
		this.printOn_(aWriter);
	}
}
