package sra.smalltalk;

import java.util.Vector;

/**
 * StMenu class
 * 
 * 	@author    MATSUDA Ryouichi
 * 	@created   1999/09/16 (by MATSUDA Ryouichi)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StMenu.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StMenu extends StMenuItem {
	protected Vector children = new Vector();
	/**
	 * StMenu constructor comment.
	 * @param aLabel java.lang.String
	 */
	public StMenu(String aLabel) {
		super(aLabel);
	}
	/**
	 * @param aMenu sra.smalltalk.StMenuItem
	 */
	public void add(StMenuItem aMenu) {
		children.addElement(aMenu);
		aMenu.setParent(this);
	}
	/**
	 * Adds a separator line.
	 */
	public void addSeparator() {
		children.addElement(null);
	}
	/**
	 * Gets the item located at the specified index of this menu.
	 *
	 * @param index int
	 * @return sra.smalltalk.StMenuItem
	 */
	public StMenuItem getItem(int index) {
		if (index < 0 || children.size() <= index) {
			return null;
		}
		return (StMenuItem) children.elementAt(index);
	}
	/**
	 * Gets the item located at the specified label of this menu.
	 *
	 * @param label java.lang.String
	 * @return sra.smalltalk.StMenuItem
	 */
	public StMenuItem getItem(String label) {
		for (int i = 0; i < children.size(); i++) {
			StMenuItem item = (StMenuItem) children.elementAt(i);
			if (item != null && label.equals(item.label)) {
				return item;
			}
		}
		return null;
	}
	/**
	 * Gets the children items.
	 *
	 * @return java.util.Vector
	 */
	public Vector getItems() {
		return children;
	}
}
