package sra.smalltalk;

import java.awt.event.*;
import java.util.Vector;

/**
 * StMenuItem class
 * 
 * 	@author    MATSUDA Ryouichi
 * 	@created   1999/09/16 (by MATSUDA Ryouichi)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StMenuItem.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StMenuItem {
	private Vector actionlisteners;
	protected String label;
	protected String actionCommand;
	protected boolean enabled = true;
	protected StMenu parent;
	/**
	 * Replacement for the # notation of Smalltalk. :-)
	 *
	 * @return sra.smalltalk.StSymbol
	 * @param aString java.lang.String
	 */
	public final static StSymbol $(String aString) {
		return StSymbol.Intern_(aString);
	}
	/**
	 * Creates a menuItem with label.
	 *
	 * @param aLabel java.lang.String
	 */
	public StMenuItem(String aLabel) {
		super();
		label = aLabel;
	}
	/**
	 *
	 */
	public void action() {
		fireActionPerformed(new ActionEvent(this,
			ActionEvent.ACTION_PERFORMED, getActionCommand()));
	}
	/**
	 * adds an ActionListener to the button
	 */
	public synchronized void addActionListener(ActionListener l) {
		if (actionlisteners == null) {
			actionlisteners = new Vector();
		}
		actionlisteners.addElement(l);
	}
	/**
	 * Invoked when a menu item has been selected. 
	 *
	 * @param e java.awt.event.ActionEvent 
	 */
	protected void fireActionPerformed(ActionEvent event) {
		if (actionlisteners == null) {
			return;
		}
		Vector copy;
		synchronized (actionlisteners) {
			copy = (Vector) actionlisteners.clone();
		}
		for (int i = 0; i < copy.size(); i++) {
			ActionListener l = (ActionListener) copy.elementAt(i);
			l.actionPerformed(event);
		}
	}
	/**
	 * Returns the command string associated with this action.
	 *
	 * @return java.lang.String
	 */
	public String getActionCommand() {
		return actionCommand;
	}
	/**
	 * @return sra.smalltalk.StMenu
	 */
	protected StMenu getParent() {
		return parent;
	}
	/**
	 * @return sra.smalltalk.StMenuModel
	 */
	protected StMenuItem getRoot() {
		StMenuItem current = this;
		StMenuItem parent = getParent();
		while (parent != null) {
			current = parent;
			parent = current.getParent();
		}
		return current;
	}
	/**
	 *
	 *
	 * @return boolean
	 */
	public boolean isEnabled() {
		return enabled;
	}
	/**
	 * removes an ActionListener from the button
	 */
	public void removeActionListener(ActionListener l) {
		actionlisteners.removeElement(l);
	}
	/**
	 * Sets the command name of the action event that is fired
	 * by this menu item.
	 *
	 * @param command java.lang.String
	 */
	public void setActionCommand(String command) {
		actionCommand = command;
	}
	/**
	 * Enable or disable the menu item.
	 *
	 * @param aBoolean boolean
	 */
	public void setEnabled(boolean aBoolean) {
		if (enabled != aBoolean) {
			enabled = aBoolean;
			StSymbol aspect = enabled ? $("enabled") : $("disable");
			StMenuItem root = getRoot();
			if (root instanceof StMenuModel) {
				((StMenuModel) root).changed_with_(aspect, this);
			}
		}
	}
	/**
	 * @param aMenu sra.smalltalk.StMenu
	 */
	protected void setParent(StMenu aMenu) {
		parent = aMenu;
	}
}
