package sra.smalltalk;

import java.util.Vector;

/**
 * StSortedCollection class
 * 
 * 	@author    nisinaka
 * 	@created   1998/12/02 (by nisinaka)
 * 	@updated   1999/08/05 (by nisinaka)
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StSortedCollection.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StSortedCollection extends StObject {
	/**
	 * The block closure for comparing two elements.
	 */
	protected StBlockClosure sortBlock = null;

	/**
	 * The collection which holds the elements in order.
	 */
	protected Vector _elements = new Vector();
	/**
	 * Need to specify the sort block.
	 */
	private StSortedCollection() {
	}
	/**
	 * Create a new instance of StSortedCollection with a sort block.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public StSortedCollection(StBlockClosure aBlock) {
		super();
		this.sortBlock_(aBlock);
	}
	/**
	 * Create a new instance of StSortedCollection with a sort block.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 * @param anArray java.lang.Object[]
	 */
	public StSortedCollection(StBlockClosure aBlock, Object[] anArray) {
		this(aBlock);
		for (int i = 0; i < anArray.length; i++) {
			this.add_(anArray[i]);
		}
	}
	/**
	 * Create a new instance of StSortedCollection with a sort block.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public StSortedCollection(StBlockClosure aBlock, Vector aCollection) {
		this(aBlock);
		Object[] anArray = new Object[aCollection.size()];
		aCollection.copyInto(anArray);
		for (int i = 0; i < anArray.length; i++) {
			this.add_(anArray[i]);
		}
	}
	/**
	 * Answer the receiver's elements as array.
	 *
	 * @return java.lang.Object[]
	 */
	public Object[] _asArray() {
		Object[] newArray = new Object[_elements.size()];
		_elements.copyInto(newArray);
		return newArray;
	}
	/**
	 * Add the new object as one of the receiver's element.
	 *
	 * @return java.lang.Object
	 * @param newObject java.lang.Object
	 */
	public synchronized Object add_(Object newObject) {
		if (_elements.isEmpty()) {
			_elements.addElement(newObject);
		} else {
			int nextIndex = this.indexForInserting_(newObject);
			_elements.insertElementAt(newObject, nextIndex);
		}
		return newObject;
	}
	/**
	 * Compare the two elements.
	 *
	 * @return boolean
	 * @param e1 java.lang.Object
	 * @param e2 java.lang.Object
	 */
	private boolean compareElement_to_(Object e1, Object e2) {
		if (sortBlock == null) {
			throw new SmalltalkException("No sort block.");
		}
		return ((Boolean) sortBlock.value_value_(e1, e2)).booleanValue();
	}
	/**
	 * Answer the first element of the collection.
	 *
	 * @return java.lang.Object
	 */
	public synchronized Object first() {
		return _elements.firstElement();
	}
	/**
	 * Answer the index for inserting the new object.
	 *
	 * @return int
	 * @param newObject java.lang.Object
	 */
	private int indexForInserting_(Object newObject) {
		int low = 0;
		int high = _elements.size() - 1;
		int index = (high + low) / 2;
		while (low <= high) {
			if (this.compareElement_to_(_elements.elementAt(index), newObject)) {
				low = index + 1;
			} else {
				high = index - 1;
			}
			index = (high + low) / 2;
		}
		return low;
	}
	/**
	 * Answer the last element of the collection.
	 *
	 * @return java.lang.Object
	 */
	public synchronized Object last() {
		return _elements.lastElement();
	}
	/**
	 * Answer my size.
	 *
	 * @return int
	 */
	public int size() {
		return _elements.size();
	}
	/**
	 * Set the sort block of the receiver.
	 *
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public void sortBlock_(StBlockClosure aBlock) {
		sortBlock = aBlock;
	}
}
