package sra.smalltalk;

import java.util.Hashtable;

/**
 * StSymbol class
 * 
 * 	@author    He Weijie
 * 	@created   1998/09/16 (by HeWeijie)
 * 	@updated   1998/11/04 (by nisinaka)
 * 	@updated   1999/06/22 (by nisinaka)
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StSymbol.java,v 4.3 2002/07/11 09:18:29 m-asada Exp $
 */
public class StSymbol extends StObject {
	/**
	 * The string representation.
	 */
	protected String _string;

	//
	protected static Hashtable _symbols = new Hashtable();
	/**
	 * The instance of StSymbol should not be created with
	 * the default constructor.
	 */
	private StSymbol() {
	}
	/**
	 * Create a new instance of StSymbol.
	 *
	 * @param aString java.lang.String
	 */
	private StSymbol(String aString) {
		_string = aString.intern();
		_symbols.put(_string, this);
	}
	/**
	 * Answer true if the receiver is equal to the Object, otherwise false.
	 *
	 * @return boolean
	 * @param anObject java.lang.Object
	 */
	public boolean equals(Object anObject) {
		return (this == anObject);
	}
	/**
	 * Answer the symbol specified with the String.
	 *
	 * @param java.lang.String
	 * @return sra.smalltalk.StSymbol
	 */
	public static StSymbol Intern_(String aString) {
		StSymbol aSymbol = (StSymbol) _symbols.get(aString.intern());
		if (aSymbol == null) {
			aSymbol = new StSymbol(aString);
		}
		return aSymbol;
	}
	/**
	 * @deprecated replaced by <code>==</code>.
	 * @return boolean
	 * @param aSymbol sra.smalltalk.StSymbol
	 */
	public boolean isEqual(StSymbol aSymbol) {
		return (this == aSymbol);
	}
	/**
	 * Answer the size.
	 *
	 * @return int
	 */
	public int size() {
		return _string.length();
	}
	/**
	 * Returns a string representation of the StString.
	 *
	 * @return java.lang.String
	 */
	public String toString() {
		return _string;
	}
}
