package sra.smalltalk;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * StUIBuilder class
 * 
 * 	@author    He Weijie
 * 	@created   1998/08/27 (by He Weijie)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StUIBuilder.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StUIBuilder extends StObject implements WindowListener {
	protected StApplicationModel model;
	protected Vector frames = new Vector();
	/**
	 * 
	 * @param aModel sra.smalltalk.StApplicationModel
	 */
	public StUIBuilder(StApplicationModel aModel) {
		super();
		model = aModel;
	}
	/**
	 * 
	 * @param aFrame java.awt.Frame
	 */
	public void addFrame_(Frame aFrame) {
		frames.addElement(aFrame);
		aFrame.addWindowListener(this);
	}
	/**
	 * 
	 * @return java.awt.Frame[]
	 */
	public Frame[] frames() {
		Frame[] array;
		synchronized (frames) {
			array = new Frame[frames.size()];
			frames.copyInto(array);
		}
		return array;
	}
	/**
	 * 
	 * @return sra.smalltalk.StApplicationModel
	 */
	protected StApplicationModel model() {
		return model;
	}
	/**
	 * 
	 * @param aFrame java.awt.Frame
	 */
	public void removeFrame_(Frame aFrame) {
		aFrame.removeWindowListener(this);
		frames.removeElement(aFrame);
	}
	/**
	 * Invoked when a window is activated.
	 */
	public void windowActivated(WindowEvent e) {
	}
	/**
	 * Invoked when a window has been closed.
	 */
	public void windowClosed(WindowEvent e) {
	}
	/**
	 * Invoked when a window is in the process of being closed.
	 * The close operation can be overridden at this point.
	 */
	public void windowClosing(WindowEvent e) {
		this.model().windowClosing(e);
	}
	/**
	 * Invoked when a window is de-activated.
	 */
	public void windowDeactivated(WindowEvent e) {
	}
	/**
	 * Invoked when a window is de-iconified.
	 */
	public void windowDeiconified(WindowEvent e) {
	}
	/**
	 * Invoked when a window is iconified.
	 */
	public void windowIconified(WindowEvent e) {
	}
	/**
	 * Invoked when a window has been opened.
	 */
	public void windowOpened(WindowEvent e) {
	}
}
