package sra.smalltalk;

import java.io.*;

/**
 * StValueHolder class
 * 
 * 	@author    nisinaka
 * 	@created   1998/10/09 (by nisinaka)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StValueHolder.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StValueHolder extends StValueModel {
	protected Object value = null;
	/**
	 * Default constructor.
	 */
	public StValueHolder() {
		super();
	}
	/**
	 * Create a new instance and initialize it with aNumber
	 *.
	 * @param aNumber double
	 */
	public StValueHolder(double aNumber) {
		super();
		this.setValue_(new Double(aNumber));
	}
	/**
	 * Create a new instance and initialize it with aNumber
	 *.
	 * @param aNumber float
	 */
	public StValueHolder(float aNumber) {
		super();
		this.setValue_(new Float(aNumber));
	}
	/**
	 * Create a new instance and initialize it with aNumber
	 *.
	 * @param aNumber int
	 */
	public StValueHolder(int aNumber) {
		super();
		this.setValue_(new Integer(aNumber));
	}
	/**
	 * Create a new instance and initialize it with aNumber
	 *.
	 * @param aNumber long
	 */
	public StValueHolder(long aNumber) {
		super();
		this.setValue_(new Long(aNumber));
	}
	/**
	 * Create a new instance and initialize it with anObject.
	 *
	 * @param anObject java.lang.Object
	 */
	public StValueHolder(Object anObject) {
		super();
		this.setValue_(anObject);
	}
	/**
	 * Assume 'value' is a kind of Number and return it as double.
	 *
	 * @return double
	 */
	public double _doubleValue() {
		return ((Number) value).doubleValue();
	}
	/**
	 * Assume 'value' is a kind of Number and return it as float.
	 *
	 * @return float
	 */
	public float _floatValue() {
		return ((Number) value).floatValue();
	}
	/**
	 * Assume 'value' is a kind of Number and return it as int.
	 *
	 * @return int
	 */
	public int _intValue() {
		return ((Number) value).intValue();
	}
	/**
	 * Assume 'value' is a kind of Number and return it as long.
	 *
	 * @return long
	 */
	public long _longValue() {
		return ((Number) value).longValue();
	}
	/**
	 * Print my string representation on aWriter.
	 * @param aWriter java.io.Writer
	 */
	public void printOn_(Writer aWriter) throws IOException {
		super.printOn_(aWriter);
		aWriter.write(" on: ");
		aWriter.write(this.value().toString());
	}
	/**
	 * Set the currently stored value, without notifying dependents.
	 *
	 * @param newValue java.lang.Object
	 */
	public void setValue_(Object newValue) {
		value = newValue;
	}
	/**
	 * Answer the currently stored value.
	 *
	 * @return java.lang.Object
	 */
	public Object value() {
		return value;
	}
	/**
	 * Set the currently stored value, and notify dependents.
	 *
	 * @param aNumber double
	 */
	public void value_(double aNumber) {
		super.value_(new Double(aNumber));
	}
	/**
	 * Set the currently stored value, and notify dependents.
	 *
	 * @param aNumber float
	 */
	public void value_(float aNumber) {
		super.value_(new Float(aNumber));
	}
	/**
	 * Set the currently stored value, and notify dependents.
	 *
	 * @param aNumber int
	 */
	public void value_(int aNumber) {
		super.value_(new Integer(aNumber));
	}
	/**
	 * Set the currently stored value, and notify dependents.
	 *
	 * @param aNumber long
	 */
	public void value_(long aNumber) {
		super.value_(new Long(aNumber));
	}
	/**
	 * Create a new instance and initialize it with anObject.
	 *
	 * @return sra.smalltalk.StValueHolder
	 * @param aClass java.lang.Class
	 * @param anObject java.lang.Object
	 */
	public static StValueHolder With_(Class aClass, Object anObject) {
		return (StValueHolder) _New(aClass, anObject);
	}
	/**
	 * Create a new instance of StValueHolder and initialize it with anObject.
	 *
	 * @return sra.smalltalk.StValueHolder
	 * @param anObject java.lang.Object
	 */
	public final static StValueHolder With_(Object anObject) {
		return With_(StValueHolder.class, anObject);
	}
}
