package sra.smalltalk;

/**
 * StValueModel class
 * 
 * 	@author    nisinaka
 * 	@created   1998/10/09 (by nisinaka)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StValueModel.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public class StValueModel extends StModel {
	/**
	 * Create a new instance and initialize it.
	 */
	public StValueModel() {
		super();
		this.initialize();
	}
	/**
	 * Convert this as StValueModel.
	 *
	 * @return sra.smalltalk.StValueModel
	 */
	public StValueModel asValue() {
		return this;
	}
	/**
	 * Answer a StBlockValue that computes aBlock with the receiver's value
	 * as the argument.
	 *
	 * @return sra.smalltalk.StBlockValue
	 * @param aBlock sra.smalltalk.StBlockClosure
	 */
	public StBlockValue compute_(StBlockClosure aBlock) {
		StValueModel[] anArray = {this};
		return StBlockValue.Block_arguments_(aBlock, anArray);
	}
	/**
	 * Initialize the instance. Subclasses may extend this.
	 *
	 * @return sra.smalltalk.StObject
	 */
	public StObject initialize() {
		return this;
	}
	/**
	 * Arrange to receive a message with aSymbol when the value aspect
	 * changes on anObject.
	 *
	 * @param aSymbol java.lang.String
	 * @param anObject sra.smalltalk.StObject
	 */
	public void onChangeSend_to_(String aSymbol, StObject anObject) {
		this.expressInterestIn_for_sendBack_($("value"), anObject, aSymbol);
	}
	/**
	 * Remove references to objects that may refer to the receiver.
	 */
	public void release() {
		this.releaseParts();
		super.release();
	}
	/**
	 * Break the dependency links from any parts of myself to myself.
	 * Subclasses holding composite values will implement this in a
	 * non-trivial way.
	 */
	public void releaseParts() {
	}
	/**
	 * Undo a send of onChangeSend_to_().
	 *
	 * @param anObject sra.smalltalk.StObject
	 */
	public void retractInterestsFor_(StObject anObject) {
		this.retractInterestIn_for_($("value"), anObject);
	}
	/**
	 * Set the currently stored value, without notifying dependents.
	 *
	 * @param newValue java.lang.Object
	 */
	public void setValue_(Object newValue) {
		throw SmalltalkException.SubclassResponsibility();
	}
	/**
	 * Answer the currently stored value.
	 *
	 * @return java.lang.Object
	 */
	public Object value() {
		throw SmalltalkException.SubclassResponsibility();
	}
	/**
	 * Set the currently stored value, and notify dependents.
	 *
	 * @param newValue java.lang.Object
	 */
	public void value_(Object newValue) {
		this.setValue_(newValue);
		this.changed_($("value"));
	}
}
