package sra.smalltalk;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

/**
 * StViewJPanel class
 * 
 * 	@author    MATSUDA Ryouichi
 * 	@created   1999/11/11 (by MATSUDA Ryouichi)
 * 	@updated   2002/04/12 (by nisinaka)
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: StViewJPanel.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public abstract class StViewJPanel extends JPanel implements StView {

    protected StModel model = null;
    protected StController controller = null;

    /**
     * Replacement for the # notation of Smalltalk. :-)
     *
     * @return sra.smalltalk.StSymbol
     * @param aString java.lang.String
     */
    public final static StSymbol $(String aString) {
        return StObject.$(aString);
    }
    /**
     * Create a new instance of StViewJPanel and initialize it.
     */
    public StViewJPanel() {
        super();

        this.buildComponent();
    }
    /**
     * Make sure to have both a model and a controller before showing.
     */
    public void addNotify() {
        if (model == null) {
            this.model_(this.defaultModel());
        }

        if (controller == null) {
            this.controller_(this.defaultController());
        }

        super.addNotify();
    }
    /**
     * Build this component.
     */
    protected void buildComponent() {
        // Do nothing as a default.
    }
    /**
     * Answer my current controller.
     *
     * @return sra.smalltalk.StController
     */
    public StController controller() {
        if (controller == null) {
            this.controller_(this.defaultController());
        }
        return controller;
    }
    /**
     * Set my new controller.
     *
     * @param newController sra.smalltalk.StController
     */
    public void controller_(StController newController) {
        if (newController != null) {
            newController.view_(this);
        }
        controller = newController;
    }
    /**
     * Answer my default controller.
     * Subclasses may override this.
     *
     * @return sra.smalltalk.StController
     */
    protected StController defaultController() {
        return new StController();
    }
    /**
     * Answer a default model.
     * Subclasses may override this.
     *
     * @return sra.smalltalk.StModel
     */
    protected StModel defaultModel() {
        return new StModel();
    }
    /**
     * Paints this component.
     *
     * @param aGraphics java.awt.Graphics
     */
    public void displayOn_(Graphics aGraphics) {
        // Do nothing as a default.
    }
    /**
     * Answer my current model.
     *
     * @return sra.smalltalk.StModel
     */
    public StModel model() {
        if (model == null) {
            this.model_(this.defaultModel());
        }
        return model;
    }
    /**
     * Set my new model.
     *
     * @param newModel sra.smalltalk.StModel
     */
    public void model_(StModel newModel) {
        if (model != newModel) {
            if (model != null) {
                model.removeDependent_(this);
            }
            StModel oldModel = model;
            model = newModel;
            if (model != null) {
                model.addDependent_(this);
            }
            rebuildComponent(oldModel, newModel);
            repaint();
            firePropertyChange("model", oldModel, newModel);
        }
    }
    /**
     * Paints this component.
     *
     * @param @param aGraphics java.awt.Graphics
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        this.displayOn_(g);
    }
    /**
     * Rebuild this component.
     *
     * @param oldModel sra.smalltalk.StModel
     * @param newModel sra.smalltalk.StModel
     */
    protected void rebuildComponent(StModel oldModel, StModel newModel) {
        // Do nothing as a default.
    }
    /**
     * Receive a change notice from an object of whom the receiver is a
     * dependent.  The argument anAspectSymbol is typically a Symbol
     * that indicates what change has occurred.
     *
     * @param evt sra.smalltalk.DependentEvent
     */
    public void update_(DependentEvent evt) {
        repaint();
    }
}