package sra.smalltalk;

import java.awt.Point;
import java.awt.Rectangle;

/**
 * SystemInterface class
 * 
 * 	@author    Hoshi Takanori
 * 	@created   2001/11/30 (by Hoshi Takanori)
 * 	@updated   N/A
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: SystemInterface.java,v 4.2 2002/07/11 09:18:29 m-asada Exp $
 */
public final class SystemInterface {
	public static final String libraryName = "StPL";
	public static final String requestVersion = "1.0";
	public static final int requestBuildNo = 4;

	private static SystemInterface Current = new SystemInterface();
	static {
		try {
			System.loadLibrary(libraryName);
			String findVersion = getVersion();
			String buildPrefix = requestVersion + " build ";
			int buildNo;
			try {
				buildNo = Integer.parseInt(findVersion.substring(buildPrefix.length()));
			} catch (NumberFormatException e) {
				buildNo = 0;
			}
			if (findVersion.startsWith(buildPrefix) && buildNo >= requestBuildNo) {
				System.out.println("JNI library '" + libraryName + "' version " + findVersion);
			} else {
				System.out.println("JNI library '" + libraryName + "' version error.");
				System.out.println("Find library is version " + findVersion);
				System.out.println("Request library is version " + requestVersion + " build " + requestBuildNo);
				throw new Error("JNI library '" + libraryName + "' version error");
			}
		} catch (UnsatisfiedLinkError e) {
			System.out.println("Can't find JNI librally '" + libraryName + "'.");
			throw e;
		}
	}

	/**
	 * Constainer Method.
	 */
	protected SystemInterface() {
		// Singleton pattern
		super();
	}

	/**
	 * Answer the only instance of SystemInterface.
	 * @return sra.smalltalk.SystemInterface
	 *
	 * Call native methods in a block of synchronized.
	 * sample:
	 *   SystemInterface StPLInterface = SystemInterface.Current();
	 *   synchronized (StPLInterface) {
	 *     pixels = StPLInterface.utilImageFromUser(dimension);
	 *   }
	 */
	public static SystemInterface Current() {
		return Current;
	}

	/**
	 * Answer a rectangle from user.
	 * @return java.awt.Rectangle
	 */
	public static Rectangle _RectangleFromUser() {
		SystemInterface StPLInterface = SystemInterface.Current();
		int[] rect;

		synchronized (StPLInterface) {
			rect = StPLInterface.utilRectangleFromUser();
		}

		if (rect != null) {
			return new Rectangle(rect[0], rect[1], rect[2], rect[3]);
		} else {
			return null;
		}
	}

	/**
	 * Answer the current mouse position.
	 * @return java.awt.Point
	 */
	public static Point _MousePoint() {
		SystemInterface StPLInterface = SystemInterface.Current();
		int[] point;

		synchronized (StPLInterface) {
			point = StPLInterface.utilMousePoint();
		}

		if (point != null) {
			return new Point(point[0], point[1]);
		} else {
			return null;
		}
	}

	/**
	 * Native Methods.
	 */
	public static native String getVersion();
	public native int[] utilImageFromUser(int[] dimension);
	public native int[] utilImageOfArea(int x, int y, int width, int height);
	public native int[] utilImageOfWholeScreen(int[] dimension);
	public native int[] utilRectangleFromUser();
	public native int utilColorSpuit();
	public native int[] utilMousePoint();
	public native void utilBrowse(String url);
}
