package sra.smalltalk;

import java.awt.*;
import java.net.*;

/**
 * SystemResourceSupport class
 * 
 * 	@author    Hirotsugu Kondo
 * 	@created   1998/12/04 (by Hirotsugu Kondo)
 * 	@updated   2002/04/26 (by Mitsuhiro Asada)
 * 	@version   4.2
 * 	@J2SDK     1.2.2 or higher
 * 	@copyright 1999-2002 SRA (Software Research Associates, Inc.)
 * 	@copyright 2001-2002 SRA/KTL (SRA Key Technology Laboratory, Inc.)
 * 
 * 	 All rights reserved.
 * 
 * 	 Redistribution and use in source and binary forms, with or without
 * 	 modification, are permitted provided that the following conditions
 * 	 are met:
 * 	
 * 	 1. Redistributions of source code must retain the above copyright
 * 	    notice, this list of conditions and the following disclaimer. 
 * 	
 * 	 2. Redistributions in binary form must reproduce the above copyright
 * 	    notice, this list of conditions and the following disclaimer in
 * 	    the documentation and/or other materials provided with the
 * 	    distribution. 
 * 	
 * 	 3. The name of the author may not be used to endorse or promote
 * 	    products derived from this software without specific prior
 * 	    written permission. 
 * 	
 * 	 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * 	 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * 	 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * 	 ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * 	 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * 	 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * 	 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * 	 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * 	 IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * 	 OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
 * 	 IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 	
 * 	 Java and all Java-based trademarks and logos are trademarks or
 * 	 registered trademarks of Sun Microsystems, Inc. in the U.S. or
 * 	 other countries, and are used under license. 
 * 
 * $Id: SystemResourceSupport.java,v 4.3 2002/07/11 09:18:29 m-asada Exp $
 */
public class SystemResourceSupport {
	protected static Frame systemFrame;
	protected static Window systemWindow;
	
	static {
		Initialize();
	}
	/**
	 */
	public SystemResourceSupport() {
	}
	/**
	 * 
	 * @return point
	 */
	public static Point _getStringExtentFor_(String string, Font font) {
		FontMetrics fontMetrics = getFontMetrics(font);
		return new Point(fontMetrics.stringWidth(string), fontMetrics.getHeight());
	}
	/**
	 * Answer a new image.
	 * replaced by <code>StImage(int, int)</code>.
	 * 
	 * @deprecated since StPL41
	 * @return java.awt.Image
	 * @param width int
	 * @param height int
	 */
	public static Image createImage(int width, int height) {
		return getWindow().createImage(width, height);
	}
	/**
	 * Answer a new image.
	 * replaced by <code>StImage(int, int)</code>.
	 * 
	 * @deprecated since StPL41
	 * @return java.awt.Image
	 * @param extent java.awt.Dimension
	 */
	public static Image createImage(Dimension extent) {
		return createImage(extent.width, extent.height);
	}
	/**
	 * Answer a new image.
	 * replaced by <code>StImage(int, int)</code>.
	 * 
	 * @deprecated since StPL41
	 * @return java.awt.Image
	 * @param extent java.awt.Point
	 */
	public static Image createImage(Point extent) {
		return createImage(extent.x, extent.y);
	}
	/**
	 * Answer an image from resourceName.
	 * 
	 * @return java.awt.Image
	 * @param resourceName java.lang.String
	 */
	public static Image createImage(String resourceName) {
		try {
			Class aClass = Class.forName("sra.smalltalk.SystemResourceSupport");
			URL url = aClass.getResource(resourceName);
			return createImage(url);
		} catch (Exception e) {
			System.out.println("Exception: " + e.getMessage());
			e.printStackTrace();
		}
		return null;
	}
	/**
	 * Answer an image from url.
	 * 
	 * @return java.awt.Image
	 * @param url java.net.URL
	 */
	public static Image createImage(URL url) {
		try {
			Frame frame = new Frame();
			Toolkit toolKit = frame.getToolkit();
			Image image = toolKit.getImage(url);
			MediaTracker mt = new MediaTracker(frame);
			mt.addImage(image, 0);
			try {
				mt.waitForAll();
			} catch (InterruptedException e) {
				System.err.println("image load interrupted.");
				return null;
			}
			int status = mt.statusAll(false);
			if ((status & MediaTracker.ABORTED) != 0 || (status & MediaTracker.ERRORED) != 0) {
				System.err.println("could not load image.");
				return null;
			}
			return image;
		} catch (Exception e) {
			System.out.println("Exception: " + e.getMessage());
			e.printStackTrace();
		}
		return null;
	}
	/**
	 * 
	 * @return Font
	 */
	public static Font getFont() {
		return getWindow().getFont();
	}
	/**
	 * 
	 * @return FontMetrics
	 */
	public static FontMetrics getFontMetrics(Font aFont) {
		Font font = aFont;
		if (font == null) {
			font = getFont();
		};
		return getWindow().getFontMetrics(font);
	}
	/**
	 * 
	 * @return java.awt.Frame
	 */
	public static Frame getFrame() {
		return systemFrame;
	}
	/**
	 * 
	 * @return Graphics
	 */
	public static Graphics getGraphics() {
		return getWindow().getGraphics();
	}
	/**
	 * 
	 * @return java.awt.Window
	 */
	public static Window getWindow() {
		// There is a case of systemWindow's peer is null.
		// ex)	run the applet and moving to other page.
		//		Again run that applet, this case will be there.
		//		So, SystemResourceSupport.createImage(w, h) returns null.
		//
		// isDisplayable() returns getPeer() != null
		if (systemWindow.getPeer() == null) {
			Initialize();
		}
		return systemWindow;
	}
	/**
	 * Initialize the class.
	 */
	protected static void Initialize() {
		systemFrame = new Frame();
		systemWindow = new Window(systemFrame);
		systemWindow.pack();
		systemWindow.setVisible(true);
		systemWindow.setVisible(false);
	}
}
